#############################################################################
# preview.tcl Copyright 2004, 2005, 2006, 2007 r0lZ
# preview functions (for Windows): frontend for PgcEditPreview.exe
#
# This file is part of PgcEdit.
#
# PgcEdit is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# PgcEdit is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#############################################################################

namespace eval ::preview {

proc preview_current_pgc_2 {} {
	if {!$::config(use_preview)} return
	if {[info exists ::dvddir]} {
		preview2 [lindex $::editor_cmds_from 0] [lindex $::editor_cmds_from 1] 0 false false 0
	} else bell
}

proc preview_pgc {type index} {
	if {!$::config(use_preview)} return
	if {[info exists ::dvddir]} {
		preview2 $type $index 0 false false 0
	} else bell
}

## preview a PGC, or a specific cell in the PGC
## If cell argument == 0, then preview all cells of the PGC, else preview the specified cell
proc preview2 {type index cell tracemode autoquit exittimeout {runinfg false}} {
	if {![info exists ::dvddir]} {bell; return}
	if {!$::config(use_preview)} return
	set indexslist [split $index ,]
	if {$type == "pgc"} {
		set vts [lindex $indexslist 0]
		if {$vts == 0} {bell; return}
		set lu  0
		set pgc [lindex $indexslist 1]
		set numcells $::pgcs($vts,$pgc,numcells)
		set cellpbtable $::pgcs($vts,$pgc,cellpbtable)
		set label [::main::pgc_label $::config(fixed_width_font) $type $vts $pgc]
	} else {
		set vts [lindex $indexslist 0]
		set lu  [lindex $indexslist 1]
		set pgc [lindex $indexslist 2]
		set numcells $::menus($vts,$lu,$pgc,numcells)
		set cellpbtable $::menus($vts,$lu,$pgc,cellpbtable)
		set label [::main::pgc_label $::config(fixed_width_font) $type $vts $lu $pgc]
	}
	if {$numcells == 0} {bell; return}
	set newlabel [string map {"  " " "} $label]
	while {$newlabel != $label} {
		set label $newlabel
		set newlabel [string map {"  " " "} $label]
	}
	set label [string map {" ," ","} $label]
	if {$vts == 0} {
		set vobfile  [file nativename [file join $::dvddir "VIDEO_TS.VOB"]]
		set dar [expr {([get_int ::raw_sectors(0,VMGM_MAT) 0x100 2] & 0x0C00)}]
		set res [expr {([get_int ::raw_sectors(0,VMGM_MAT) 0x100 2] & 0x0018) / 8}]
	} else {
		if {$type == "menu"} {
			set vobfile [file nativename [file join $::dvddir "VTS_[format %02d $vts]_0.VOB"]]
			set dar [expr {([get_int ::raw_sectors($vts,VTSI_MAT) 0x100 2] & 0x0C00)}]
			set res [expr {([get_int ::raw_sectors($vts,VTSI_MAT) 0x100 2] & 0x0018) / 8}]
		} else {
			set vobfile [file nativename [file join $::dvddir "VTS_[format %02d $vts]_1.VOB"]]
			set dar [expr {([get_int ::raw_sectors($vts,VTSI_MAT) 0x200 2] & 0x0C00)}]
			set res [expr {([get_int ::raw_sectors($vts,VTSI_MAT) 0x200 2] & 0x0018) / 8}]
		}
	}
	if {! [file exists $vobfile]} {return}
	if {$dar == [expr {(0x0C00)}]} {set dar 16} else {set dar 4}
	if {$res > 1} {set res 2} else {set res 1}
	set lbastxt ""
	if {$cell == 0} {
		for {set c 1} {$c <= $numcells} {incr c} {
			set cellidx [expr {($c - 1) * 24}]
			set anglemode [expr {[get_int cellpbtable $cellidx 1] & 0xF0}]
			set lbastxt "$lbastxt$c=[get_int cellpbtable [expr {$cellidx + 8}] 4] "
			switch $anglemode {
				80 - 144 {	;# first and middle angle: pass entry_VOBU_sector and first_ILVU_end
					set lbastxt "$lbastxt[get_int cellpbtable [expr {$cellidx +12}] 4]\n"
				}
				default {	;# normal and last_angle of block: pass entry_VOBU_sector and last_VOBU_end
					set lbastxt "$lbastxt[get_int cellpbtable [expr {$cellidx +20}] 4]\n"
				}
			}
		}
		if {$numcells == 1} {set cell 1}
		set blankstring "Kill PGC PB"
	} else {
		set numcells 1
		set cellidx [expr {($cell - 1) * 24}]
		set lbastxt "1=[get_int cellpbtable [expr {$cellidx + 8}] 4] "
		set lbastxt "$lbastxt[get_int cellpbtable [expr {$cellidx +20}] 4] "
		set thislba [get_int cellpbtable [expr {$cellidx + 8}] 4]
#		set blankstring "Kill Cell PB"
	}
	if {$lbastxt == ""} {bell; return}
	set txt "\[PgcEditPreview\]\n"
	set txt "${txt}VOBFileName = $vobfile\n"
	set txt "${txt}NumCells = $numcells\n"
	set txt "${txt}DAR = $dar\n"
	set txt "${txt}Scale = [expr {round(100 / $::config(preview_screen_divisor)) * $res}]\n"
	set txt "${txt}PGCTitle = $label\n"
	set txt "${txt}DefaultFileName = [file nativename [file join $::config(config_directory) "preview.cfg.ini"]]\n"
	set txt "${txt}CellNumber = $cell\n"
	set txt "${txt}StartPlay = $::config(preview_autoplay)\n"	;# TODO: don't autoplay when still frames...
	set txt "${txt}AutoLocate = $::config(preview_autolocate)\n"
	set txt "${txt}NoOverlay = $::config(preview_no_overlay)\n"
	set txt "${txt}AutoQuit = $autoquit\n"
	set txt "${txt}SleepOnSingleFrame = 3000\n"
	set txt "${txt}MinDisplayTime = 2000\n"
	set txt "${txt}ThisID = [pid]\n"
	if {$tracemode} {
		if {$::tr::wait_step == 2} {
			set txt "${txt}ExitStopButton = 1\n"
		}
		if {$exittimeout > 0} {
			set txt "${txt}ExitTimeOut = [expr {$exittimeout * 1000}]\n"
		}
#		if {[::blankcell::NumButtonsInCell $type $index $cell] <= 0} {
			set blanklist [::blankcell::LastVOBCellIsBlankCell $type $vts]
			if {[lindex $blanklist 0] == 0} {
				set blankcelllba -1
			} else {
				set blankpbentry [lindex $blanklist 2]
				set blankcelllba [get_int blankpbentry 8 4]
			}
			if {$thislba == $blankcelllba} {
				set txt "${txt}BlankString = Cell blanked\n"
				set txt "${txt}BlankIsChecked = 1\n"
				set alreadyblanked true
			} else {
				set txt "${txt}BlankString = Blank this Cell\n"
				set txt "${txt}BlankIsChecked = 0\n"
				set alreadyblanked false
			}
#		}
		set returnfilename [file join $::config(tempdir) "preview_return.tcl"]
		set preview_break false
		set preview_blanking false
	}
	set txt "${txt}\n\[CellStartEnds\]\n"
	set txt "${txt}$lbastxt"
	if {$::config(debug) > 0} {
		puts $txt
	}
	set progname [file nativename [file join $::config(bindir) "PgcEditPreview.exe"]]
	set tempfile [file nativename [file join $::config(tempdir) "preview.args.ini"]]
	catch {file delete $tempfile}
	if {[saveFile $tempfile auto $txt] != 0} {bell; return}
	if {$tracemode} {
		catch {file delete $returnfilename}
		if {$::config(preview_launcher) == ""} {
			set rc [catch {exec -- $progname "$tempfile"}]
		} else {
			set rc [catch {exec -- $::config(preview_launcher) $progname "$tempfile" 2>/dev/null}]
		}
		set ::tr::no_pause true
		if {$rc == 1} {
			set ::tr::wait_step 0
		}
		catch {source $returnfilename}
		catch {file delete $returnfilename}
		if {$preview_break} {
			set ::tr::wait_step 0
		}
		if {$preview_blanking && !$alreadyblanked} {
			::blankcell::blankThisCell $type $index $cell
		}
		update idletasks
	} else {
		if {$runinfg} {
			if {$::config(preview_launcher) == ""} {
				catch {exec -- $progname "$tempfile"}
			} else {
				catch {exec -- $::config(preview_launcher) $progname "$tempfile"}
			}
		} else {
			if {$::config(preview_launcher) == ""} {
				catch {exec -- $progname "$tempfile" &}
			} else {
				catch {exec -- $::config(preview_launcher) $progname "$tempfile" &}
			}
		}
		catch {source $returnfilename}
	}
}

## preview a specific VOB/Cell ID, or a specific cell in the PGC
## If cell argument == 0, then preview all cells of the PGC, else preview the specified cell
## Example: ::preview::preview_vcid pgc 4 2 7
proc preview_vcid {type vts vid cid {autoquit false}} {
	if {!$::config(use_preview)} return
	if {$type == "pgc"} {
		if {$vts == 0} {bell; return}
		set label "VTS $vts, Title VOB(s), VOB ID $vid, Cell ID $cid"
	} else {
		if {$vts == 0} {
			set label VMGM
		} else {
			set label "VTSM $vts"
		}
		set label "$label menu VOB, VOB ID $vid, Cell ID $cid"
	}
	if {$vts == 0} {
		set vobfile  [file nativename [file join $::dvddir "VIDEO_TS.VOB"]]
		set dar [expr {([get_int ::raw_sectors(0,VMGM_MAT) 0x100 2] & 0x0C00)}]
		set res [expr {([get_int ::raw_sectors(0,VMGM_MAT) 0x100 2] & 0x0018) / 8}]
	} else {
		if {$type == "menu"} {
			set vobfile [file nativename [file join $::dvddir "VTS_[format %02d $vts]_0.VOB"]]
			set dar [expr {([get_int ::raw_sectors($vts,VTSI_MAT) 0x100 2] & 0x0C00)}]
			set res [expr {([get_int ::raw_sectors($vts,VTSI_MAT) 0x100 2] & 0x0018) / 8}]
		} else {
			set vobfile [file nativename [file join $::dvddir "VTS_[format %02d $vts]_1.VOB"]]
			set dar [expr {([get_int ::raw_sectors($vts,VTSI_MAT) 0x200 2] & 0x0C00)}]
			set res [expr {([get_int ::raw_sectors($vts,VTSI_MAT) 0x200 2] & 0x0018) / 8}]
		}
	}
	if {! [file exists $vobfile]} {return}
	if {$dar == [expr {(0x0C00)}]} {set dar 16} else {set dar 4}
	if {$res > 1} {set res 2} else {set res 1}
	set lbastxt ""

	set lbastxt ""
	if {$type == "pgc"} {
		for {set pgc 1} {$pgc <= $::pgcs($vts,numpgcs)} {incr pgc} {
			for {set cell 1} {$cell <= $::pgcs($vts,$pgc,numcells)} {incr cell} {
				set cellposidx [expr {($cell-1)*4}]
				if {[get_int ::pgcs($vts,$pgc,cellpostable) $cellposidx 2] == $vid && \
					[get_int ::pgcs($vts,$pgc,cellpostable) [expr $cellposidx+3] 1] == $cid} {
					set cellidx [expr {($cell - 1) * 24}]
					set lbastxt "1=[get_int ::pgcs($vts,$pgc,cellpbtable) [expr {$cellidx + 8}] 4] "
					set lbastxt "$lbastxt[get_int ::pgcs($vts,$pgc,cellpbtable) [expr {$cellidx +20}] 4]"
					break
				}
			}
		}
	} else {
		for {set lu 1} {$lu <= $::menus($vts,numlus)} {incr lu} {
			for {set pgc 1} {$pgc <= $::menus($vts,$lu,numpgcs)} {incr pgc} {
				for {set cell 1} {$cell <= $::menus($vts,$lu,$pgc,numcells)} {incr cell} {
					set cellposidx [expr {($cell-1)*4}]
					if {[get_int ::menus($vts,$lu,$pgc,cellpostable) $cellposidx 2] == $vid && \
						[get_int ::menus($vts,$lu,$pgc,cellpostable) [expr $cellposidx+3] 1] == $cid} {
						set cellidx [expr {($cell - 1) * 24}]
						set lbastxt "1=[get_int ::menus($vts,$lu,$pgc,cellpbtable) [expr {$cellidx + 8}] 4] "
						set lbastxt "$lbastxt[get_int ::menus($vts,$lu,$pgc,cellpbtable) [expr {$cellidx +20}] 4]"
						break
					}
				}
			}
		}
	}
	if {$lbastxt == ""} {bell; return}

	set txt "\[PgcEditPreview\]\n"
	set txt "${txt}VOBFileName = $vobfile\n"
	set txt "${txt}NumCells = 1\n"
	set txt "${txt}DAR = $dar\n"
	set txt "${txt}Scale = [expr {round(100 / $::config(preview_screen_divisor)) * $res}]\n"
	set txt "${txt}PGCTitle = $label\n"
	set txt "${txt}DefaultFileName = [file nativename [file join $::config(config_directory) "preview.cfg.ini"]]\n"
	set txt "${txt}CellNumber = 1\n"
	set txt "${txt}StartPlay = $::config(preview_autoplay)\n"	;# TODO: don't autoplay when still frames...
	set txt "${txt}AutoLocate = $::config(preview_autolocate)\n"
	set txt "${txt}NoOverlay = $::config(preview_no_overlay)\n"
	set txt "${txt}AutoQuit = $autoquit\n"
	set txt "${txt}SleepOnSingleFrame = 3000\n"
	set txt "${txt}ThisID = [pid]\n"
	set txt "${txt}\n\[CellStartEnds\]\n"
	set txt "${txt}$lbastxt"
	if {$::config(debug) > 0} {
		puts $txt
	}
	set progname [file nativename [file join $::config(bindir) "PgcEditPreview.exe"]]
	set tempfile [file nativename [file join $::config(tempdir) "preview.args.ini"]]
	catch {file delete $tempfile}
	if {[saveFile $tempfile auto $txt] != 0} {bell; return}
	if {$::config(preview_launcher) == ""} {
		catch {exec -- $progname "$tempfile" &}
	} else {
		catch {exec -- $::config(preview_launcher) $progname "$tempfile" &}
	}
}

## preview a whole domain VOBs
proc preview_current_domain {} {
	if {![info exists ::dvddir]} {bell; return}
	if {!$::config(use_preview)} return
	set type [lindex $::editor_cmds_from 0]
	if {$type=="pgcs"} {
		set type pgc
	}
	if {$type=="menus"} {
		set type menu
	}
	preview_domain $type [lindex [split [lindex $::editor_cmds_from 1] ,] 0]
}
proc preview_domain {type vts} {
	if {![info exists ::dvddir]} {bell; return}
	if {!$::config(use_preview)} return

	if {$vts == 0} {
		if {$type == "pgc"} {bell; return}
		set vobfile  [file nativename [file join $::dvddir "VIDEO_TS.VOB"]]
		set dar [expr {([get_int ::raw_sectors(0,VMGM_MAT) 0x100 2] & 0x0C00)}]
		set res [expr {([get_int ::raw_sectors(0,VMGM_MAT) 0x100 2] & 0x0018) / 8}]
		set tablename "VMGM_C_ADT"
		set label "VMGM menu domain (VIDEO_TS.VOB)"
	} else {
		if {$type == "menu"} {
			set vobfile [file nativename [file join $::dvddir "VTS_[format %02d $vts]_0.VOB"]]
			set dar [expr {([get_int ::raw_sectors($vts,VTSI_MAT) 0x100 2] & 0x0C00)}]
			set res [expr {([get_int ::raw_sectors($vts,VTSI_MAT) 0x100 2] & 0x0018) / 8}]
			set tablename "VTSM_C_ADT"
			set label "VTSM $vts menu domain (VTS_[format %02d $vts]_0.VOB)"
		} else {
			set vobfile [file nativename [file join $::dvddir "VTS_[format %02d $vts]_1.VOB"]]
			set dar [expr {([get_int ::raw_sectors($vts,VTSI_MAT) 0x200 2] & 0x0C00)}]
			set res [expr {([get_int ::raw_sectors($vts,VTSI_MAT) 0x200 2] & 0x0018) / 8}]
			set tablename "VTS_C_ADT"
			set label "VTST $vts title domain (VTS_[format %02d $vts]_1.VOB...)"
		}
	}
	if {! [file exists $vobfile]} {return}
	if {$dar == [expr {(0x0C00)}]} {set dar 16} else {set dar 4}
	if {$res > 1} {set res 2} else {set res 1}

	set txt "\[PgcEditPreview\]\n"
	set txt "${txt}VOBFileName = $vobfile\n"
	set txt "${txt}NumCells = 0\n"
	set txt "${txt}DAR = $dar\n"
	set txt "${txt}Scale = [expr {round(100 / $::config(preview_screen_divisor)) * $res}]\n"
	set txt "${txt}PGCTitle = $label\n"
	set txt "${txt}DefaultFileName = [file nativename [file join $::config(config_directory) "preview.cfg.ini"]]\n"
	set txt "${txt}CellNumber = 0\n"
	set txt "${txt}StartPlay = $::config(preview_autoplay)\n"	;# TODO: don't autoplay when still frames...
	set txt "${txt}AutoLocate = $::config(preview_autolocate)\n"
	set txt "${txt}NoOverlay = $::config(preview_no_overlay)\n"
	set txt "${txt}AutoQuit = 0\n"
	set txt "${txt}SleepOnSingleFrame = 1000\n"
	set txt "${txt}ThisID = [pid]\n"
	if {$::config(debug) > 0} {
		puts $txt
	}
	set progname [file nativename [file join $::config(bindir) "PgcEditPreview.exe"]]
	set tempfile [file nativename [file join $::config(tempdir) "preview.args.ini"]]
	catch {file delete $tempfile}
	if {[saveFile $tempfile auto $txt] != 0} {bell; return}

	if {$::config(preview_launcher) == ""} {
		catch {exec -- $progname "$tempfile" &}
	} else {
		catch {exec -- $::config(preview_launcher) $progname "$tempfile" 2>/dev/null &}
	}
}

proc close_preview {} {
	if {!$::config(use_preview)} return
	set txt "\[PgcEditPreview\]\n"
	set txt "${txt}ThisID = [pid]\n"
	set txt "${txt}KillSelf = 1\n"

	set progname [file nativename [file join $::config(bindir) "PgcEditPreview.exe"]]
	set tempfile [file nativename [file join $::config(tempdir) "preview.args.ini"]]
	catch {file delete $tempfile}
	catch {saveFile $tempfile auto $txt}
	if {$::config(preview_launcher) == ""} {
		catch {exec -- $progname "$tempfile" &}
	} else {
		catch {exec -- $::config(preview_launcher) $progname "$tempfile" 2>/dev/null &}
	}
}

}	;# end namespace ::preview
