#############################################################################
# pgceditor.tcl Copyright 2004, 2005, 2006, 2007 r0lZ
# The PGC Header editor stuff.
#
# This file is part of PgcEdit.
#
# PgcEdit is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# PgcEdit is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#############################################################################

namespace eval ::pgced {

namespace export edit_pgc dump_pgc

set play {
	#define play_bitmap_width 8
	#define play_bitmap_height 13
	static unsigned char play_bitmap_bits[] = {
	   0x01,
	   0x03,
	   0x07,
	   0x0f,
	   0x1f,
	   0x3f,
	   0x7f,
	   0x3f,
	   0x1f,
	   0x0f,
	   0x07,
	   0x03,
	   0x01};
}
image create bitmap ::preview::bitmapplay -background {} -data $play
unset play


proc BCDtime {bcd showfps} {
	set pbtimetxt           [format  %i [expr { [get_int bcd 0 1] / 16}]]
	set pbtimetxt $pbtimetxt[format  %i [expr { [get_int bcd 0 1] & 15}]]
	set pbtimetxt $pbtimetxt:[format %i [expr { [get_int bcd 1 1] / 16}]]
	set pbtimetxt $pbtimetxt[format  %i [expr { [get_int bcd 1 1] & 15}]]
	set pbtimetxt $pbtimetxt:[format %i [expr { [get_int bcd 2 1] / 16}]]
	set pbtimetxt $pbtimetxt[format  %i [expr { [get_int bcd 2 1] & 15}]]
	set pbtimetxt $pbtimetxt.[format %i [expr {([get_int bcd 3 1]& 0X30)/16}]]
	set pbtimetxt $pbtimetxt[format  %i [expr { [get_int bcd 3 1] & 15}]]
	if {$showfps} {
		set fpsmode [expr {[get_int bcd 3 1] & 0XC0}]
		switch $fpsmode {
			192 {set pbtimetxt "$pbtimetxt (at 30 fps)"}
			64  {set pbtimetxt "$pbtimetxt (at 25 fps)"}
			default {set pbtimetxt "$pbtimetxt (illegal FPS)"}
		}
	}
	return $pbtimetxt
}

proc short_BCDtime {bcd} {
	set t1 [format %i%i [expr { [get_int bcd 0 1] / 16}] [expr { [get_int bcd 0 1] & 15}]]:
	set t2 [format %i%i [expr { [get_int bcd 1 1] / 16}] [expr { [get_int bcd 1 1] & 15}]]:
	set t3 [format %i%i [expr { [get_int bcd 2 1] / 16}] [expr { [get_int bcd 2 1] & 15}]].
	set t4 [format %i%i [expr {([get_int bcd 3 1]& 0X30)/16}] [expr { [get_int bcd 3 1] & 15}]]

	if {$t1 == "00:"} {
		set t1 ""
		if {[string range $t2 0 0] == "0"} {
			set t2 [string range $t2 1 end]
		}
	} else {
		set t1 [string trimleft $t1 "0"]
	}

	return $t1$t2$t3$t4
}


proc AddBCDtime {bcd1 bcd2} {
	set fpsmode [expr {[get_int bcd1 3 1] & 0XC0}]
	switch $fpsmode {
		192 {set fps 30}
		64  {set fps 25}
		default {set fps 0}
	}
	if {$fps == 0} {return "\000\000\000\000"}

	set f1 [expr {((([get_int bcd1 3 1]& 0X30)/16)*10) + ([get_int bcd1 3 1] & 15)}]
	set s1 [expr { (([get_int bcd1 2 1] / 16) * 10) + ([get_int bcd1 2 1] & 15)}]
	set m1 [expr { (([get_int bcd1 1 1] / 16) * 10) + ([get_int bcd1 1 1] & 15)}]
	set h1 [expr { (([get_int bcd1 0 1] / 16) * 10) + ([get_int bcd1 0 1] & 15)}]

	set f2 [expr {((([get_int bcd2 3 1]& 0X30)/16)*10) + ([get_int bcd2 3 1] & 15)}]
	set s2 [expr { (([get_int bcd2 2 1] / 16) * 10) + ([get_int bcd2 2 1] & 15)}]
	set m2 [expr { (([get_int bcd2 1 1] / 16) * 10) + ([get_int bcd2 1 1] & 15)}]
	set h2 [expr { (([get_int bcd2 0 1] / 16) * 10) + ([get_int bcd2 0 1] & 15)}]

	set f [expr {$f1 + $f2}]
	set s [expr {$s1 + $s2}]
	set m [expr {$m1 + $m2}]
	set h [expr {$h1 + $h2}]

	while {$f >= $fps} {
		set f [expr {$f - $fps}]
		incr s
	}
	while {$s >= 60} {
		set s [expr {$s - 60}]
		incr m
	}
	while {$m >= 60} {
		set m [expr {$m - 60}]
		incr h
	}

	set f "[expr {((($f / 10) * 16) + ($f % 10)) | $fpsmode}]"
	set s "[expr {(($s / 10) * 16) + ($s % 10)}]"
	set m "[expr {(($m / 10) * 16) + ($m % 10)}]"
	set h "[expr {(($h / 10) * 16) + ($h % 10)}]"

	return "[format %c%c%c%c $h $m $s $f]"
}

proc frames_to_BCDtime {frames fps} {
	set f [expr $frames % $fps]
	set numsecs [expr $frames / $fps]
	set s [expr $numsecs % 60]
	set nummins [expr $numsecs / 60]
	set m [expr $nummins % 60]
	set h [expr $nummins / 60]
	set f "[expr {(($f / 10) * 16) + ($f % 10)}]"
	set s "[expr {(($s / 10) * 16) + ($s % 10)}]"
	set m "[expr {(($m / 10) * 16) + ($m % 10)}]"
	set h "[expr {(($h / 10) * 16) + ($h % 10)}]"
	if {$fps == 25} {
		incr f 64
	} else {
		incr f 192
	}
	return "[format %c $h][format %c $m][format %c $s][format %c $f]"
}

# dump PGC contents as text
proc dump_pgc {pgctype index pgcheader numprograms programmap numcells cellpbtable cellpostable PTTlist} {

	set vts   [lindex [split $index ","] 0]

	# get editables values from the header's binary data
	set txt " Playback time:  [BCDtime [string range $pgcheader 4 7] true]\n"

	set ::pgced::pbmode    [get_int pgcheader 0XA2 1]
	if {$::pgced::pbmode == 0} {
		set pbmodetxt "sequential"
	} else {
		set ::pgced::pbprogs [expr {$::pgced::pbmode & 0X7F}]
		if {[expr {$::pgced::pbmode & 0X80}] == 0} {
			set pbmodetxt "random: $::pgced::pbprogs programs"
		} else {
			set pbmodetxt "shuffle: $::pgced::pbprogs programs"
		}
	}
	set txt "${txt} PG Playback mode:  $pbmodetxt\n"

	set puos      [get_int pgcheader 8 4]
	set txt "${txt} PUOs:           $puos (0x[format %08X $puos])\n"
	for {set p 0} {$p <= 24} {incr p} {
		set state [expr {$puos & round(pow(2,$p))}]
		if {$state > 0} {
			set txt "${txt}                 - [lindex $::pgced::puos_labels $p] (0x[format %08X [expr {round(pow(2,$p))}]])\n"
		}
	}

	set txt "${txt} NextPGCN:       [get_int pgcheader 0X9C 2]\n"
	set txt "${txt} PrevPGCN:       [get_int pgcheader 0X9E 2]\n"
	set txt "${txt} GoUpPGCN:       [get_int pgcheader 0XA0 2]\n"
	set txt "${txt} PGC Still Time: [get_int pgcheader 0XA3 1]\n"

	for {set a 1} {$a <= 8} {incr a} {
		set audio_status [get_int pgcheader [expr {(0X0A + ($a*2))}] 2]
		if {$audio_status != 0} {
			set txt "${txt} Audio  stream  $a status: [format 0x%08X $audio_status] (stream=[expr {($audio_status & 0x0700)/0x100}])\n"
		}
	}
	for {set s 1} {$s <= 32} {incr s} {
		binary scan [string range $pgcheader [expr {(0X18 + ($s*4))}] [expr {(0X18 + ($s*4) + 3)}]] cccc s1 s2 s3 s4
		if {$s1 != 0} {
			set txt "${txt} Subpic stream [format %2d $s] status: [format 0x%08X [get_int pgcheader [expr {(0X18) + ($s*4)}] 4]] (streams for 4:3=[expr {$s1 & 0X1F}], wide=$s2, letterbox=$s3, pan&scan=$s4)\n"
		}
	}

	set txt "${txt}\n"
	set end_time "\000\000\000\000"
	if {$numcells > 0} {
#		if {$pgctype == "menu"} {set txt "${txt} But- "} else {set txt "${txt} Chap."}

		if {$pgctype == "menu"} {set txt "${txt} But- "} else {set txt "${txt} BOVs Chap. "}
		set	txt "${txt}Prog. Cell  Type  Seam- Ang VOBU  Cell  Cell  Playback     End          Entry    First    Last     Last     VOB Cell\n"
		if {$pgctype == "menu"} {set txt "${txt} tons"} else {set txt "${txt}      (PTT)"}
		set	txt "${txt}             Flags less      Still Still Cmd.  Time         Time         VOBU     ILVU     VOBU     VOBU     ID  ID\n"
		if {$pgctype == "pgc"} {set txt "${txt}      "}
		set	txt "${txt}                        Joint           Time  #                               sector   End      Start    End\n\n"

		for {set c 1} {$c <= $numcells} {incr c} {
			set cell_program($c) ""
			set prog_chapter($c) ""
		}
		for {set p 1} {$p <= $numprograms} {incr p} {
			set cell_program([get_int programmap [expr {$p-1}] 1]) $p
		}
		for {set c 1} {$c <= $numcells} {incr c} {
			set prog_chapter($c) 0
		}
		foreach pttpair $PTTlist {
			set pg  [lindex $pttpair 0]
			set ptt [lindex $pttpair 1]
			for {set c 1} {$c <= $numcells} {incr c} {
				if {$pg == $cell_program($c) && $prog_chapter($c) == 0} {
					set prog_chapter($c) $ptt
				}
			}
		}

		for {set c 1} {$c <= $numcells} {incr c} {
			set cellidx1 [expr {($c - 1) * 24}]
			set cellidx2 [expr {($c - 1) * 4}]

			set vobid  [get_int cellpostable $cellidx2 2]
			set cellid [get_int cellpostable [expr {$cellidx2 + 3}] 1]

			if {$pgctype == "menu"} {
				if {[info exists ::buttons::butsinfo(menu,$vts,$vobid,$cellid,numbuts)]} {
					set txt "$txt [format %-5s $::buttons::butsinfo(menu,$vts,$vobid,$cellid,numbuts)]"
				} else {
					set txt "$txt ?   "
				}
			} else {
				if {[info exists ::buttons::butsinfo(pgc,$vts,$vobid,$cellid,numbuts)]} {
					set txt "$txt [format %-4s $::buttons::butsinfo(pgc,$vts,$vobid,$cellid,numbuts)]"
				} else {
					set txt "$txt ?   "
				}
				if {$prog_chapter($c) == 0} {
					set txt "$txt       "
				} else {
					set txt "$txt [format %-5s $prog_chapter($c)] "
				}
			}
			set txt "$txt[format %-5s $cell_program($c)] "
			set txt "$txt[format %-5d $c] "

			set type [get_int cellpbtable $cellidx1 1]
			set txt "$txt[format %-5d $type] "
			if {[expr {$type & 8}] == 8} {
				set txt "${txt}yes   "
			} else {
				set txt "${txt}no    "
			}

			if {[expr $type & 16]==0} {
				set angle "-"
				set lastangle "-"
			} else {
				switch [expr $type & 192] {
					case 0 {set angle "???"; set lastangle "???"}
					case 64 {set angle 1; set lastangle 1}
					default {
						if {[string is integer $lastangle]} {
							set angle [expr $lastangle+1]
						} else {
							set angle "?"
						}
						set lastangle $angle
						if {[expr $type & 192] == 192} {set lastangle "-"}
					}
				}
			}
			set txt "$txt[format %-3s $angle] "

			set var [expr {[get_int cellpbtable [expr {$cellidx1 + 1}] 1] & 64}]
			if {$var == 0} {
				set txt "${txt}no    "
			} else {
				set txt "${txt}yes   "
			}

			set txt "$txt[format %-5d [get_int cellpbtable [expr {$cellidx1 + 2}] 1]] "

			set txt "$txt[format %-5d [get_int cellpbtable [expr {$cellidx1 + 3}] 1]] "

			set cell_time [string range $cellpbtable [expr {$cellidx1 + 4}] [expr {$cellidx1 + 7}]]
			set txt "$txt[BCDtime $cell_time false]  "

			# Add cell time to total PGC time, but only if angle 1.
			if {[expr {$type & 0x10}] == 0 || [expr {$type & 0xC0}] == 0x40} {
				set end_time [AddBCDtime $cell_time $end_time]
			}
			set txt "$txt[BCDtime $end_time false]  "

			set txt "$txt[format %-8d [get_int cellpbtable [expr {$cellidx1 +  8}]  4]] "
			set txt "$txt[format %-8d [get_int cellpbtable [expr {$cellidx1 +  12}] 4]] "
			set txt "$txt[format %-8d [get_int cellpbtable [expr {$cellidx1 +  16}] 4]] "
			set txt "$txt[format %-8d [get_int cellpbtable [expr {$cellidx1 +  20}] 4]] "

			set txt "$txt[format %-3d $vobid] "
			set txt "$txt$cellid\n"
		}
	}
	return $txt
}
proc dump_pgc_top {type vts args} {
	if {$type == "menu"} {
		set lu  [lindex $args 0]
		set pgc [lindex $args 1]
		set index "$vts,$lu,$pgc"
		set label [::main::pgc_label "" menu $vts $lu $pgc]
	} else {
		set pgc [lindex $args 0]
		set index "$vts,$pgc"
		set label [::main::pgc_label "" pgc $vts $pgc]
	}
	set pgcheader    [subst \$::${type}s($index,header)]
	set numprograms  [subst \$::${type}s($index,numprograms)]
	set programmap   [subst \$::${type}s($index,programmap)]
	set numcells     [subst \$::${type}s($index,numcells)]
	set cellpbtable  [subst \$::${type}s($index,cellpbtable)]
	set cellpostable [subst \$::${type}s($index,cellpostable)]
	if {$type == "pgc" && [info exists ::${type}s($index,pttlist)]} {
		set pttlist [subst \$::${type}s($index,pttlist)]
	} else {
		set pttlist ""
	}
	set vtsnum [lindex [split $index ","] 0]

	return " ************ $label ************\n\n[::pgced::dump_pgc $type \
			$index $pgcheader $numprograms $programmap \
			$numcells $cellpbtable $cellpostable $pttlist]"
}

proc dump_all_pgcs {} {
	set totaldvdtime "\000\000\000\000"
	if {! [info exists ::dvddir] || $::scratchbook_active} {
		bell
		return
	}
	set progress_max [expr $::pgcs(numvts)+1]
	show_msg "Dumping VMG..." 1 $progress_max

	set out "DVD: \"$::dvddir\"\n"
	set out "${out}DVD-TEXT General Name: \"[::utils::get_General_Name]\"\n"
	set out "${out}Provider ID: \"[::utils::get_provider_ID]\"\n"
	set out "${out}Number of VTS: $::pgcs(numvts)\n\n"

	# VIDEO_TS.IFO
	set out "${out}VMG (VIDEO_TS.IFO)\n"
	set out "${out}------------------\n\n"
	set out "${out}[dump_pgc_top pgc 0 0]\n\n"

	if {$::menus(0,numlus)==0} {
		set out "${out}VMGM: no menu.\n\n"
	} else {
		set totalbcdtime "\000\000\000\000"
		for {set lu 1} {$lu <= $::menus(0,numlus)} {incr lu} {
			set out "${out}VMGM , LU $lu ($::menus(0,$lu,lang)):\n\n"
			for {set pgc 1} {$pgc <= $::menus(0,$lu,numpgcs)} {incr pgc} {
				set out "$out[dump_pgc_top menu 0 $lu $pgc]\n\n"
				set totalbcdtime [AddBCDtime [string range $::menus(0,$lu,$pgc,header) 4 7] $totalbcdtime]
				set totaldvdtime [AddBCDtime [string range $::menus(0,$lu,$pgc,header) 4 7] $totaldvdtime]
			}
		}
		set out "${out} VMGM total playing time: [BCDtime $totalbcdtime true]\n\n\n"
	}

	# VTS_xx_0.IFO
	for {set vts 1} {$vts <= $::pgcs(numvts)} {incr vts} {
		show_msg "Dumping VTS $vts..." [expr $vts+1] $progress_max
		set out "${out}VTS $vts (VTS_[format %02i $vts]_0.IFO)\n"
		set out "${out}--------------------\n\n"

		if {$::menus($vts,numlus)==0} {
			set out "${out}VTSM $vts: no menu.\n\n"
		} else {
			set totalbcdtime "\000\000\000\000"
			for {set lu 1} {$lu <= $::menus($vts,numlus)} {incr lu} {
				set out "${out}VTSM $vts , LU $lu ($::menus($vts,$lu,lang)):\n\n"
				for {set pgc 1} {$pgc <= $::menus($vts,$lu,numpgcs)} {incr pgc} {
					set out "$out[dump_pgc_top menu $vts $lu $pgc]\n\n"
					set totalbcdtime [AddBCDtime [string range $::menus($vts,$lu,$pgc,header) 4 7] $totalbcdtime]
					set totaldvdtime [AddBCDtime [string range $::menus($vts,$lu,$pgc,header) 4 7] $totaldvdtime]
				}
			}
			set out "${out} VTSM $vts total playing time: [BCDtime $totalbcdtime true]\n\n\n"
		}

		set out "${out}VTST $vts:\n\n"
		set totalbcdtime "\000\000\000\000"
		for {set pgc 1} {$pgc <= $::pgcs($vts,numpgcs)} {incr pgc} {
			set out "$out[dump_pgc_top pgc $vts $pgc]\n\n"
			set totalbcdtime [AddBCDtime [string range $::pgcs($vts,$pgc,header) 4 7] $totalbcdtime]
			set totaldvdtime [AddBCDtime [string range $::pgcs($vts,$pgc,header) 4 7] $totaldvdtime]
		}
		set out "${out} VTST $vts total playing time: [BCDtime $totalbcdtime true]\n\n\n"
	}
	set out "${out}DVD total playing time: [BCDtime $totaldvdtime true]\n"
	::main::showtext .dumpallpgcs "DVD PGCs Info" [string trimright $out \n]\n
	show_msg ""
}


#### Main entry for PgcEditor
proc edit_pgc {pgctype index pgcheader numprograms programmap numcells cellpbtable cellpostable PTTlist usercomment {default_field ""}} {
	set w ".pgceditor"
	if {[winfo exists $w]} {return ""}

	show_msg "Building PGC editor window..."

	set ::pgced::__pgcheader    $pgcheader
	set ::pgced::__programmap   $programmap
	set ::pgced::__numcells     $numcells
	set ::pgced::__numprograms  $numprograms
	set ::pgced::__cellpbtable  $cellpbtable
	set ::pgced::__cellpostable $cellpostable
	set ::pgced::__PTTlist      $PTTlist
	set ::pgced::__pgctype		$pgctype
	set ::pgced::__index		$index
	set ::pgced::__need_new_tmap false
	set ::pgced::__usercomment	$usercomment

	set type [lindex $::editor_cmds_from 0]
	set vts [lindex [split $::pgced::__index ","] 0]

	# get editables values from the header's binary data

	set ::pgced::__pgcpbtime [string range $::pgced::__pgcheader 4 7]
	set ::pgced::pbtimetxt [BCDtime $::pgced::__pgcpbtime true]

	set ::pgced::pbmode    [get_int ::pgced::__pgcheader 0XA2 1]
	if {$::pgced::pbmode > 0} {
		set ::pgced::pbprogs [expr {$::pgced::pbmode & 0X7F}]
		if {[expr {$::pgced::pbmode & 0X80}] == 0} {
			set ::pgced::pbmode 1	;# random
		} else {
			set ::pgced::pbmode 2	;# shuffle
		}
	} else {
		set ::pgced::pbprogs 2
	}

	set ::pgced::puos      [get_int ::pgced::__pgcheader 8 4]
	set ::pgced::nextpgcn  [get_int ::pgced::__pgcheader 0X9C 2]
	set ::pgced::prevpgcn  [get_int ::pgced::__pgcheader 0X9E 2]

	set tmp [get_int ::pgced::__pgcheader 0xA0 2]

	if {$type == "menu"} {
		if {$tmp == 65535} {
			set ::pgced::goup_type 0
			set ::pgced::gouppgcn 0
		} else {
			set ::pgced::goup_type 1
			set ::pgced::gouppgcn $tmp
		}
	} else {
		set ::pgced::goup_type 1
		set ::pgced::gouppgcn $tmp
	}
	set ::pgced::stilltime [get_int ::pgced::__pgcheader 0XA3 1]

	set cmds [subst \$::[lindex $::editor_cmds_from 0]s([lindex $::editor_cmds_from 1])]
	set ::pgced::cellcmds [get_type_cmds cell cmds]
	unset cmds

	# build interface
	toplevel $w -class Dialog
	wm withdraw $w
	catch {grab set $w}
	wm protocol $w WM_DELETE_WINDOW {set ::pgced::done 0}
	if {$::config(tk) != "aqua"} {wm transient $w .}
	if {$::tcl_platform(platform) == "windows"} {catch {wm attributes $w -toolwindow 1}}
	bind $w <Escape> {set ::pgced::done 0}
	bind $w <Return> {set ::pgced::done 1}

	frame $w.f1
	pack  $w.f1 -pady 2 -anchor w -fill x

	# misc PGC values
	frame $w.f1.fl
	pack  $w.f1.fl -fill x -expand 1 -padx 2 -pady 1 -side left
	frame $w.f1.fl.mf
	pack  $w.f1.fl.mf -padx 2 -fill x -expand 1
	label $w.f1.fl.mf.l0 -text "Playback Time "
	entry $w.f1.fl.mf.e0 -textvariable ::pgced::pbtimetxt -state readonly -justify center

	label $w.f1.fl.mf.l1 -text "PG Playback Mode "

	frame $w.f1.fl.mf.f1 -bd 2 -relief groove
	radiobutton $w.f1.fl.mf.f1.r1 -pady 0 -bd 1 -variable ::pgced::pbmode -value 0 \
			-text "Sequential" -command "$w.f1.fl.mf.f1.f2.e configure -state disabled; $w.f1.fl.mf.e6 configure -state disabled"
	frame $w.f1.fl.mf.f1.f1
	radiobutton $w.f1.fl.mf.f1.f1.r2 -pady 0 -bd 1 -variable ::pgced::pbmode -value 1 \
			-text "Random:" -command "$w.f1.fl.mf.f1.f2.e configure -state normal; focus $w.f1.fl.mf.f1.f2.e; $w.f1.fl.mf.e6 configure -state normal"
	radiobutton $w.f1.fl.mf.f1.f1.r3 -pady 0 -bd 1 -variable ::pgced::pbmode -value 2 \
			-text "Shuffle:"  -command "$w.f1.fl.mf.f1.f2.e configure -state normal; focus $w.f1.fl.mf.f1.f2.e; $w.f1.fl.mf.e6 configure -state normal"
	pack  $w.f1.fl.mf.f1.r1 -pady 0 -anchor w
	pack  $w.f1.fl.mf.f1.f1 -side left
	pack  $w.f1.fl.mf.f1.f1.r2 -pady 0 -anchor w
	pack  $w.f1.fl.mf.f1.f1.r3 -pady 0 -anchor w
	frame $w.f1.fl.mf.f1.f2
	pack  $w.f1.fl.mf.f1.f2 -side left
	entry $w.f1.fl.mf.f1.f2.e -width 4 -textvariable ::pgced::pbprogs -justify center \
			-validate key -validatecommand "::pgced::value_entry_validate 2 127 {%d} {%P}"
	pack  $w.f1.fl.mf.f1.f2.e -side left
	::balloon::set_balloon $w.f1.fl.mf.f1.f2.e "Number of programs to play - 1\n(min 1, max 127)"
	label $w.f1.fl.mf.f1.f2.l -text "programs"
	pack  $w.f1.fl.mf.f1.f2.l -side left

	label $w.f1.fl.mf.l2 -text "Prohibited User Operations "
	entry $w.f1.fl.mf.e2 -textvariable ::pgced::puos -justify center \
			-validate key -validatecommand "::pgced::value_entry_validate 0 0x1FFFFFF {%d} {%P}"
	button $w.f1.fl.mf.b2 -text "..." -pady 0 -width 2 \
			-command "::pgced::puos_editor {Prohibited User Operations} $w"
	bind   $w.f1.fl.mf.b2 <3> {set ::pgced::puos 0}

	label $w.f1.fl.mf.l3 -text "Previous PGCN link "
	entry $w.f1.fl.mf.e3 -textvariable ::pgced::prevpgcn  -justify center \
			-validate key -validatecommand "::pgced::validate_PGCNLink_value $w.f1.fl.mf.e3 {%d} {%P}"
	label $w.f1.fl.mf.l4 -text "Next PGCN link "
	entry $w.f1.fl.mf.e4 -textvariable ::pgced::nextpgcn  -justify center \
			-validate key -validatecommand "::pgced::validate_PGCNLink_value $w.f1.fl.mf.e4 {%d} {%P}"
	label $w.f1.fl.mf.l5 -text "Go Up (Return) link "
	frame $w.f1.fl.mf.e5 -bd 2 -relief groove
	frame $w.f1.fl.mf.e5.f1
	pack  $w.f1.fl.mf.e5.f1 -side top -fill x -expand 1 -pady 0
	radiobutton $w.f1.fl.mf.e5.f1.c -text "PGCN" -variable ::pgced::goup_type -value 1 \
			-command "$w.f1.fl.mf.e5.f1.e configure -state normal; focus $w.f1.fl.mf.e5.f1.e" -padx 0 -pady 0 -bd 1
	pack  $w.f1.fl.mf.e5.f1.c -side left -padx 0
	entry $w.f1.fl.mf.e5.f1.e -textvariable ::pgced::gouppgcn -justify center -width 3 \
			-validate key -validatecommand "::pgced::validate_PGCNLink_value $w.f1.fl.mf.e5.f1.e {%d} {%P}"
	pack  $w.f1.fl.mf.e5.f1.e -side left -fill x -expand 1 -padx 2
	radiobutton $w.f1.fl.mf.e5.c -text "Resume (menus only)" -variable ::pgced::goup_type -value 0 \
			-command "$w.f1.fl.mf.e5.f1.e configure -state disabled" -padx 0 -pady 0 -bd 1
	pack  $w.f1.fl.mf.e5.c -side left -pady 0
	if {$type != "menu"} {
		$w.f1.fl.mf.e5.c configure -state disabled
	}
	if {! $::pgced::goup_type} {
		$w.f1.fl.mf.e5.f1.e configure -state disabled
	}

	label $w.f1.fl.mf.l6 -text "PGC Still Time (255=infinite) "
	entry $w.f1.fl.mf.e6 -textvariable ::pgced::stilltime  -justify center \
			-validate key -validatecommand "::pgced::value_entry_validate 0 0XFF {%d} {%P}"
	::balloon::set_balloon $w.f1.fl.mf.e6 "0 to 254 seconds\nor 255 = infinite"

	grid  $w.f1.fl.mf.l0 -row 0 -column 0 -sticky e ;#-ipady 1
	grid  $w.f1.fl.mf.e0 -row 0 -column 1 -sticky ew -columnspan 2
	grid  $w.f1.fl.mf.l1 -row 1 -column 0 -sticky e -ipady 1
	grid  $w.f1.fl.mf.f1 -row 1 -column 1 -sticky ew -columnspan 2
	grid  $w.f1.fl.mf.l2 -row 2 -column 0 -sticky e ;#-ipady 1
	grid  $w.f1.fl.mf.e2 -row 2 -column 1 -sticky ew
	grid  $w.f1.fl.mf.b2 -row 2 -column 2 -sticky ew
	grid  $w.f1.fl.mf.l3 -row 3 -column 0 -sticky e -ipady 1
	grid  $w.f1.fl.mf.e3 -row 3 -column 1 -sticky ew -columnspan 2
	grid  $w.f1.fl.mf.l4 -row 4 -column 0 -sticky e ;#-ipady 1
	grid  $w.f1.fl.mf.e4 -row 4 -column 1 -sticky ew -columnspan 2
	grid  $w.f1.fl.mf.l5 -row 5 -column 0 -sticky e ;#-ipady 1
	grid  $w.f1.fl.mf.e5 -row 5 -column 1 -sticky ew -columnspan 2 ;#-pady 2
	grid  $w.f1.fl.mf.l6 -row 6 -column 0 -sticky e ;#-ipady 1
	grid  $w.f1.fl.mf.e6 -row 6 -column 1 -sticky ew -columnspan 2
	grid columnconfigure $w.f1.fl.mf 1 -weight 100

	# audio streams control
	frame $w.f1.mfa -borderwidth 2 -relief groove
	pack  $w.f1.mfa -padx 2 -ipadx 4 -ipady 2 -side left -anchor n
	label $w.f1.mfa.tit -text "Audio VOB's\nStreams" -pady 0
	grid  $w.f1.mfa.tit -row 0 -column 0 -columnspan 2
	for {set a 1} {$a <= 8} {incr a} {
		label $w.f1.mfa.al$a -text "[expr {$a-1}]"
		button $w.f1.mfa.ab$a -textvariable ::pgced::audiostream$a -width 9 -pady 0 \
				-command "::pgced::set_audio ::pgced::audiostream$a $a"
		if {$::config(tk) == "aqua"} {$w.f1.mfa.ab$a configure -width 3}
		grid  $w.f1.mfa.al$a -row $a -column 0 -sticky e
		grid  $w.f1.mfa.ab$a -row $a -column 1 -sticky ew
		if {$::pgced::__pgctype == "menu"} break
	}
	bind  $w.f1.mfa     $::config(popups_button) "tk_popup $w.f1.mfa.menu \[winfo pointerx .\] \[winfo pointery .\] 0"
	bind  $w.f1.mfa.tit $::config(popups_button) "tk_popup $w.f1.mfa.menu \[winfo pointerx .\] \[winfo pointery .\] 0"
	if {! [winfo exists $w.f1.mfa.menu]} {
		menu $w.f1.mfa.menu -tearoff 0
		$w.f1.mfa.menu add command -label "Copy all"  -command "::pgced::copy_all_audio"
		$w.f1.mfa.menu add command -label "Paste all" -command "::pgced::paste_all_audio"
	}

	# sub-picture streams control
	frame $w.f1.mfs -borderwidth 2 -relief groove
	pack  $w.f1.mfs -padx 2 -pady 1 -ipadx 4 -ipady 2 -side left -anchor n
	label $w.f1.mfs.tit -pady 0
	grid  $w.f1.mfs.tit -row 0 -column 0 -columnspan 8
	set numsubpics [::streams::get_pgc_num_subpic $type $index]
	for {set s 1} {$s <= 32} {incr s} {
		if {$s <= $numsubpics} {
			set descr [string map {" Off:" : " On:" :} [::streams::get_pgc_subpic_descr $type $index [expr {$s-1}]]]
		} else {
			set descr "[expr {$s-1}]: not used."
		}
		binary scan [string range $::pgced::__pgcheader [expr {(0X18 + ($s*4))}] [expr {(0X18 + ($s*4) + 3)}]] cccc s1 s2 s3 s4
		if {$s1 == 0} {
			set ::pgced::subpicstream$s "none"
		} else {
			set ::pgced::subpicstream$s [string map {{ } {  }} [format "%2d %2d %2d %2d" [expr {$s1 & 0X1F}] $s2 $s3 $s4]]
		}
		label $w.f1.mfs.al$s -text " [expr {$s-1}]"
		button $w.f1.mfs.ab$s -textvariable ::pgced::subpicstream$s -width 12 -pady 0 \
				-command "::pgced::set_subpic ::pgced::subpicstream$s \"$descr\""
		if {$::config(tk) == "aqua"} {$w.f1.mfs.ab$s configure -width 10}
		set row [expr {(($s-1) % 8) + 1}]
		set col [expr {(($s-1) / 8) * 2}]
		grid  $w.f1.mfs.al$s -row $row -column $col -sticky e
		grid  $w.f1.mfs.ab$s -row $row -column [expr {$col+1}] -sticky ew
		::balloon::set_balloon $w.f1.mfs.ab$s $descr
		if {$::pgced::__pgctype == "menu"} break
	}
	bind  $w.f1.mfs     $::config(popups_button) "tk_popup $w.f1.mfs.menu \[winfo pointerx .\] \[winfo pointery .\] 0"
	bind  $w.f1.mfs.tit $::config(popups_button) "tk_popup $w.f1.mfs.menu \[winfo pointerx .\] \[winfo pointery .\] 0"
	if {! [winfo exists $w.f1.mfs.menu]} {
		menu $w.f1.mfs.menu -tearoff 0
		$w.f1.mfs.menu add command -label "Copy all"  -command "::pgced::copy_all_subpic"
		$w.f1.mfs.menu add command -label "Paste all" -command "::pgced::paste_all_subpic"
	}

	# bottom frame
	frame  $w.f2
	pack   $w.f2 -expand 1 -fill both

	# bottom-left frame
	frame $w.f2.f
	pack  $w.f2.f -side left -fill both

	# color table
	frame $w.f2.f.mfc -borderwidth 2 -relief groove -pady 2
	pack  $w.f2.f.mfc -side top -fill x -ipadx 2 -padx 4 -anchor nw
	label $w.f2.f.mfc.l0 -text "Sub-picture\nColor Lookup Table "
	pack  $w.f2.f.mfc.l0 -fill x -pady 4
	frame $w.f2.f.mfc.f1
	pack  $w.f2.f.mfc.f1
	set col 0
	set row 0
	for {set c 0} {$c < 16} {incr c} {
		if {$::config(tk) == "x11"} {
			button $w.f2.f.mfc.f1.b$c -width 3 -height 2 -padx 0 -pady 0 -highlightthickness 1 \
					-command "::pgced::set_color $c $w.f2.f.mfc.f1.b$c" ;#-text $c
			grid   $w.f2.f.mfc.f1.b$c -row $row -column $col -ipadx 3 -ipady 3 -sticky ns
		} elseif {$::config(tk) == "aqua"} {
			frame  $w.f2.f.mfc.f1.b$c -width 30 -height 30 -bd 2 -relief raised
			bind $w.f2.f.mfc.f1.b$c <1> "::pgced::set_color $c $w.f2.f.mfc.f1.b$c"
			grid   $w.f2.f.mfc.f1.b$c -row $row -column $col -ipadx 3 -ipady 3 -sticky ns
		} else {
			button $w.f2.f.mfc.f1.b$c -width 2 -height 1 -padx 1 -pady 0 -highlightthickness 1 \
					-command "::pgced::set_color $c $w.f2.f.mfc.f1.b$c" ;#-text $c
			grid   $w.f2.f.mfc.f1.b$c -row $row -column $col -ipadx 3 -ipady 3 -padx 1 -pady 1 -sticky ns
		}
		::balloon::set_balloon $w.f2.f.mfc.f1.b$c $c
		incr col
		if {$col == 4} {
			set col 0
			grid rowconfigure $w.f2.f.mfc.f1 $row -weight 1
			incr row
		}
	}
	frame $w.f2.f.mfc.f2
	pack  $w.f2.f.mfc.f2 -pady 2
	button $w.f2.f.mfc.f2.bc -text "Copy all"  -width 8 -command ::pgced::copy_colors
	button $w.f2.f.mfc.f2.bp -text "Paste all" -width 8 -command ::pgced::paste_colors
	set txt "You may copy from one DVD to another\none by launching PgcEdit twice.\nYou must do the paste before closing the\nsource PgcEdit!"
	::balloon::set_balloon $w.f2.f.mfc.f2.bc $txt
	::balloon::set_balloon $w.f2.f.mfc.f2.bp $txt
	pack   $w.f2.f.mfc.f2.bc -side left -padx 2
	pack   $w.f2.f.mfc.f2.bp -side right -padx 2

	frame $w.f2.f.mfc.f3
	pack  $w.f2.f.mfc.f3 -pady 2
	button $w.f2.f.mfc.f3.bc -text "Export" -width 8 -command ::pgced::export_colors
	button $w.f2.f.mfc.f3.bp -text "Import" -width 8 -command ::pgced::import_colors
	set txt " the raw YcRcB color table as 64 binary bytes."
	::balloon::set_balloon $w.f2.f.mfc.f3.bc "Export$txt"
	::balloon::set_balloon $w.f2.f.mfc.f3.bp "Import$txt"
	pack   $w.f2.f.mfc.f3.bc -side left -padx 2
	pack   $w.f2.f.mfc.f3.bp -side right -padx 2

	# User comment
	frame $w.f2.f.pad -height 4
	pack  $w.f2.f.pad
	frame $w.f2.f.uc -borderwidth 2 -relief groove -pady 2
	pack  $w.f2.f.uc -side bottom -fill x -ipadx 2 -padx 4 -anchor nw
	label $w.f2.f.uc.l -text "User's PGC comment"
	pack  $w.f2.f.uc.l
	entry $w.f2.f.uc.e -textvariable ::pgced::__usercomment -width 16 \
			-validate all -validatecommand "::utils::validate_gprm_label {%d} {%P}"
	pack  $w.f2.f.uc.e -fill x -padx 2 -expand 1

	### Cells table
	frame  $w.f2.fc -borderwidth 2 -relief groove
	pack   $w.f2.fc -side left -padx 2 -expand 1 -fill both -anchor w
	canvas $w.f2.fc.c -yscrollcommand "$w.f2.fc.vs set" -height 8 \
			-highlightcolor [$w.f2.fc cget -background]
	pack   $w.f2.fc.c -side left -expand 1 -fill both
	scrollbar $w.f2.fc.vs -command "$w.f2.fc.c yview" -orient vert
	pack   $w.f2.fc.vs -side left -fill y
	frame  $w.f2.fc.c.f	;# destroyed later

	### buttons
	frame  $w.fb
	pack   $w.fb -padx 4 -pady 4 -fill x
	if {! ($type == "pgc" && $vts == 0)} {
		button $w.fb.ss -text " Current domain streams attributes " \
				-command "::streams::set_streams_attribs $w true; ::pgced::validate_streams_display"
		pack   $w.fb.ss -side left -padx 2
	}
	if {$::pgced::__numcells > 1} {
		# new!
		button $w.fb.rc -text " Remove Cells " \
				-command "::pgced::remove_cell $w"
		pack   $w.fb.rc -side left -padx 2
	}
	if {! ($type == "pgc" && $vts == 0)} {
		button $w.fb.cc -text " Create new cell " -command "::pgced::create_new_cell $w"
		pack  $w.fb.cc -side left -padx 2
	}
	if {$type == "pgc" && $vts != 0} {
		button $w.fb.ang -text " Del Angles " -command "::pgced::remove_angles"
		pack   $w.fb.ang -side left -padx 2
	}
	if {$::pgced::__numcells > 0} {
		button $w.fb.ct -text " CellTimes.txt " \
				-command "::pgced::export_celltimes $w"
		pack   $w.fb.ct -side left -padx 2
	}
	if {$type == "pgc" && $vts != 0} {
		button $w.fb.ptt -text " Edit Chapter (PTT) table " \
				-command "if \[::pgced::ptted::ptt_table_editor $w\] ::pgced::refresh_chapters"
		pack   $w.fb.ptt -side left -padx 2
	}
	frame  $w.fb.pad
	pack   $w.fb.pad -side left -fill x -expand 1

	button $w.fb.ok -text "OK" -width 10 -command {set ::pgced::done 1}
	pack   $w.fb.ok -side left -padx 3
	button $w.fb.cancel -text "Cancel" -width 10 -command {set ::pgced::done 0}
	pack   $w.fb.cancel -side left -padx 2
	if {$::config(platform) == "mac"} {
		pack [frame $w.fb.macpad] -side left -padx 6
	}

	wm title $w "PGC Editor - [$::widget(main_cmd_lab) cget -text]"

	if {$::pgced::pbmode == 0} {
		$w.f1.fl.mf.f1.f2.e configure -state disabled
		$w.f1.fl.mf.e6 configure -state disabled
	} else {
		$w.f1.fl.mf.f1.f2.e configure -state normal
		$w.f1.fl.mf.e6 configure -state normal
	}
	validate_streams_display

	for {set c 0} {$c < 16} {incr c} {
		set color [yc2rgb [string range $::pgced::__pgcheader [expr {(0XA4 + ($c*4))}] [expr {(0XA4 + ($c*4) + 3)}]]]
		$w.f2.f.mfc.f1.b$c configure -background $color
	}
	set ::pgced::__color_table_modified false
	set numaudios [::streams::get_pgc_num_audio $type $index]
	for {set a 1} {$a <= 8} {incr a} {
		if {$a <= $numaudios} {
			set descr [::streams::get_pgc_audio_descr $type $index [expr {$a-1}]]
			::balloon::set_balloon $w.f1.mfa.ab$a $descr
		} else {
			::balloon::set_balloon $w.f1.mfa.ab$a "[expr {$a-1}]: not used."
		}
		binary scan [string range $::pgced::__pgcheader [expr {(0X0A + ($a*2))}] [expr {(0X0A + ($a*2))}]] c audio_status
		if {$audio_status == 0} {
			set ::pgced::audiostream$a "none"
		} else {
			set ::pgced::audiostream$a [expr {$audio_status & 7}]
		}
		if {$::pgced::__pgctype == "menu"} break
	}

	# window geometry
	update
	if {! [winfo exists $w]} {return ""}
	wm resizable $w 1 1
	wm minsize $w 300 5
	if {$::config(pgceditor_win_geometry) != ""} {
		if {[catch {wm geometry $w $::config(pgceditor_win_geometry)}]} {
			set ::config(pgceditor_win_geometry) ""
		}
	}
	if {$::config(pgceditor_win_geometry) == ""} {
		set ww [winfo reqwidth $w]
		set wh [expr [winfo reqheight $w]+4]
		set wx [expr {[winfo rootx .]+(([winfo width  .]-$ww)/2)}]
		set wy [expr {[winfo rooty .]+(([winfo height .]-$wh)/2)-25}]
		if {$wx < 10} {set wx 10}
		if {$wy < 22} {set wy 22}
		$w.f2.fc.c configure -height 1
		wm geometry $w ${ww}x$wh+$wx+$wy
	}

	# Build the cells table
	set end_time [build_cell_table $w]

	if {! [winfo ismapped $w]} {
		wm deiconify $w
	}
	grab set $w
	focus $w

	if {$default_field != ""} {
		switch [string tolower $default_field] {
			"prevpgcn" {set default_field $w.f1.fl.mf.e3}
			"nextpgcn" {set default_field $w.f1.fl.mf.e4}
			"gouppgcn" {set default_field $w.f1.fl.mf.e5}
			default {set default_field ""}
		}
		if {$default_field != ""} {
			catch {focus -force $default_field}
			catch {$default_field selection from 0}
			catch {$default_field selection to end}
			catch {$default_field icursor end}
		}
	}

	update idletasks
	if {$end_time != -1 && $end_time != $::pgced::__pgcpbtime} {
		set txt "WARNING: The PGC Playback Time ([BCDtime $::pgced::__pgcpbtime true]) doesn't match"
		set txt "$txt\nthe total of the cells times ([BCDtime $end_time true])."
		set txt "$txt\n\nFix it?"
		set r [tk_messageBox -title "Pgc Editor" -icon warning \
			-message $txt -parent . -type yesno -default yes]
		if {$r == "yes"} {
			set ::pgced::__pgcheader "[string range $::pgced::__pgcheader 0 3]$end_time[string range $::pgced::__pgcheader 8 end]"
			set ::pgced::pbtimetxt [BCDtime $end_time true]
			set ::pgced::__pgcpbtime $end_time
		}
	}

	show_msg ""

	set ::pgced::done -1
	tkwait variable ::pgced::done
	wm withdraw $w
	update idletasks

	set ::config(pgceditor_win_geometry) [wm geometry $w]
	if {$::pgced::done} {
		accept_modifs $w
		destroy $w
		return [list $::pgced::__pgcheader $::pgced::__cellpbtable $::pgced::__cellpostable $::pgced::__programmap $::pgced::__PTTlist $::pgced::__numcells $::pgced::__usercomment]
	} else {
		destroy $w
		return ""
	}
}
proc validate_streams_display {} {
	set w ".pgceditor"
	set vattr [::streams::get_video_attribute_value]
	set videodescr [::streams::video_attributes $vattr]
	if {[lindex $videodescr 0] == "4:3"} {
		set txt "(Given the current domain stream attributes, only the 4:3 stream is used.)"
	} else {
		set txt ""
		if {[string first "Letterbox" [lindex $videodescr 6]] > -1} {
			set txt "Letterbox"
		}
		if {[string first "Pan&Scan" [lindex $videodescr 6]] > -1} {
			if {$txt != ""} {set txt "$txt and Pan&Scan"} else {set txt "Pan&Scan"}
		}
		if {[llength $txt] == 1} {set txt " and $txt"} else {set txt ", $txt"}
		set txt "(Given the stream attributes, the Wide$txt streams are used.)"
	}
	$w.f1.mfs.tit configure -text "Sub-picture VOB's Decoding Streams, for 4:3, wide, letterbox, pan&scan.\n$txt"

	set type [lindex $::editor_cmds_from 0]
	set enabledcolor  [$w.fb.ok cget -foreground]
	set disabledcolor [$w.fb.ok cget -disabledforeground]
	set numaudios [::streams::get_pgc_num_audio $type $::pgced::__index]
	for {set a 1} {$a <= 8} {incr a} {
		if {$a <= $numaudios} {
			$w.f1.mfa.ab$a configure -foreground $enabledcolor
		} else {
			$w.f1.mfa.ab$a configure -foreground $disabledcolor
		}
		if {$::pgced::__pgctype == "menu"} break
	}
	set numsubpics [::streams::get_pgc_num_subpic $type $::pgced::__index]
	for {set s 1} {$s <= 32} {incr s} {
		if {$s <= $numsubpics} {
			$w.f1.mfs.ab$s  configure -foreground $enabledcolor
		} else {
			$w.f1.mfs.ab$s  configure -foreground $disabledcolor
		}
		if {$::pgced::__pgctype == "menu"} break
	}
}

proc accept_modifs {w} {
	switch $::pgced::pbmode {
		0 {
			set_int ::pgced::__pgcheader 0xA2 0 1
		}
		1 {
			set_int ::pgced::__pgcheader 0xA2 $::pgced::pbprogs 1
		}
		2 {
			set_int ::pgced::__pgcheader 0xA2 [expr {$::pgced::pbprogs | 0x80}] 1
		}
	}
	set ::pgced::__pgcheader "[string range $::pgced::__pgcheader 0 3]$::pgced::__pgcpbtime[string range $::pgced::__pgcheader 8 end]"

	if {$::pgced::puos      != ""} {set_int ::pgced::__pgcheader 8    [expr {$::pgced::puos}] 4}
	if {$::pgced::nextpgcn  != ""} {set_int ::pgced::__pgcheader 0X9C [expr {$::pgced::nextpgcn}] 2}
	if {$::pgced::prevpgcn  != ""} {set_int ::pgced::__pgcheader 0X9E [expr {$::pgced::prevpgcn}] 2}
	if {$::pgced::goup_type} {
		if {$::pgced::gouppgcn  != ""} {set_int ::pgced::__pgcheader 0XA0 [expr {$::pgced::gouppgcn}] 2}
	} else {
		set_int ::pgced::__pgcheader 0XA0 65535 2
	}
	if {$::pgced::stilltime != ""} {set_int ::pgced::__pgcheader 0XA3 [expr {$::pgced::stilltime}] 1}
	if {$::pgced::__color_table_modified} {
		for {set c 0} {$c < 16} {incr c} {
			set rgb_color [$w.f2.f.mfc.f1.b$c cget -background]
			set_int ::pgced::__pgcheader [expr {(0XA4 + (4*$c))}] [rgb2yc $rgb_color] 4
		}
	}
	if {$::pgced::__pgctype == "menu"} {
		set last 1
	} else {
		set last 8
	}
	for {set a 1} {$a <= $last} {incr a} {
		set varname ::pgced::audiostream$a
		if {[subst $$varname] == "none"} {
			set as 0
		} else {
			set as [expr {[subst $$varname] | 0X80}]
		}
		set_int ::pgced::__pgcheader [expr {(0X0A + ($a*2))}] $as 1
	}
	if {$::pgced::__pgctype == "menu"} {
		set last 1
	} else {
		set last 32
	}
	for {set s 1} {$s <= $last} {incr s} {
		set varname ::pgced::subpicstream$s
		if {[subst $$varname] == "none"} {
			set ss 0
		} else {
			set sl [subst $$varname]
			set ss [expr {(0X80000000 + \
					([lindex $sl 0] * 0X1000000) + \
					([lindex $sl 1] * 0X10000) + \
					([lindex $sl 2] * 0X100) + \
					[lindex $sl 3])}]
		}
		set_int ::pgced::__pgcheader [expr {(0X18 + ($s*4))}] $ss 4
	}

	for {set c 1} {$c <= $::pgced::__numcells} {incr c} {
		set cellidx1 [expr {($c - 1) * 24}]
		set cflags [$w.f2.fc.c.f.w_${c}_3 cget -text]
		set_int ::pgced::__cellpbtable $cellidx1 [lindex $cflags 0] 1
		set origcflags2 [expr [get_int ::pgced::__cellpbtable [expr {$cellidx1 + 1}] 1] & 0x9F]
		set_int ::pgced::__cellpbtable [expr {$cellidx1 + 1}] [expr [lindex $cflags 1] | $origcflags2] 1
#		set_int ::pgced::__cellpbtable [expr {$cellidx1 + 1}] [subst \$::pgced::var_${c}_XXX6] 1
		set tmp [subst \$::pgced::var_${c}_6]
		if {$tmp == "" || $tmp == "0x" || $tmp == "0X"} {set tmp 0}
		set_int ::pgced::__cellpbtable [expr {$cellidx1 + 2}] [expr {$tmp}] 1
		set tmp [subst \$::pgced::var_${c}_7]
		if {$tmp == "" || $tmp == "0x" || $tmp == "0X"} {set tmp 0}
		set_int ::pgced::__cellpbtable [expr {$cellidx1 + 3}] [expr {$tmp}] 1
	}

	if {$::pgced::__need_new_tmap && $::pgced::__pgctype == "pgc"} {
		set tmp [split $::pgced::__index ,]
		::tmap::need_new_tmap [lindex $tmp 0] [lindex $tmp 1]
	}
}

### build Cells table (+ PTT and Programs infos)
proc build_cell_table {w} {

	destroy $w.f2.fc.c.f
	frame $w.f2.fc.c.f
	pack  $w.f2.fc.c.f -anchor nw
	$w.f2.fc.c create window 0 0 -window $w.f2.fc.c.f -anchor nw

	if {$::pgced::__numcells == 0} {
		label $w.f2.fc.c.f.w_0_0 -text "Dummy PGC :  no cells."
		pack  $w.f2.fc.c.f.w_0_0 -padx 10 -pady 10
		set end_time -1
	} else {
		show_msg "Building cell table..."
		if {$::pgced::__pgctype == "menu"} {
			label $w.f2.fc.c.f.w_0_0 -text "Num\nBut-\ntons"
			::balloon::set_balloon $w.f2.fc.c.f.w_0_0 "Number of buttons or BOVs in this cell."
		} else {
			label $w.f2.fc.c.f.w_0_0 -text "Chap.\n(PTT)"
			::balloon::set_balloon $w.f2.fc.c.f.w_0_0 "Chapter number (PTTN).\nIf blank, then this cell belongs\nto the previous chapter."
		}
		label $w.f2.fc.c.f.w_0_1 -text "Prog."
		#::balloon::set_balloon $w.f2.fc.c.f.w_0_1 "Program number."
		label $w.f2.fc.c.f.w_0_2 -text "Cell"
		#::balloon::set_balloon $w.f2.fc.c.f.w_0_2 "Cell number."
		label $w.f2.fc.c.f.w_0_3 -text "Type\nFlags"
		::balloon::set_balloon $w.f2.fc.c.f.w_0_3 "Type of the cell (STC discontinuity, Interleaved,\nSeamless joint, Angle type...)\nClick to show details or edit."
		label $w.f2.fc.c.f.w_0_4 -text "seam\nless\nJoint" -justify left	;# LB!
		::balloon::set_balloon $w.f2.fc.c.f.w_0_4 "Shortcut to the Seamless Joint Type Flag.\nWhen OFF, the cell will not be played seamlessly.\nThis COULD indicate the location of the layer break,\nbut this flag has also many other usages!"
		label $w.f2.fc.c.f.w_0_5 -text "   \nA.\n   "
		::balloon::set_balloon $w.f2.fc.c.f.w_0_5 "Angle number\ncomputed from the Type Flags.\n( \" - \" = cell played in all angles)"
#		label $w.f2.fc.c.f.w_0_XXX6 -text "VOBU\nstill"
#		::balloon::set_balloon $w.f2.fc.c.f.w_0_XXX6 "Infinite pause at each VOBU of the cell.\nYou must press Play to continue."
		label $w.f2.fc.c.f.w_0_6 -text "Cell\nStill\nTime"
		::balloon::set_balloon $w.f2.fc.c.f.w_0_6 "Still time (in seconds).\nPause at the end of the cell playback.\n0 to 254 secs.  255 = infinite pause."
		label $w.f2.fc.c.f.w_0_7 -text "Cell\nCmd\n#"
		::balloon::set_balloon $w.f2.fc.c.f.w_0_7 "Cell command number.\nThe command will be executed\nafter the playback of the cell.\n(Implies a seamless discontinuity!)"
		label $w.f2.fc.c.f.w_0_8 -text "Playback\nTime"
		#::balloon::set_balloon $w.f2.fc.c.f.w_0_8 ""
		label $w.f2.fc.c.f.w_0_9 -text "End\nTime"
		#::balloon::set_balloon $w.f2.fc.c.f.w_0_9 ""
		label $w.f2.fc.c.f.w_0_10 -text "Entry\nVOBU\nsector"	-width 7 -justify right -anchor e
		#::balloon::set_balloon $w.f2.fc.c.f.w_0_10 ""
		label $w.f2.fc.c.f.w_0_11 -text "First\nILVU\nEnd"		-width 7 -justify right -anchor e
		#::balloon::set_balloon $w.f2.fc.c.f.w_0_11 ""
		label $w.f2.fc.c.f.w_0_12 -text "Last\nVOBU\nStart"		-width 7 -justify right -anchor e
		#::balloon::set_balloon $w.f2.fc.c.f.w_0_12 ""
		label $w.f2.fc.c.f.w_0_13 -text "Last\nVOBU\nEnd"		-width 7 -justify right -anchor e
		#::balloon::set_balloon $w.f2.fc.c.f.w_0_13 ""
		label $w.f2.fc.c.f.w_0_14 -text "VOB/Cell\nID"
		::balloon::set_balloon $w.f2.fc.c.f.w_0_14 "VOB / Cell IDs.\nClick to assign another cell,\nor to blank the cell."
		if {$::config(use_preview)} {
			button $w.f2.fc.c.f.w_0_15 -image ::preview::bitmapplay -width 13 -height 15 -command \
					"::preview::preview_current_pgc_2"
			::balloon::set_balloon $w.f2.fc.c.f.w_0_15 "Preview cell\n(This button:\npreview original PGC)"
		}

		for {set c 1} {$c <= $::pgced::__numcells} {incr c} {
			set cell_program($c) ""
			set prog_chapter($c) 0
		}
		for {set p 1} {$p <= $::pgced::__numprograms} {incr p} {
			set cell_program([get_int ::pgced::__programmap [expr {$p-1}] 1]) $p
		}
		foreach pttpair $::pgced::__PTTlist {
			set pg  [lindex $pttpair 0]
			set ptt [lindex $pttpair 1]
			for {set c 1} {$c <= $::pgced::__numcells} {incr c} {
				if {$pg == $cell_program($c) && $prog_chapter($c) == 0} {
					set prog_chapter($c) $ptt
				}
			}
		}

		set vts [lindex [split $::pgced::__index ","] 0]
		set end_time "\000\000\000\000"
		set_int end_time 3 [expr [get_int ::pgced::__cellpbtable 7 1] & 0XC0] 1
		set lastangle "-"
		set progress_max [expr $::pgced::__numcells * 1.2]

		for {set c 1} {$c <= $::pgced::__numcells} {incr c} {
			update_progress $c $progress_max
			set cellidx1 [expr {($c - 1) * 24}]
			set cellidx2 [expr {($c - 1) * 4}]
			set vobid  [get_int ::pgced::__cellpostable $cellidx2 2]
			set cellid [get_int ::pgced::__cellpostable [expr {$cellidx2 + 3}] 1]

			# number of menu buttons or PTT
			if {$::pgced::__pgctype == "menu"} {
				if {[info exists ::buttons::butsinfo(menu,$vts,$vobid,$cellid,numbuts)]} {
					set txt $::buttons::butsinfo(menu,$vts,$vobid,$cellid,numbuts)
				} else {
					set txt "?"
				}
				label $w.f2.fc.c.f.w_${c}_0 -text $txt
			} else {
				set txt $prog_chapter($c)
				if {$txt == "0"} {set txt ""}
				label $w.f2.fc.c.f.w_${c}_0 -text $txt
			}

			# PG
			button $w.f2.fc.c.f.w_${c}_1 -text $cell_program($c) -anchor e -pady 0 -width 3 \
					-command "::pgced::toggle_program $w $c"
			if {$::pgced::__pgctype == "pgc" && $cell_program($c) == ""} {
				$w.f2.fc.c.f.w_${c}_0 configure -relief flat -state disabled -text ""
			}

			# Cell number
			label $w.f2.fc.c.f.w_${c}_2 -text $c

			# Type flags
			set type  [get_int ::pgced::__cellpbtable $cellidx1 1]
			set type2 [expr [get_int ::pgced::__cellpbtable [expr $cellidx1+1] 1] & 0x60]
			if {$type2 < 10} {set type2 "  $type2"}
			button $w.f2.fc.c.f.w_${c}_3 -text "$type  $type2" -width 7 -anchor e -pady 0 \
					-command "::pgced::cell_type_editor $c"

			# Seamless flag
			set [subst ::pgced::var_${c}_4] [expr {[expr {$type & 8}] == 8}]
			checkbutton $w.f2.fc.c.f.w_${c}_4 -variable ::pgced::var_${c}_4 \
					-command "::pgced::set_seamless_flag $c"

			# Angle
			if {[expr $type & 16]==0} {
				set angle "-"
				set lastangle "-"
			} else {
				switch [expr $type & 192] {
					case 0 {set angle "???"; set lastangle "???"}
					case 64 {set angle 1; set lastangle 1}
					default {
						if {[string is integer $lastangle]} {
							set angle [expr $lastangle+1]
						} else {
							set angle "?"
						}
						set lastangle $angle
						if {[expr $type & 192] == 192} {set lastangle "-"}
					}
				}
			}
			label $w.f2.fc.c.f.w_${c}_5 -text $angle

			# Cell still time
			set  ::pgced::var_${c}_6 [get_int ::pgced::__cellpbtable [expr {$cellidx1 + 2}] 1]
			entry $w.f2.fc.c.f.w_${c}_6 -textvariable ::pgced::var_${c}_6 -width 3 -justify right \
					-validate all -validatecommand "::pgced::validate_cellstilltime $c {%d} {%P}"

			# Cell command
			set  ::pgced::var_${c}_7 [get_int ::pgced::__cellpbtable [expr {$cellidx1 + 3}] 1]
			entry $w.f2.fc.c.f.w_${c}_7 -textvariable ::pgced::var_${c}_7 -width 3 -justify right \
					-validate all -validatecommand "::pgced::validate_cellcommand $c {%d} {%P}"
			set cell_time [string range $::pgced::__cellpbtable [expr {$cellidx1 + 4}] [expr {$cellidx1 + 7}]]
			label $w.f2.fc.c.f.w_${c}_8 -text [BCDtime $cell_time false]

			# Add cell time to total PGC time, but only if angle 1.
			if {[expr {$type & 0x10}] == 0 || [expr {$type & 0xC0}] == 0x40} {
				set end_time [AddBCDtime $cell_time $end_time]
			}

			# Cell duration and VOBUs
			label $w.f2.fc.c.f.w_${c}_9 -text [BCDtime $end_time false]
			label $w.f2.fc.c.f.w_${c}_10 -text [get_int ::pgced::__cellpbtable [expr {$cellidx1 + 8}]  4] -width 7 -justify right -anchor e
			label $w.f2.fc.c.f.w_${c}_11 -text [get_int ::pgced::__cellpbtable [expr {$cellidx1 + 12}] 4] -width 7 -justify right -anchor e
			label $w.f2.fc.c.f.w_${c}_12 -text [get_int ::pgced::__cellpbtable [expr {$cellidx1 + 16}] 4] -width 7 -justify right -anchor e
			label $w.f2.fc.c.f.w_${c}_13 -text [get_int ::pgced::__cellpbtable [expr {$cellidx1 + 20}] 4] -width 7 -justify right -anchor e

			# V/CID
			button $w.f2.fc.c.f.w_${c}_14 -pady 0 \
					-text "$vobid / $cellid" \
					-command "::pgced::accept_modifs $w ; \
							::pgced::reassign_VOB_ID $w $c $vobid $cellid"

			# Preview button
			if {$::config(use_preview)} {
				# BUG: the button command should be updated after a V/CID change or a remove cell
				button $w.f2.fc.c.f.w_${c}_15 -image ::preview::bitmapplay -width 13 -height 15 -command \
						"::preview::preview_vcid [lindex $::editor_cmds_from 0] $vts $vobid $cellid"
			}

			# Seamless flag warnings
			validate_seamless_flag $c
		}
		update_progress 90
		for {set c 0} {$c <= $::pgced::__numcells} {incr c} {
			for {set f 0} {$f <= 14} {incr f} {
				grid $w.f2.fc.c.f.w_${c}_$f -row $c -column $f
			}
			grid configure $w.f2.fc.c.f.w_${c}_14 -sticky ew -padx 3
			if {$::config(use_preview)} {
				grid $w.f2.fc.c.f.w_${c}_15 -row $c -column 16
			}
		}
	}

	# inform scrollbar of the height of the canvas's internal window
	if {[winfo exists $w.f2.fc.c.f]} {
		update
		update_progress 95
		$w.f2.fc.c configure -width [winfo width $w.f2.fc.c.f] \
				-scrollregion "0 0 0 [winfo height $w.f2.fc.c.f]"
	}
	update_progress 100
	#wm geometry $w [winfo reqwidth $w]x[winfo reqheight $w]
	show_msg ""
	return $end_time
}

proc validate_seamless_flag {cell {prevstilltime -1} {prevcellcmd -1}} {
	if {$cell<1} return
	if {![winfo exists .pgceditor.f2.fc.c.f.w_${cell}_4]} return

	set cellflag_checkbox .pgceditor.f2.fc.c.f.w_${cell}_4
	set v ::pgced::var_${cell}_4
	set cellflag_value [subst \$$v]
	$cellflag_checkbox configure -state normal -selectcolor $::config(system_window_color)
	if {$cell==1} {
		if {$cellflag_value} {
			$cellflag_checkbox configure -selectcolor $::config(system_window_color)
			::balloon::set_balloon $cellflag_checkbox "Must be clear\non first cell!"
		} else {
			$cellflag_checkbox configure -state disabled
		}
	} else {
		if {$cellflag_value} {
			if {$prevcellcmd==-1} {
				set v ::pgced::var_[expr {$cell-1}]_7
				set prevcellcmd   [subst \$$v]
			}
			if {[string is integer $prevcellcmd] && $prevcellcmd > 0} {
				$cellflag_checkbox configure -selectcolor pink
				::balloon::set_balloon $cellflag_checkbox "Must be clear after\na cell command!"
			}
			if {$prevstilltime==-1} {
				set v ::pgced::var_[expr {$cell-1}]_6
				set prevstilltime [subst \$$v]
			}
			if {[string is integer $prevstilltime] && $prevstilltime > 0} {
				$cellflag_checkbox configure -selectcolor pink
				::balloon::set_balloon $cellflag_checkbox "Must be clear after\na cell still time > 0 !"
			}
		} else {
			# TODO: Better handling of ILVed cells
			set entryvobu [.pgceditor.f2.fc.c.f.w_${cell}_10 cget -text]
			set ilved [.pgceditor.f2.fc.c.f.w_${cell}_11 cget -text]
			set prevlastvobuend [.pgceditor.f2.fc.c.f.w_[expr $cell-1]_13 cget -text]
			if {[expr {$prevlastvobuend+1}] != $entryvobu && $ilved==0} {
		 		.pgceditor.f2.fc.c.f.w_${cell}_4 configure -state disabled
		 	}
		}
	}
}

proc validate_cellstilltime {cell action val} {
	set valid [::ced::value_entry_validate 0 255 $action $val]
	if {$valid && $action!=-1} {
		validate_seamless_flag [expr {$cell+1}] $val -1
	}
	return $valid
}

proc validate_cellcommand {cell action val} {
	set numcellcmds [llength $::pgced::cellcmds]
	set w .pgceditor.f2.fc.c.f.w_${cell}_7
	$w configure -background $::config(system_window_color)
	set valid [::ced::value_entry_validate 0 128 $action $val]
	if {$valid} {
		if {$val == 0 || $val == "" || ![string is integer $val]} {
			::balloon::set_balloon $w ""
		} else {
			if {$val > $numcellcmds} {
				::balloon::set_balloon $w "Cell command\ndoesn't exist!"
				$w configure -background pink
			} else {
				::balloon::set_balloon $w [::disasm::disasm [lindex $::pgced::cellcmds [expr {$val-1}]]]
			}
		}
		validate_seamless_flag [expr {$cell+1}] -1 $val
	}
	return $valid
}


proc validate_PGCNLink_value {w action val} {
	set val [expr $val]
	set valid [::ced::value_entry_validate 0 32767 $action $val]
	if {$valid && $val != "" && [string is integer $val]} {
		set type [lindex $::editor_cmds_from 0]
		set indexes [split [lindex $::editor_cmds_from 1] ,]
		set vts [lindex $indexes 0]
		if {$type == "pgc"} {
			if {$vts == 0} {
				set numpgcs 0
			} else {
				set numpgcs $::pgcs($vts,numpgcs)
			}
			if {$val > $numpgcs} {
				$w configure -background pink
				::balloon::set_balloon $w "Target PGC\ndoesn't exist!"
			} else {
				if {$val > 0 && $::pgcs([lindex $::editor_cmds_from 1],TTN) \
						!= $::pgcs($vts,$val,TTN)} {
					$w configure -background pink
					::balloon::set_balloon $w "Target PGC points\nto another TTN!"
				} else {
					$w configure -background $::config(system_window_color)
					::balloon::set_balloon $w ""
				}
			}
		} else {
			if {$val > $::menus($vts,[lindex $indexes 1],numpgcs)} {
				$w configure -background pink
				::balloon::set_balloon $w "Target PGC\ndoesn't exist!"
			} else {
				$w configure -background $::config(system_window_color)
				::balloon::set_balloon $w ""
			}
		}
	}
	return $valid
}
proc refresh_chapters {} {
	for {set c 1} {$c <= $::pgced::__numcells} {incr c} {
		set prog_chapter($c) 0
		.pgceditor.f2.fc.c.f.w_${c}_0 configure -text ""
	}
	foreach pttpair $::pgced::__PTTlist {
		set pg  [lindex $pttpair 0]
		set ptt [lindex $pttpair 1]
		for {set c 1} {$c <= $::pgced::__numcells} {incr c} {
			set cell_program [.pgceditor.f2.fc.c.f.w_${c}_1 cget -text]
			if {$pg == $cell_program && $prog_chapter($c) == 0} {
				set prog_chapter($c) $ptt
				if {$ptt == 0} {set ptt ""}
				.pgceditor.f2.fc.c.f.w_${c}_0 configure -text $ptt
			}
		}
	}
}

proc refresh_angle_values {} {
	set lastangle "-"
	for {set c 1} {$c <= $::pgced::__numcells} {incr c} {
		set type [.pgceditor.f2.fc.c.f.w_${c}_3 cget -text]
		if {[expr $type & 16]==0} {
			set angle "-"
			set lastangle "-"
		} else {
			switch [expr $type & 192] {
				case 0 {set angle "???"; set lastangle "???"}
				case 64 {set angle 1; set lastangle 1}
				default {
					if {[string is integer $lastangle]} {
						set angle [expr $lastangle+1]
					} else {
						set angle "?"
					}
					set lastangle $angle
					if {[expr $type & 192] == 192} {set lastangle "-"}
				}
			}
		}
		.pgceditor.f2.fc.c.f.w_${c}_5 configure -text $angle
	}
}

proc value_entry_validate {min max action val} {
	if {$min > 0 && $val < $min} {
		bell
		return true
	}
	if {$action != 0} {
		if {$val=="0"||$val=="0x"||$val=="0X"} {
			if {$min>0 && $val=="0"} bell
			return true
		}
		if {![string is integer $val]} {
			bell
			return false
		}
		if {$val < $min} {
			bell
			return false
		}
		if {$val > $max} {
			bell
			return false
		}
	}
	return true
}

proc rgb2yc {rgb_color} {
	if {[string range $rgb_color 0 0] == "#"} {
		set rgb_color [string range $rgb_color 1 end]
	}
	set r [x2d [string range $rgb_color 0 1]]
	set g [x2d [string range $rgb_color 2 3]]
	set b [x2d [string range $rgb_color 4 5]]

	set y  [expr {round(( 0.257*$r) + (0.504*$g) + (0.098*$b) + 16)}]
	set cr [expr {round(( 0.439*$r) - (0.368*$g) - (0.071*$b) + 128)}]
	set cb [expr {round((-0.148*$r) - (0.291*$g) + (0.439*$b) + 128)}]

	if {$y  < 0} {set y  0}
	if {$cr < 0} {set cr 0}
	if {$cb < 0} {set cb 0}
	if {$y  > 255} {set y  255}
	if {$cr > 255} {set cr 255}
	if {$cb > 255} {set cb 255}

	return [expr {($y*0X10000) + ($cr*0X100) + $cb}]
}

proc yc2rgb {yc_color} {
	if {[string length $yc_color] == 4} {
		set yc_color [string range $yc_color 1 end]
	}
	binary scan $yc_color ccc y cr cb
	set y  [expr {($y  + 0X100) % 0X100}]
	set cr [expr {($cr + 0X100) % 0X100}]
	set cb [expr {($cb + 0X100) % 0X100}]

	set r [expr {round((1.164*($y-16)) + (1.596*($cr-128)))}]
	set g [expr {round((1.164*($y-16)) - (0.813*($cr-128)) - (0.392*($cb-128)))}]
	set b [expr {round((1.164*($y-16)) + (2.017*($cb-128)))}]

	if {$r < 0} {set r 0}
	if {$g < 0} {set g 0}
	if {$b < 0} {set b 0}
	if {$r > 255} {set r 255}
	if {$g > 255} {set g 255}
	if {$b > 255} {set b 255}

	return [format "#%02X%02X%02X" $r $g $b]
}

proc set_color {colornum w} {
	set newcolor [tk_chooseColor -initialcolor [$w cget -background] -parent .pgceditor -title "Color #$colornum"]
	if {$newcolor == ""} return
	$w configure -background $newcolor
	set ::pgced::__color_table_modified true
}

proc copy_colors {} {
	set colors {}
	for {set c 0} {$c < 16} {incr c} {
		lappend colors [.pgceditor.f2.f.mfc.f1.b$c cget -background]
	}
	clipboard clear
	clipboard append $colors
}

proc paste_colors {} {
	set err [catch {set colors [clipboard get]}]
	if {$err || [llength $colors] != 16} {
		bell
		return
	}
	for {set c 0} {$c < 16} {incr c} {
		.pgceditor.f2.f.mfc.f1.b$c configure -background [lindex $colors $c]
	}
	set ::pgced::__color_table_modified true
}

proc copy_all_audio {} {
	set streams pgcedit_audio_streams
	if {$::pgced::__pgctype == "menu"} {
		set last 1
	} else {
		set last 8
	}
	for {set a 1} {$a <= $last} {incr a} {
		set varname ::pgced::audiostream$a
		lappend streams [subst $$varname]
	}
	clipboard clear
	clipboard append $streams
}
proc paste_all_audio {} {
	set streams [clipboard get]
	if {$::pgced::__pgctype == "menu"} {
		set last 1
	} else {
		set last 8
	}
	if {[llength $streams] != [expr $last+1] || [lindex $streams 0] != "pgcedit_audio_streams"} {bell;return}
	for {set a 1} {$a <= $last} {incr a} {
		set ::pgced::audiostream$a [lindex $streams $a]
	}
}
proc copy_all_subpic {} {
	set streams pgcedit_subpic_streams
	if {$::pgced::__pgctype == "menu"} {
		set last 1
	} else {
		set last 32
	}
	for {set s 1} {$s <= $last} {incr s} {
		set varname ::pgced::subpicstream$s
		lappend streams [subst $$varname]
	}
	clipboard clear
	clipboard append $streams
}
proc paste_all_subpic {} {
	set streams [clipboard get]
	if {$::pgced::__pgctype == "menu"} {
		set last 1
	} else {
		set last 32
	}
	if {[llength $streams] != [expr $last+1] || [lindex $streams 0] != "pgcedit_subpic_streams"} {bell;return}
	for {set s 1} {$s <= $last} {incr s} {
		set ::pgced::subpicstream$s [lindex $streams $s]
	}
}

proc export_colors {} {
	set txt "Export YCrCb colors as 64 bytes raw data?"
	set txt "$txt\nAnswer 'No' to export RGB colors as ASCII text."
	set r [tk_messageBox -title "Export Color Lookup Table" -icon question \
			-message $txt -parent .pgceditor -type yesnocancel -default yes]
	if {$r == "cancel"} return
	if {$r == "yes"} {
		set extension .clb
		set filetypes {{"YCrCb raw color table" {*.clb}} {"all files" {*.*}}}
	} else {
		set extension .txt
		set filetypes {{"Text files with RGB colors" {*.txt}} {"all files" {*.*}}}
	}

	set type  [lindex $::editor_cmds_from 0]
	set index [lindex $::editor_cmds_from 1]
	set defaultfilename "[file rootname [::main::get_default_filename $::editor_cmds_from]]$extension"
	set filename [tk_getSaveFile -filetypes $filetypes \
		-parent .pgceditor -initialdir $::config(import_initialdir) \
		-initialfile $defaultfilename -title "PgcEdit: Export PGC color lookup table"]
	if {$filename == ""} {
		return
	}
	set ::config(import_initialdir) [file dirname $filename]
	if {[string toupper [file extension $filename]] != [string toupper $extension]} {
		set filename "${filename}$extension"
	}

	if {$r == "yes"} {
		saveFile $filename binary [string range $::pgced::__pgcheader 0xA4 0xE3]
	} else {
		set txt ""
		for {set c 0} {$c < 16} {incr c} {
			set rgb [.pgceditor.f2.f.mfc.f1.b$c cget -background]
			set r [x2d [string range $rgb 1 2]]
			set g [x2d [string range $rgb 3 4]]
			set b [x2d [string range $rgb 5 6]]
			set txt "${txt}\t\tColor $c=$r, $g, $b\n"
		}
		set txt [string trim $txt \n]
		saveFile $filename auto $txt
	}
}

proc import_colors {} {
	set type  [lindex $::editor_cmds_from 0]
	set index [lindex $::editor_cmds_from 1]
	set defaultfilename "[file rootname [::main::get_default_filename $::editor_cmds_from]].clb"
	set filename [tk_getOpenFile \
		-filetypes {{"All color lookup table formats" {*.clb *.txt *.mxp VTS_*_0.IFO}} {"YCrCb raw color table" {*.clb}} {"Text files with RGB colors" {*.txt}} {"VTS IFO files" {VTS_*_0.IFO}} {"Muxman project files (RGB)" {*.mxp}} {"all files" {*.*}}} \
		-parent .pgceditor -initialdir $::config(import_initialdir) \
		-initialfile $defaultfilename -title "PgcEdit: Import PGC color lookup table"]
	if {$filename == ""} {
		return
	}
	set ::config(import_initialdir) [file dirname $filename]
	if {![file exists $filename] && [string toupper [file extension $filename]] != ".CLB"} {
		set filename "${filename}.clb"
	}
	if {![file exists $filename]} {
		show_warning -icon error -title "PgcEdit $::program_version error" \
			-message "File not found:\n$filename" -parent .pgceditor
		return
	}
	set rawcolors [readFile $filename binary]
	if {[string length $rawcolors] == 64} {
		for {set c 0} {$c < 16} {incr c} {
			set yc_color [string range $rawcolors [expr {$c*4}] [expr {($c*4) + 3}]]
			.pgceditor.f2.f.mfc.f1.b$c configure -background [yc2rgb $yc_color]
		}
		set ::pgced::__color_table_modified true
	} elseif {[string toupper [file extension $filename]] == ".IFO"} {
		set pgcit_start [expr [get_int rawcolors 0xCC 4] * 0x800]
		set numsrp [get_int rawcolors $pgcit_start 2]
		set start [expr $pgcit_start + 8 + ($numsrp * 8) + 0xA4]
		set rawcolors [string range $rawcolors $start [expr $start + 63]]
		for {set c 0} {$c < 16} {incr c} {
			set yc_color [string range $rawcolors [expr {$c*4}] [expr {($c*4) + 3}]]
			.pgceditor.f2.f.mfc.f1.b$c configure -background [yc2rgb $yc_color]
		}
		set ::pgced::__color_table_modified true
	} else {
		# try to load the color table from a TXT or a Muxman project file
		unset -nocomplain colors
		set fh [open $filename]
		while {1} {
			set line [string toupper [string trim [gets $fh] \t]]
			if {[eof $fh]} {
				break
			}
			if {[string first "COLOR" $line] == 0} {
				set line [string range $line 6 end]
				set line [string map {= " " , " "} $line]
				set ns [scan $line "%d %d %d %d" cn r g b]
				if {$ns != 4} continue
				set colors($cn) [format "#%02X%02X%02X" $r $g $b]
				if {$cn == 15} {
					set ok true
					for {set c 0} {$c < 16} {incr c} {
						if {![info exists colors($c)]} {
							set ok false
							break
						}
					}
					if {$ok} {
						close $fh
						break
					}
				}
			}
		}
		catch {close $fh}
		set ok true
		for {set c 0} {$c < 16} {incr c} {
			if {![info exists colors($c)]} {
				set ok false
				break
			}
		}
		if {$ok} {
			for {set c 0} {$c < 16} {incr c} {
				.pgceditor.f2.f.mfc.f1.b$c configure -background $colors($c)
			}
			set ::pgced::__color_table_modified true
		} else {
			show_warning -icon error -title "PgcEdit $::program_version error" \
				-message "File not of required type:\n$filename" -parent .pgceditor
		}
	}
}

set puos_labels {
	{Time play or search}
	{PTT play or search}
	{Title play}
	{Stop}
	{GoUp  (Not used in PGCs) }
	{Time or PTT search}
	{TopPG or PrevPG search}
	{NextPG search}
	{Forward scan}
	{Backward scan}
	{Menu call - Title}
	{Menu call - Root}
	{Menu call - Subpicture}
	{Menu call - Audio}
	{Menu call - Angle}
	{Menu call - PTT}
	{Resume}
	{Button select or activate}
	{Still off}
	{Pause on}
	{Audio stream change}
	{Subpicture stream change}
	{Angle change}
	{Karaoke audio mix change}
	{Video presentation mode change}
}

proc puos_editor {label parent {return_same_if_cancel true}} {
	# puos labels
	# note: GoUp (bit 4) is not used in PGC

	set w ".puoseditor"
	if {[winfo exists $w]} {
		destroy $w
	}
	toplevel $w -class Dialog
	wm withdraw $w
	wm protocol $w WM_DELETE_WINDOW {set ::pgced::puosdone 0}
	bind $w <Escape> "set ::pgced::puosdone 0"
	bind $w <Return> "set ::pgced::puosdone 1"
	if {$::config(tk) != "aqua"} {wm transient $w $parent}
	if {$::tcl_platform(platform) == "windows"} {catch {wm attributes $w -toolwindow 1}}
	wm title $w "Prohibited User Operations"

	label $w.lab -text $label
	pack  $w.lab -fill x -padx 4

	frame $w.f1
	pack  $w.f1 -fill x -padx 4

	set puon 0
	foreach p $::pgced::puos_labels {
		checkbutton $w.f1.puo$puon -text "$p ([expr {round(pow(2,$puon))}])" -anchor w \
				-pady 0 -bd $::config(min_bd) -variable ::pgced::puo_$puon -command ::pgced::set_puo_var
		pack $w.f1.puo$puon -fill x -pady 0
		incr puon
	}
	#$w.f1.puo4 configure -state disabled	;# GoUp flag normally not used in PGC (?)
	frame  $w.fb
	pack   $w.fb -fill x -padx 2 -pady 4
	button $w.fb.clear -text "Clear all" -command {set ::pgced::tmppuos 0; ::pgced::set_puos_vars}
	pack   $w.fb.clear -side left -padx 5
	button $w.fb.e -text "Input value" -command {
		set tmp 0
		for {set p 0} {$p <= 24} {incr p} {
			set varname ::pgced::puo_$p
			if {[subst $$varname]} {
				set tmp [expr {$tmp | round(pow(2,$p))}]
			}
		}
		set tmp [::stringreq::stringreq -window .inputpuos -title "PUO value" \
				-def [expr round($tmp)] -width 10 -geometry c \
				-callback ::pgced::validate_input_puo_value]
		grab set .puoseditor
		if {$tmp != ""} {
			set ::pgced::tmppu\os $tmp
			::pgced::set_puos_vars
		}
	}
	pack  $w.fb.e -side left

	frame  $w.fb.pad
	pack   $w.fb.pad -side left -expand 1 -fill x
	button $w.fb.ok -text "OK" -width 6 -command {set ::pgced::puosdone 1}
	pack   $w.fb.ok -side left
	button $w.fb.cancel -text "Cancel" -width 6 -command {set ::pgced::puosdone 0}
	pack   $w.fb.cancel -side left -padx 5

	update
	set wx [expr {[winfo rootx $parent]+(([winfo width  $parent]-[winfo reqwidth  $w])/2)}]
	set wy [expr {[winfo rooty $parent]+(([winfo height $parent]-[winfo reqheight $w])/2)-25}]
	if {$wx < 10} {set wx 10}
	if {$wy < 22} {set wy 22}
	wm geometry $w +$wx+$wy
	wm resizable $w 0 0

	set ::pgced::tmppuos $::pgced::puos
	#set ::pgced::tmppuos [expr {$::pgced::puos & 0X1FFFFEF}]	;# disable GoUp flag, not used in PGC (?)
	::pgced::set_puos_vars
	$w.lab configure -text $label

	wm deiconify $w
	focus $w
	grab set $w

	set ::pgced::puosdone -1
	tkwait variable ::pgced::puosdone
	grab release $w
	if {$parent != "."} {
		grab set $parent
	}
	destroy $w
	if {$::pgced::puosdone} {
		set ::pgced::puos 0
		for {set p 0} {$p <= 24} {incr p} {
			set varname ::pgced::puo_$p
			if {[subst $$varname]} {
				set ::pgced::puos [expr {$::pgced::puos | round(pow(2,$p))}]
			}
		}
		return $::pgced::puos
	} else {
		if {$return_same_if_cancel} {
			return $::pgced::puos
		} else {
			return -1
		}
	}
}
proc set_puos_vars {} {
	for {set p 0} {$p <= 24} {incr p} {
		if {[expr {$::pgced::tmppuos & round(pow(2,$p))}] == 0 || $::pgced::tmppuos == -1} {
			set ::pgced::puo_$p 0
		} else {
			set ::pgced::puo_$p 1
		}
	}
}
proc validate_input_puo_value {val} {
	if {$val == ""} {return true}
	set rc [::pgced::value_entry_validate 0 33554431 1 $val]
	return $rc
}

proc set_audio {streamvar streamnum} {
	set type [lindex $::editor_cmds_from 0]
	set numaudios [::streams::get_pgc_num_audio $type $::pgced::__index]
	if {$streamnum <= $numaudios} {
		set descr [::streams::get_pgc_audio_descr $type $::pgced::__index [expr {$streamnum-1}]]
		set ::pgced::startaudioid 0
		if {[string first (AC3)		  $descr]>=0} {set ::pgced::startaudioid [expr {(0x80)}]}
		if {[string first (MPEG)	  $descr]>=0} {set ::pgced::startaudioid [expr {(0xC0)}]}
		if {[string first (MPEG2-ext) $descr]>=0} {set ::pgced::startaudioid [expr {(0xC8)}]}
		if {[string first (LPCM)	  $descr]>=0} {set ::pgced::startaudioid [expr {(0xA0)}]}
		if {[string first (DTS)		  $descr]>=0} {set ::pgced::startaudioid [expr {(0x88)}]}
		if {[string first (SDDS)	  $descr]>=0} {set ::pgced::startaudioid [expr {(0x90)}]}
	} else {
		set descr "[expr {$streamnum-1}]: not used"
		set ::pgced::startaudioid 0
	}

	set w .audiosetup
	if {! [winfo exists $w]} {
		toplevel $w -class Dialog
		wm withdraw $w
		if {$::config(tk) != "aqua"} {wm transient $w .pgceditor}
		wm protocol $w WM_DELETE_WINDOW {set ::pgced::audiodone 0}
		bind $w <Escape> "set ::pgced::audiodone 0"
		bind $w <Return> "set ::pgced::audiodone 1"
		wm title $w "Audio stream setup"
		wm resizable $w 0 0
		if {$::tcl_platform(platform) == "windows"} {catch {wm attributes $w -toolwindow 1}}

		label $w.lab -text "Audio $descr"
		pack  $w.lab -pady 4

		frame $w.f
		pack  $w.f
		label $w.f.l1 -text "VOB Audio stream number"
		label $w.f.l2 -text "ID"
		scale $w.f.s -from 0 -to 7 -variable "::pgced::audio_status" -length 170 -command {
				if {$::pgced::startaudioid == 0} {
					set ::pgced::audio_id "\n0x??"
				} else {
					set ::pgced::audio_id \n[format 0x%02X [expr {$::pgced::audio_status+$::pgced::startaudioid}]]
				}
				set tmp }
		label $w.f.id -textvariable ::pgced::audio_id -width 4 -anchor w
		grid  $w.f.l1 -row 0 -column 0
		grid  $w.f.l2 -row 0 -column 1
		grid  $w.f.s  -row 1 -column 0
		grid  $w.f.id -row 1 -column 1 -sticky se

		frame  $w.fb
		pack   $w.fb -padx 2 -pady 4 -anchor e
		button $w.fb.dis -text "Disable" -width 7 -command {set ::pgced::audiodone 2}
		pack   $w.fb.dis -side left -padx 4
		::balloon::set_balloon $w.fb.dis \
				"When an audio stream is disabled,\nit will not show up anymore.\nSelecting this stream via a menu\nbutton will play stream 1!"
		button $w.fb.ok -text "Set" -width 7 -command {set ::pgced::audiodone 1}
		pack   $w.fb.ok -side left
		button $w.fb.cancel -text "Cancel" -width 7 -command {set ::pgced::audiodone 0}
		pack   $w.fb.cancel -side left -padx 4

		update
		set wx [expr {[winfo rootx .pgceditor]+(([winfo width  .pgceditor]-[winfo reqwidth  $w])/2)}]
		set wy [expr {[winfo rooty .pgceditor]+(([winfo height .pgceditor]-[winfo reqheight $w])/2)-25}]
		if {$wx < 10} {set wx 10}
		if {$wy < 22} {set wy 22}
		wm geometry $w +$wx+$wy
		wm resizable $w 0 0
	}
	if {[subst $$streamvar] == "none"} {
		set ::pgced::audio_status 0
	} else {
		set ::pgced::audio_status [subst $$streamvar]
	}
	if {$::pgced::startaudioid == 0} {
		set ::pgced::audio_id "\n0x??"
	} else {
		set ::pgced::audio_id \n[format 0x%02X [expr {$::pgced::audio_status+$::pgced::startaudioid}]]
	}

	wm deiconify $w
	focus $w
	grab set $w

	set ::pgced::audiodone -1
	tkwait variable ::pgced::audiodone
	grab release $w
	grab set .pgceditor
	destroy $w
	switch $::pgced::audiodone {
		1 {set $streamvar $::pgced::audio_status}
		2 {set $streamvar "none"}
	}
	return
}

proc set_subpic {streamvar descr} {
	set w .subpicsetup
	if {! [winfo exists $w]} {
		toplevel $w -class Dialog
		wm withdraw $w
		if {$::config(tk) != "aqua"} {wm transient $w .pgceditor}
		wm protocol $w WM_DELETE_WINDOW {set ::pgced::subpicdone 0}
		bind $w <Escape> "set ::pgced::subpicdone 0"
		bind $w <Return> "set ::pgced::subpicdone 1"
		wm title $w "Subpic streams setup"
		wm resizable $w 0 0
		if {$::tcl_platform(platform) == "windows"} {catch {wm attributes $w -toolwindow 1}}

		label $w.lab -text "Subpic $descr"
		pack  $w.lab -pady 4

		frame $w.f
		pack  $w.f -padx 4

		label $w.f.tl0 -text "Video modes"
		label $w.f.tl1 -text "VOB Sub-picture stream numbers"
		label $w.f.tl2 -text "IDs"

		label $w.f.l1 -text "\n4:3"
		scale $w.f.s1 -from 0 -to 31 -variable "::pgced::subpic_1_status" -length 170 \
				-command {::pgced::validate_subpic_value 1}
		label $w.f.id1 -textvariable ::pgced::subpic_1_id -width 4 -anchor w
		label $w.f.l2 -text "\nwide"
		scale $w.f.s2 -from 0 -to 31 -variable "::pgced::subpic_2_status" \
				-command {::pgced::validate_subpic_value 2}
		label $w.f.id2 -textvariable ::pgced::subpic_2_id -width 4 -anchor w
		label $w.f.l3 -text "\nletterbox"
		scale $w.f.s3 -from 0 -to 31 -variable "::pgced::subpic_3_status" \
				-command {::pgced::validate_subpic_value 3}
		label $w.f.id3 -textvariable ::pgced::subpic_3_id -width 4 -anchor w
		label $w.f.l4 -text "\npan&scan"
		scale $w.f.s4 -from 0 -to 31 -variable "::pgced::subpic_4_status" \
				-command {::pgced::validate_subpic_value 4}
		label $w.f.id4 -textvariable ::pgced::subpic_4_id -width 4 -anchor w

		grid  $w.f.tl0 -row 0 -column 0 -sticky e
		grid  $w.f.tl1 -row 0 -column 1
		grid  $w.f.tl2 -row 0 -column 2
		grid  $w.f.l1  -row 1 -column 0 -sticky e
		grid  $w.f.s1  -row 1 -column 1 -sticky ew
		grid  $w.f.id1 -row 1 -column 2 -sticky sw
		grid  $w.f.l2  -row 2 -column 0 -sticky e
		grid  $w.f.s2  -row 2 -column 1 -sticky ew
		grid  $w.f.id2 -row 2 -column 2 -sticky sw
		grid  $w.f.l3  -row 3 -column 0 -sticky e
		grid  $w.f.s3  -row 3 -column 1 -sticky ew
		grid  $w.f.id3 -row 3 -column 2 -sticky sw
		grid  $w.f.l4  -row 4 -column 0 -sticky e
		grid  $w.f.s4  -row 4 -column 1 -sticky ew
		grid  $w.f.id4 -row 4 -column 2 -sticky sw

		frame  $w.fb
		pack   $w.fb -padx 2 -pady 4 -anchor e
		button $w.fb.dis -text "Disable" -width 7 -command {set ::pgced::subpicdone 2}
		pack   $w.fb.dis -side left -padx 4
		::balloon::set_balloon $w.fb.dis \
				"When a subpic stream is disabled,\nit will not show up anymore.\nSelecting this stream via a menu\nbutton will play stream 1!"
		button $w.fb.ok -text "Set" -width 7 -command {set ::pgced::subpicdone 1}
		pack   $w.fb.ok -side left
		button $w.fb.cancel -text "Cancel" -width 7 -command {set ::pgced::subpicdone 0}
		pack   $w.fb.cancel -side left -padx 4

		update
		set wx [expr {[winfo rootx .pgceditor]+(([winfo width  .pgceditor]-[winfo reqwidth  $w])/2)}]
		set wy [expr {[winfo rooty .pgceditor]+(([winfo height .pgceditor]-[winfo reqheight $w])/2)-25}]
		if {$wx < 10} {set wx 10}
		if {$wy < 22} {set wy 22}
		wm geometry $w +$wx+$wy
		wm resizable $w 0 0
	}
	if {[subst $$streamvar] == "none"} {
		set ::pgced::subpic_1_status 0
		set ::pgced::subpic_2_status 0
		set ::pgced::subpic_3_status 0
		set ::pgced::subpic_4_status 0
	} else {
		set sublist [subst $$streamvar]
		set ::pgced::subpic_1_status [lindex $sublist 0]
		set ::pgced::subpic_2_status [lindex $sublist 1]
		set ::pgced::subpic_3_status [lindex $sublist 2]
		set ::pgced::subpic_4_status [lindex $sublist 3]
	}
	set vattr [::streams::get_video_attribute_value]
	set videodescr [::streams::video_attributes $vattr]
	set enabledcolor  [$w.fb.ok cget -foreground]
	set disabledcolor [$w.fb.ok cget -disabledforeground]
	if {[lindex $videodescr 0] == "4:3"} {
		$w.f.l1  configure -foreground $enabledcolor
		$w.f.id1 configure -foreground $enabledcolor
		$w.f.l2  configure -foreground $disabledcolor
		$w.f.id2 configure -foreground $disabledcolor
		$w.f.l3  configure -foreground $disabledcolor
		$w.f.id3 configure -foreground $disabledcolor
		$w.f.l4  configure -foreground $disabledcolor
		$w.f.id4 configure -foreground $disabledcolor
	} else {
		$w.f.l1  configure -foreground $disabledcolor
		$w.f.id1 configure -foreground $disabledcolor
		$w.f.l2  configure -foreground $enabledcolor
		$w.f.id2 configure -foreground $enabledcolor
		if {[string first "Letterbox" [lindex $videodescr 6]] > -1} {
			$w.f.l3  configure -foreground $enabledcolor
			$w.f.id3 configure -foreground $enabledcolor
		} else {
			$w.f.l3  configure -foreground $disabledcolor
			$w.f.id3 configure -foreground $disabledcolor
		}
		if {[string first "Pan&Scan" [lindex $videodescr 6]] > -1} {
			$w.f.l4  configure -foreground $enabledcolor
			$w.f.id4 configure -foreground $enabledcolor
		} else {
			$w.f.l4  configure -foreground $disabledcolor
			$w.f.id4 configure -foreground $disabledcolor
		}
	}
	wm deiconify $w
	focus $w
	grab set $w

	set ::pgced::subpicdone -1
	tkwait variable ::pgced::subpicdone
	grab release $w
	grab set .pgceditor
	destroy $w
	switch $::pgced::subpicdone {
		1 {set $streamvar [string map {{ } {  }} [format "%2d %2d %2d %2d" $::pgced::subpic_1_status \
				$::pgced::subpic_2_status $::pgced::subpic_3_status $::pgced::subpic_4_status]]}
		2 {set $streamvar "none"}
	}
	return
}
proc validate_subpic_value {streamnum value} {
	set ::pgced::subpic_${streamnum}_id [format 0x%02X [expr {$value+32}]]
}

proc toggle_program {w cell} {
	set prog [$w.f2.fc.c.f.w_${cell}_1 cget -text]

	if {$prog == ""} {
		# create new program
		set newprog 0
		for {set c 1} {$c < $cell} {incr c} {
			set p [$w.f2.fc.c.f.w_${c}_1 cget -text]
			if {$p != ""} {
				set newprog $p
			}
		}
		incr newprog
		for {set c 1} {$c <= $::pgced::__numcells} {incr c} {
			set p [$w.f2.fc.c.f.w_${c}_1 cget -text]
			if {$p != ""} {
				if {$p >= $newprog} {
					incr p
					$w.f2.fc.c.f.w_${c}_1 configure -text $p
				}
			}
		}
		$w.f2.fc.c.f.w_${cell}_1 configure -text $newprog
		incr ::pgced::__numprograms
	} else {
		if {$::pgced::__numprograms == 1} {
			show_msg "Cannot remove the last Program!"
			bell
			return
		}
		# remove program
		$w.f2.fc.c.f.w_${cell}_1 configure -text ""
		set prog 1
		for {set c 1} {$c <= $::pgced::__numcells} {incr c} {
			set p [$w.f2.fc.c.f.w_${c}_1 cget -text]
			if {$p != ""} {
				$w.f2.fc.c.f.w_${c}_1 configure -text $prog
				incr prog
			}
		}
		incr ::pgced::__numprograms -1
	}

	# rebuild program map
	set ::pgced::__programmap {}
	for {set c 1} {$c <= $::pgced::__numcells} {incr c} {
		set p [$w.f2.fc.c.f.w_${c}_1 cget -text]
		if {$p != ""} {
			set ::pgced::__programmap ${::pgced::__programmap}[d2c $c 1]
		}
	}

	# refresh PTT buttons
	if {$::pgced::__pgctype == "pgc"} {
		set error false
		foreach pttpair $::pgced::__PTTlist {
			set pg  [lindex $pttpair 0]
			set ptt [lindex $pttpair 1]
			if {! [info exists pg_ptt($pg)]} {
				set pg_ptt($pg) $ptt
				if {$pg > $::pgced::__numprograms} {
					set error true
				}
			}
		}

		for {set c 1} {$c <= $::pgced::__numcells} {incr c} {
			set p [$w.f2.fc.c.f.w_${c}_1 cget -text]
			if {$p != ""} {
				if {[info exists pg_ptt($p)]} {
					$w.f2.fc.c.f.w_${c}_0 configure -text $pg_ptt($p) -state normal
				} else {
					$w.f2.fc.c.f.w_${c}_0 configure -text "" -state normal
				}
			} else {
				$w.f2.fc.c.f.w_${c}_0 configure -text "" -state disabled
			}
		}

		if {$error} {
			.pgceditor.fb.ptt configure -foreground red
			if {![info exists ::skip_dialog_during_session(chap_ed_warning)]} {
				set txt "You must edit the chapter table to remove or reassign\nchapters pointing to removed programs!\n\nClick on the \"Edit Chapter (PTT) Table\" button."
				set r [show_warning -title "Toggle Program" -icon warning -message $txt -parent .pgceditor \
						-skip "PGC Editor: Toggle PG warning" -type "OK to All|OK" -default ok]
				if {$r == "ok to all"} {
					set ::skip_dialog_during_session(chap_ed_warning) true
				}
			}
		}
	}
	return ::pgced::__numprograms
}


namespace eval ::pgced::ptted {
proc ptt_table_editor {w} {
	set indexes [split [lindex $::editor_cmds_from 1] ,]
	set vts [lindex $indexes 0]
	set pgc [lindex $indexes 1]
	set ttn $::pgcs($vts,$pgc,TTN)
	set numpgcsinttn 0
	for {set p 1} {$p <= $::pgcs($vts,numpgcs)} {incr p} {
		if {$::pgcs($vts,$p,TTN) == $ttn && $::pgcs($vts,$p,numprograms)>0} {
			incr numpgcsinttn
			if {$numpgcsinttn > 1} {
				set txt "The current Title is not a one_sequential title."
				set txt "$txt\nIt contains several PGCs, with chapters in each of them."
				set txt "$txt\nThis editor cannot be used to edit the PTTs of such titles."
				set txt "$txt\nPlease use the PTT Editor from the main PgcEdit window."
				set txt "$txt\n(Select Title -> PTT (Chapter) Editor.)"
				tk_messageBox -message $txt -title "Chapter (PTT) Table editor"
				return false
			}
		}
	}

	set w .ptteditor
	toplevel $w -class Dialog
	wm withdraw $w
	wm title $w "Chapter (PTT) Table editor"
	if {$::config(tk) != "aqua"} {wm transient $w .pgceditor}
	if {$::tcl_platform(platform) == "windows"} {catch {wm attributes $w -toolwindow 1}}
	wm protocol $w WM_DELETE_WINDOW {set ::utils::pttedit_done 0}
	bind $w <Escape> "set ::utils::pttedit_done 0"
	bind $w <Return> "set ::utils::pttedit_done 1"

	frame $w.f
	pack  $w.f -padx 4 -pady 2 -fill both -expand 1

	listbox   $w.f.lb -exportselection 0 -selectmode extended \
			-listvariable ::pgced::ptted::lblist -width 33 -height 16 \
			-activestyle dotbox -yscrollcommand "$w.f.ys set" \
			-font $::config(fixed_width_font)
	scrollbar $w.f.ys -command "$w.f.lb yview"
	grid  $w.f.lb -column 0 -row 0 -sticky nesw
	grid  $w.f.ys -column 1 -row 0 -sticky ns
	grid columnconf $w.f 0 -weight 1000
	grid rowconf    $w.f 0 -weight 1000

	frame $w.b1
	pack  $w.b1 -padx 3 -pady 1 -fill x
	button $w.b1.badd -text "Add chapter" -width 20 -command {
				lappend ::pgced::ptted::ptts [expr {[llength $::pgced::ptted::ptts]+1}]
				lappend ::pgced::ptted::pgs  [expr {[lindex $::pgced::ptted::pgs end end]+1}]
				::pgced::ptted::refresh_listbox
			}
	pack  $w.b1.badd -fill x -padx 1 -side left -expand 1
	button $w.b1.incr -text "Increment programs" -width 20 -command {
				foreach e [.ptteditor.f.lb curselection] {
					lset ::pgced::ptted::pgs $e [expr {[lindex $::pgced::ptted::pgs $e]+1}]
				}
				::pgced::ptted::refresh_listbox
			}
	pack  $w.b1.incr -fill x -padx 1 -side left -expand 1
	frame $w.b2
	pack  $w.b2 -padx 3 -pady 1 -fill x
	button $w.b2.bdel -text "Delete last chapter" -width 20 -command {
				if {[llength $::pgced::ptted::ptts] > 1} {
					set ::pgced::ptted::ptts [lreplace $::pgced::ptted::ptts end end]
					set ::pgced::ptted::pgs  [lreplace $::pgced::ptted::pgs  end end]
					::pgced::ptted::refresh_listbox
				} else bell
			}
	pack  $w.b2.bdel -fill x -padx 1 -side left -expand 1
	button $w.b2.decr -text "Decrement programs" -width 20 -command {
				foreach e [.ptteditor.f.lb curselection] {
					set num [expr {[lindex $::pgced::ptted::pgs $e]-1}]
					if {$num > 0} {
						lset ::pgced::ptted::pgs $e [expr {[lindex $::pgced::ptted::pgs $e]-1}]
					}
				}
				::pgced::ptted::refresh_listbox
			}
	pack  $w.b2.decr -fill x -padx 1 -side left -expand 1

	button $w.eq -text "Set selected Programs = PTTs" -command {
				foreach e [.ptteditor.f.lb curselection] {
					lset ::pgced::ptted::pgs $e [expr {$e+1}]
				}
				::pgced::ptted::refresh_listbox
			}
	pack  $w.eq -fill x -padx 4 -pady 1

	frame $w.b3
	pack  $w.b3 -padx 3 -pady 1 -fill x
	label $w.b3.l -text "Set selected Programs to"
	pack  $w.b3.l -side left
	entry $w.b3.e -textvariable ::pgced::ptted::entry -width 3 \
			-validatecommand "::pgced::value_entry_validate 1 255 {%d} {%P}"
	pack  $w.b3.e -side left
	bind  $w.b3.e <Return> ::pgced::ptted::apply_pg_num
	frame $w.b3.pad -width 4
	pack  $w.b3.pad -side left
	button $w.b3.b -text "Apply" -width 6 -command ::pgced::ptted::apply_pg_num
	pack  $w.b3.b -fill x -padx 1 -side left -expand 1

	# buttons
	frame  $w.fb
	pack   $w.fb -padx 2 -pady 4 -anchor e
	button $w.fb.ok -text "OK" -width 10 -command {set ::utils::pttedit_done 1}
	pack   $w.fb.ok -side left -padx 2
	button $w.fb.cancel -text "Cancel" -width 10 -command {set ::utils::pttedit_done 0}
	pack   $w.fb.cancel -side left -padx 2
	if {$::config(platform) == "mac"} {
		pack [frame $w.fb.macpad] -side left -padx 6
	}

	set ::pgced::ptted::ptts {}
	set ::pgced::ptted::pgs {}
	foreach pttpair $::pgced::__PTTlist {
		lappend ::pgced::ptted::ptts [lindex $pttpair 1]
		lappend ::pgced::ptted::pgs  [lindex $pttpair 0]
	}
	refresh_listbox

	update
	wm minsize $w [winfo reqwidth $w] [winfo reqheight $w]
	set wx [expr {[winfo rootx .]+(([winfo width  .]-[winfo reqwidth  $w])/2)}]
	set wy [expr {[winfo rooty .]+(([winfo height .]-[winfo reqheight $w])/2)-25}]
	if {$wx < 10} {set wx 10}
	if {$wy < 22} {set wy 22}
	wm geometry $w +$wx+$wy
	wm deiconify $w
	focus $w
	grab set $w

	tkwait variable ::utils::pttedit_done

	unset ::pgced::ptted::lblist
	destroy $w
	grab set .pgceditor
	if {$::utils::pttedit_done} {
		if {$::pgced::__PTTlist != ""} {
			set ttu [lindex [lindex $::pgced::__PTTlist 0] 2]
		} else {
			set ttu $::pgcs($::pgced::__index,TTU)
		}
		set ::pgced::__PTTlist {}
		set error false
		for {set e 0} {$e < [llength $::pgced::ptted::ptts]} {incr e} {
			set pg  [lindex $::pgced::ptted::pgs $e]
			set ptt [lindex $::pgced::ptted::ptts $e]
			lappend ::pgced::__PTTlist [list $pg $ptt $ttu]
			if {$pg > [string length $::pgced::__programmap]} {
				set error true
			}
		}
		if {$error} {
			.pgceditor.fb.ptt configure -foreground red
			set txt "You must remove or reassign the chapters\npointing to out of range programs!\n\nClick on the \"Edit Chapter (PTT) Table\" button."
			show_warning -title "Remove Cell" -icon warning \
				-message $txt -parent . -skip "PGC Editor: edit PTT table warning"
		} else {
			.pgceditor.fb.ptt configure -foreground black
		}
		return true
	} else {
		return false
	}
}
proc refresh_listbox {} {
	set sel [.ptteditor.f.lb curselection]
	set ::pgced::ptted::lblist {}
	set idx 0
	foreach ptt $::pgced::ptted::ptts {
		lappend ::pgced::ptted::lblist " Chapter (PTT) [format %2d $ptt] : Program [format %3d [lindex $::pgced::ptted::pgs $idx]] "
		incr idx
	}
	foreach s $sel {
		.ptteditor.f.lb selection set $s
	}
}
proc apply_pg_num {} {
	if {$::pgced::ptted::entry == "" || $::pgced::ptted::entry == 0} {
		bell
	} else {
		foreach e [.ptteditor.f.lb curselection] {
			lset ::pgced::ptted::pgs $e $::pgced::ptted::entry
		}
		refresh_listbox
	}
}
}

proc cell_type_editor {cell} {

	set w ".celltypeeditor"
	if {![winfo exists $w]} {
		toplevel $w -class Dialog
		wm withdraw $w
		wm protocol $w WM_DELETE_WINDOW {set ::pgced::celltypedone 0}
		bind $w <Escape> "set ::pgced::celltypedone 0"
		bind $w <Return> "set ::pgced::celltypedone 1"
		if {$::config(tk) != "aqua"} {wm transient $w .pgceditor}
		if {$::tcl_platform(platform) == "windows"} {catch {wm attributes $w -toolwindow 1}}
		wm title $w "Cell Type Flags"

		frame $w.f1 -bd 2 -relief groove
		pack  $w.f1 -fill x -padx 4 -pady 4

		label $w.f1.l0 -text "Byte 0:"
		pack  $w.f1.l0 -anchor w

		checkbutton $w.f1.c0 -text "Seamless angle change   (1)" -anchor w \
				-bd $::config(min_bd) -variable ::pgced::celltype_0 -offvalue 0 -onvalue 1
		pack $w.f1.c0 -fill x
		checkbutton $w.f1.c1 -text "STC (SCR) discontinuity  (2)" -anchor w \
				-bd $::config(min_bd) -variable ::pgced::celltype_1 -offvalue 0 -onvalue 2
		pack $w.f1.c1 -fill x
		checkbutton $w.f1.c2 -text "Interleaved allocation  (4)" -anchor w \
				-bd $::config(min_bd) -variable ::pgced::celltype_2 -offvalue 0 -onvalue 4
		pack $w.f1.c2 -fill x
		checkbutton $w.f1.c3 -text "Seamless playback joint  (8)" -anchor w \
				-bd $::config(min_bd) -variable ::pgced::celltype_3 -offvalue 0 -onvalue 8
		pack $w.f1.c3 -fill x

		frame $w.f1.ft -borderwidth 2 -relief groove
		pack  $w.f1.ft -fill x -padx 4 -pady 2

		checkbutton $w.f1.ft.c4 -text "Angle block (Titles only)  (16)" -anchor w \
				-bd $::config(min_bd) -variable ::pgced::celltype_4 -offvalue 0 -onvalue 16 \
				-command "::pgced::validate_cell_type_editor $w"
		pack $w.f1.ft.c4 -fill x
		radiobutton $w.f1.ft.r1 -variable ::pgced::celltype_67 -value 64 -anchor w \
				-text "first of angle block  (64)" -pady 0 -bd $::config(min_bd)
		pack  $w.f1.ft.r1 -fill x
		radiobutton $w.f1.ft.r2 -variable ::pgced::celltype_67 -value 128 -anchor w \
				-text "middle of angle block  (128)" -pady 0 -bd $::config(min_bd)
		pack  $w.f1.ft.r2 -fill x
		radiobutton $w.f1.ft.r3 -variable ::pgced::celltype_67 -value 192 -anchor w \
				-text "last of angle block  (192)" -pady 0 -bd $::config(min_bd)
		pack  $w.f1.ft.r3 -fill x

		frame $w.f2 -bd 2 -relief groove
		pack  $w.f2 -fill x -padx 4

		label $w.f2.l1 -text "Byte 1:"
		pack  $w.f2.l1 -anchor w

		checkbutton $w.f2.c0 -text "Access restricted  (32)" -anchor w \
				-bd $::config(min_bd) -variable ::pgced::celltype_13 -offvalue 0 -onvalue 32
		pack $w.f2.c0 -fill x
		checkbutton $w.f2.c1 -text "VOBU Still  (64)" -anchor w \
				-bd $::config(min_bd) -variable ::pgced::celltype_14 -offvalue 0 -onvalue 64
		pack $w.f2.c1 -fill x

		frame  $w.fb
		pack   $w.fb -anchor e -padx 4 -pady 6
		button $w.fb.ok -text "OK" -width 10 -command {set ::pgced::celltypedone 1}
		pack   $w.fb.ok -side left -padx 4
		button $w.fb.cancel -text "Cancel" -width 10 -command {set ::pgced::celltypedone 0}
		pack   $w.fb.cancel -side left

		update
		set wx [expr {[winfo rootx .pgceditor]+(([winfo width  .pgceditor]-[winfo reqwidth  $w])/2)}]
		set wy [expr {[winfo rooty .pgceditor]+(([winfo height .pgceditor]-[winfo reqheight $w])/2)-25}]
		if {$wx < 10} {set wx 10}
		if {$wy < 22} {set wy 22}
		wm geometry $w +$wx+$wy
		wm resizable $w 0 0
	}
	set parentbutton .pgceditor.f2.fc.c.f.w_${cell}_3
	set ct [$parentbutton cget -text]
	set ct0 [lindex $ct 0]
	set ct1 [lindex $ct 1]
	set ::pgced::celltype_0  [expr {$ct0 & 1}]
	set ::pgced::celltype_1  [expr {$ct0 & 2}]
	set ::pgced::celltype_2  [expr {$ct0 & 4}]
	set ::pgced::celltype_3  [expr {$ct0 & 8}]
	set ::pgced::celltype_4  [expr {$ct0 & 16}]
	set ::pgced::celltype_67 [expr {$ct0 & 192}]
	set ::pgced::celltype_13 [expr {$ct1 & 32}]
	set ::pgced::celltype_14 [expr {$ct1 & 64}]

	validate_cell_type_editor $w

	wm deiconify $w
	focus $w
	grab set $w

	set ::pgced::celltypedone -1
	tkwait variable ::pgced::celltypedone
	grab release $w
	grab set .pgceditor
	wm withdraw $w
	if {$::pgced::celltypedone} {
		set  r1 $::pgced::celltype_0
		incr r1 $::pgced::celltype_1
		incr r1 $::pgced::celltype_2
		incr r1 $::pgced::celltype_3
		incr r1 $::pgced::celltype_4
		incr r1 $::pgced::celltype_67

		set  r2 $::pgced::celltype_13
		incr r2 $::pgced::celltype_14

		if {$r2 < 10} {
			$parentbutton configure -text "$r1    $r2"
		} else {
			$parentbutton configure -text "$r1  $r2"
		}

		if {$::pgced::celltype_3 == 8} {
			set ::pgced::var_${cell}_4 1
		} else {
			set ::pgced::var_${cell}_4 0
		}
		validate_seamless_flag $cell
		refresh_angle_values
		return [list $r1 $r2]
	} else {
		return ""
	}
}
proc validate_cell_type_editor {w} {
	if {$::pgced::celltype_4} {
		set state normal
		if {$::pgced::celltype_67 == 0} {
			set ::pgced::celltype_67 64
		}
	} else {
		set state disabled
		set ::pgced::celltype_67 0
	}
	$w.f1.ft.r1 configure -state $state
	$w.f1.ft.r2 configure -state $state
	$w.f1.ft.r3 configure -state $state
}

proc set_seamless_flag {cell} {
	set v ::pgced::var_${cell}_4
	set flag [subst \$$v]
	set typebutton .pgceditor.f2.fc.c.f.w_${cell}_3
	set typevals [$typebutton cget -text]
	set type  [lindex $typevals 0]
	set type2 [lindex $typevals 1]
	if {$flag} {
		set type [expr {$type | 8}]
	} else {
		set type [expr {$type & 0XF7}]
	}
	if {$type2 < 10} {set type2 "  $type2"}
	$typebutton configure -text "$type  $type2"

	validate_seamless_flag $cell
}

# ::pgced::reassign_VOB_ID .pgceditor 1 1 1
proc reassign_VOB_ID {parent cur_cellnum cur_vobid cur_cellid} {
	if {! [info exists ::pgced::rebuild_time_map]} {
		set ::pgced::rebuild_time_map 1
	}

	set cur_type  [lindex $::editor_cmds_from 0]
	set cur_index [lindex $::editor_cmds_from 1]
	set cur_vts   [lindex [split $cur_index ","] 0]

	# Search for all VOB IDs in the current VTS, and for all Cell IDs in each VOB IDs
	# and build the list of vobids, and the lists of cellids for all vobids.
	# Also, for each pair VOB/Cell ID, keep track of the belonging PGC's cell.
	set all_vobids ""
	if {$cur_type == "menu"} {
		for {set lu 1} {$lu <= $::menus($cur_vts,numlus)} {incr lu} {
			for {set pgc 1} {$pgc <= $::menus($cur_vts,$lu,numpgcs)} {incr pgc} {
				set cellpostable $::menus($cur_vts,$lu,$pgc,cellpostable)
				for {set cell 1} {$cell <= $::menus($cur_vts,$lu,$pgc,numcells)} {incr cell} {
					set cellidx [expr {($cell - 1) * 4}]
					set vobid  [get_int cellpostable $cellidx 2]
					set cellid [get_int cellpostable [expr {$cellidx + 3}] 1]
					if {[lsearch $all_vobids $vobid] == -1} {
						lappend all_vobids $vobid
						set all_cellids($vobid) ""
					}
					if {[lsearch $all_cellids($vobid) $cellid] == -1} {
						lappend all_cellids($vobid) $cellid
					}
					set src_pgc_cell($vobid,$cellid) [list "menu" $lu $pgc $cell]
				}
			}
		}
	} else {
		for {set pgc 1} {$pgc <= $::pgcs($cur_vts,numpgcs)} {incr pgc} {
			set cellpostable $::pgcs($cur_vts,$pgc,cellpostable)
			for {set cell 1} {$cell <= $::pgcs($cur_vts,$pgc,numcells)} {incr cell} {
				set cellidx [expr {($cell - 1) * 4}]
				set vobid  [get_int cellpostable $cellidx 2]
				set cellid [get_int cellpostable [expr {$cellidx + 3}] 1]
				if {[lsearch $all_vobids $vobid] == -1} {
					lappend all_vobids $vobid
					set all_cellids($vobid) ""
				}
				if {[lsearch $all_cellids($vobid) $cellid] == -1} {
					lappend all_cellids($vobid) $cellid
				}
				set src_pgc_cell($vobid,$cellid) [list "pgc" 0 $pgc $cell]
			}
		}
	}
	set all_vobids [lsort -integer $all_vobids]
	foreach vobid $all_vobids {
		set all_cellids($vobid) [lsort -integer $all_cellids($vobid)]
	}
	set lastvobcelllist [::blankcell::LastVOBCellIsBlankCell $cur_type $cur_vts]
	if {![info exists all_cellids([lindex $lastvobcelllist 0])]} {
		set new_cell_already_exists $lastvobcelllist
		set lastvobcelllist {0 0 ""}
	}

	# GUI
	set ::pgced::reassign_cell_mode 1	;# 1=use existing cell, 2=create new blank cell

	set w ".assignvobid"
	if {[winfo exists $w]} {destroy $w}
	toplevel $w -class Dialog
	wm withdraw $w
	wm protocol $w WM_DELETE_WINDOW {set ::pgced::vobiddone 0}
	bind $w <Escape> "set ::pgced::vobiddone 0"
	bind $w <Return> "set ::pgced::vobiddone 1"
	if {$::config(tk) != "aqua"} {wm transient $w $parent}
	if {$::tcl_platform(platform) == "windows"} {catch {wm attributes $w -toolwindow 1}}
	wm title $w "Assign VOB/Cell ID for PGC Cell $cur_cellnum"

	set txt "Assing another VOB ID and/or Cell ID to Cell $cur_cellnum of the"
	set txt "$txt\ncurrent PGC."
	set txt "$txt\nWARNING: The old VOB/Cell ID will be unavailable"
	set txt "$txt\nin the future, unless it is reused in another PGC !"
	if {$cur_type == "pgc"} {
		set txt "$txt\n(Hint: Use the \"New 'Play All' Title\" utility to 'backup'"
		set txt "$txt\nthe old VOB Cell references before using this function.)"
	}
	label $w.lab -text $txt -justify left -anchor w
	pack  $w.lab -fill x -padx 2

	frame $w.f1 -bd 2 -relief groove
	pack  $w.f1 -fill x -padx 4
	frame $w.f1.tf
	pack  $w.f1.tf -fill x
	radiobutton $w.f1.tf.rb -text "Pick an existing VOB/Cell ID:" \
			-variable ::pgced::reassign_cell_mode -value 1 -anchor w \
			-command "::pgced::validate_reassign_gui $w"
	pack  $w.f1.tf.rb -side left
	if {$::config(use_preview)} {
		button $w.f1.tf.pv -text "Preview" -pady 0 -command \
				"::preview::preview_vcid [lindex $::editor_cmds_from 0] $cur_vts \$::pgced::new_vobid \$::pgced::new_cellid"
		pack  $w.f1.tf.pv -side right -padx 4 -pady 2
	}

	frame $w.f1.l1
	pack  $w.f1.l1 -fill x -padx 5
	label $w.f1.l1.lab -text "New VOB ID:" -width 17 -anchor w
	pack  $w.f1.l1.lab -side left
	entry $w.f1.l1.e -width 4 -state readonly -justify right -bd 0 -relief flat \
			-textvariable ::pgced::new_vobid
	pack  $w.f1.l1.e -side left
	scale $w.f1.s1 -showvalue false -from 0 -to [expr {[llength $all_vobids]-1}] \
			-orient horizontal -variable ::pgced::vobid_scale \
			-command {::pgced::validate_vobid_scale .assignvobid $all_vobids all_cellids}
	pack  $w.f1.s1 -padx 2 -fill x

	frame $w.f1.l2
	pack  $w.f1.l2 -fill x -padx 5
	label $w.f1.l2.lab -text "New Cell ID:" -width 17 -anchor w
	pack  $w.f1.l2.lab -side left
	entry $w.f1.l2.e -width 3 -state readonly -justify right -bd 0 -relief flat \
			-textvariable ::pgced::new_cellid
	pack  $w.f1.l2.e -side left
	scale $w.f1.s2 -showvalue false -from 0 -to [expr {[llength $all_cellids($cur_vobid)]-1}] \
			-orient horizontal -variable ::pgced::cellid_scale \
			-command "::pgced::validate_cellid_scale \{$all_cellids($cur_vobid)\}"
	pack  $w.f1.s2 -padx 2 -fill x
	if {[lindex $lastvobcelllist 0] == 0} {
		set txt " (Not available)"
		set state disabled
		set cmd bell
	} else {
		set txt "(VOB/Cell ID [lindex $lastvobcelllist 0]/[lindex $lastvobcelllist 1])"
		set state normal
		set tmp_vid [lindex $lastvobcelllist 0]
		set tmp_cid [lindex $lastvobcelllist 1]
		set cmd "set ::pgced::vobid_scale [lsearch $all_vobids $tmp_vid]; \
				 set ::pgced::cellid_scale [lsearch $all_cellids($tmp_vid) $tmp_cid]; \
				 set ::pgced::new_vobid  $tmp_vid; set ::pgced::new_cellid $tmp_cid"
	}
	button $w.f1.reuse -text "Use last blank cell in VOB $txt" -state $state -command $cmd
	pack  $w.f1.reuse -fill x -padx 4 -pady 2
	::balloon::set_balloon $w.f1.reuse "To blank a PGC Cell, use this button whenever\npossible, or create a new blank VOB cell."
	label $w.f1.lab -text "Note: The VOB file is not modified by this operation." -anchor w
	pack  $w.f1.lab -fill x -padx 2

	frame $w.f2 -bd 2 -relief groove
	pack  $w.f2 -fill x -padx 4 -pady 4
	radiobutton $w.f2.rb -text "Create a new blank VOB Cell:" \
			-variable ::pgced::reassign_cell_mode -value 2 -anchor w \
			-command "::pgced::validate_reassign_gui $w"
	pack  $w.f2.rb -fill x
	set txt "A new blank cell will be appended to the VOB file"
	set txt "$txt\nand its VOB ID and Cell ID will be assigned to Cell $cur_cellnum"
	set txt "$txt\nof the current PGC."
	label $w.f2.lab -text $txt -justify left -anchor w
	pack  $w.f2.lab -fill x -padx 4
	checkbutton $w.f2.cb -text "Copy original VOB to Backup folder" \
			-variable ::pgced::copy_vob_to_backup -anchor w
	pack  $w.f2.cb -fill x -padx 2
	set ::pgced::copy_vob_to_backup 1
	set txt "WARNING: The VOB file will be modified immediately."
	set txt "$txt\nTHE DVD WILL BE SAVED after this operation, to"
	set txt "$txt\nkeep track of the new VOB cell in the IFOs!"
	label $w.f2.lab2 -text $txt -justify left -anchor w -bd 2 -relief groove
	pack  $w.f2.lab2 -fill x -padx 4 -pady 2
	checkbutton $w.f2.notm -text "Rebuild the Time Map of the current Title\n(Recommended, but slow.)" \
			-variable ::pgced::rebuild_time_map -anchor w -justify left
	if {$::pgced::__pgctype == "pgc"} {
		pack  $w.f2.notm -fill x -padx 2
	}

	frame  $w.fb
	pack   $w.fb -anchor e -padx 4 -pady 4
	button $w.fb.ok -text "OK" -width 10 -command {set ::pgced::vobiddone 1}
	pack   $w.fb.ok -side left -padx 4
	button $w.fb.cancel -text "Cancel" -width 10 -command {set ::pgced::vobiddone 0}
	pack   $w.fb.cancel -side left

	update
	set wx [expr {[winfo rootx $parent]+(([winfo width  $parent]-[winfo reqwidth  $w])/2)}]
	set wy [expr {[winfo rooty $parent]+(([winfo height $parent]-[winfo reqheight $w])/2)-25}]
	if {$wx < 10} {set wx 10}
	if {$wy < 22} {set wy 22}
	wm geometry $w +$wx+$wy
	wm resizable $w 0 0

	validate_reassign_gui $w

	wm deiconify $w
	focus $w
	grab set $w

	set ::pgced::vobid_scale  [lsearch $all_vobids $cur_vobid]
	set ::pgced::cellid_scale [lsearch $all_cellids($cur_vobid) $cur_cellid]

	set ::pgced::vobiddone -1
	tkwait variable ::pgced::vobiddone
	grab release $w
	if {$parent != "."} {
		grab set $parent
	}
	wm withdraw $w
	if {!$::pgced::vobiddone} return
	set new_vobid  $::pgced::new_vobid
	set new_cellid $::pgced::new_cellid
	set new_lba [::blankcell::numSectorsInVOB $cur_type $cur_vts]
	if {$::pgced::reassign_cell_mode == 1 && $new_vobid == $cur_vobid && $new_cellid == $cur_cellid} return

	# Pick an existing VOB/Cell ID
	if {$::pgced::reassign_cell_mode == 1} {
		# Use the new VOB ID $new_vobid and the new Cell ID $new_cellid.
		set src $src_pgc_cell($new_vobid,$new_cellid)
		set src_type [lindex $src 0]
		set src_lu   [lindex $src 1]
		set src_pgc  [lindex $src 2]
		set src_cell [lindex $src 3]

		# get the relevant entry in the source cell playback table
		if {$src_type == "menu"} {
			set src_index "$cur_vts,$src_lu,$src_pgc"
		} else {
			set src_index "$cur_vts,$src_pgc"
		}
		set src_cellpbtable [subst \$::${src_type}s($src_index,cellpbtable)]
		set src_cellpbidx   [expr {($src_cell - 1) * 24}]
		set src_cellpbentry [string range $src_cellpbtable $src_cellpbidx [expr {$src_cellpbidx + 23}]]

		if {$cur_type == "menu"} {
			if {[info exists ::buttons::butsinfo(menu,$cur_vts,$cur_vobid,$cur_cellid,numbuts)]} {
				incr ::menus($cur_index,numbuttons) \
						[expr {$::buttons::butsinfo(menu,$cur_vts,$cur_vobid,$cur_cellid,numbuts) * -1}]
			}
			if {[info exists ::buttons::butsinfo(menu,$cur_vts,$new_vobid,$new_cellid,numbuts)]} {
				incr ::menus($cur_index,numbuttons) \
						$::buttons::butsinfo(menu,$cur_vts,$new_vobid,$new_cellid,numbuts)
			}
		} elseif {[info exists ::pgcs($cur_index,numbuttons)]} {
			if {[info exists ::buttons::butsinfo(pgc,$cur_vts,$cur_vobid,$cur_cellid,numbuts)]} {
				incr ::pgcs($cur_index,numbuttons) \
						[expr {$::buttons::butsinfo(pgc,$cur_vts,$cur_vobid,$cur_cellid,numbuts) * -1}]
			}
			if {[info exists ::buttons::butsinfo(pgc,$cur_vts,$new_vobid,$new_cellid,numbuts)]} {
				incr ::pgcs($cur_index,numbuttons) \
						$::buttons::butsinfo(pgc,$cur_vts,$new_vobid,$new_cellid,numbuts)
			}
		}
		set rebuild_time_map 1
	} else {
		# Append a blank cell to the VOB
		set resultlist [::blankcell::AppendBlankCell $cur_type $cur_vts $::pgced::copy_vob_to_backup]

		set new_vobid [lindex $resultlist 0]
		set src_cellpbentry [lindex $resultlist 2]
		if {$new_vobid == 0} return
		if {$new_vobid < 0} {
			set txt "Error appending new Cell to VOB file:\n"
			set txt "$txt\n$src_cellpbentry"
			tk_messageBox -title "Append Blank Cell to VOB" -icon error \
					-message $txt -parent $parent
			return
		}
		set new_cellid 1

		if {$cur_type == "menu"} {
			incr ::menus($cur_index,numbuttons) \
					[expr {$::buttons::butsinfo(menu,$cur_vts,$cur_vobid,$cur_cellid,numbuts) * -1}]
			set ::buttons::butsinfo(menu,$cur_vts,numvobids) $new_vobid
			set ::buttons::butsinfo(menu,$cur_vts,$::buttons::butsinfo(menu,$cur_vts,numvobids),numcellids) 1
			set ::buttons::butsinfo(menu,$cur_vts,$new_vobid,$new_cellid,navpack) $new_lba
			set ::buttons::butsinfo(menu,$cur_vts,$new_vobid,$new_cellid,numbuts) 0
			unset -nocomplain ::buttons::butsinfo(menu,$cur_vts,$new_vobid,$new_cellid,numgroups)
			unset -nocomplain ::buttons::butsinfo(menu,$cur_vts,$new_vobid,$new_cellid,forceselectbutn)
			unset -nocomplain ::buttons::butsinfo(menu,$cur_vts,$new_vobid,$new_cellid,forceactionbutn)
			unset -nocomplain ::buttons::butsinfo(menu,$cur_vts,$new_vobid,$new_cellid,1,grouptype)
			unset -nocomplain ::buttons::butsinfo(menu,$cur_vts,$new_vobid,$new_cellid,2,grouptype)
			unset -nocomplain ::buttons::butsinfo(menu,$cur_vts,$new_vobid,$new_cellid,3,grouptype)
		} else {
			if {[info exists ::pgcs($cur_index,numbuttons)]} {
				incr ::pgcs($cur_index,numbuttons) \
						[expr {$::buttons::butsinfo(pgc,$cur_vts,$cur_vobid,$cur_cellid,numbuts) * -1}]
			}
			set ::buttons::butsinfo(pgc,$cur_vts,numvobids) $new_vobid
			set ::buttons::butsinfo(pgc,$cur_vts,$::buttons::butsinfo(pgc,$cur_vts,numvobids),numcellids) 1
			set ::buttons::butsinfo(pgc,$cur_vts,$new_vobid,$new_cellid,navpack) $new_lba
			set ::buttons::butsinfo(pgc,$cur_vts,$new_vobid,$new_cellid,numbuts) 0
			unset -nocomplain ::buttons::butsinfo(pgc,$cur_vts,$new_vobid,$new_cellid,numgroups)
			unset -nocomplain ::buttons::butsinfo(pgc,$cur_vts,$new_vobid,$new_cellid,forceselectbutn)
			unset -nocomplain ::buttons::butsinfo(pgc,$cur_vts,$new_vobid,$new_cellid,forceactionbutn)
			unset -nocomplain ::buttons::butsinfo(pgc,$cur_vts,$new_vobid,$new_cellid,1,grouptype)
			unset -nocomplain ::buttons::butsinfo(pgc,$cur_vts,$new_vobid,$new_cellid,2,grouptype)
			unset -nocomplain ::buttons::butsinfo(pgc,$cur_vts,$new_vobid,$new_cellid,3,grouptype)
		}
		set rebuild_time_map $::pgced::rebuild_time_map
	}
	# copy the cell type and the cell command number from the original cell
	set cur_cellpbidx [expr {($cur_cellnum - 1) * 24}]
	set cur_celltype [expr {([get_int ::pgced::__cellpbtable $cur_cellpbidx 1] & 0xF7) | 2}] ;# add SRC Discontinuity flag, and remove Seamless Joint
	set cur_cmdnum [get_int ::pgced::__cellpbtable [expr {$cur_cellpbidx + 3}] 1]
	set_int src_cellpbentry 0 $cur_celltype 1
	set_int src_cellpbentry 3 $cur_cmdnum 1

	# replace the new entry in the current cell playback table
	set ::pgced::__cellpbtable "[string range $::pgced::__cellpbtable 0 [expr {$cur_cellpbidx-1}]]$src_cellpbentry[string range $::pgced::__cellpbtable [expr {$cur_cellpbidx+24}] end]"

	# set the VOB ID and Cell ID in the Cell Position table
	set cur_cellposidx [expr {($cur_cellnum - 1) * 4}]
	set_int ::pgced::__cellpostable $cur_cellposidx $new_vobid 2
	set_int ::pgced::__cellpostable [expr {$cur_cellposidx + 3}] $new_cellid 1

	# Set the STC Discontinuity flag in the modified Cell entry, and clear the Seamless flag
	set idx [expr {($cur_cellnum-1) * 24}]
	set_int ::pgced::__cellpbtable $idx [expr {([get_int ::pgced::__cellpbtable $idx 1] | 2) & 0xF7}] 1
	# Set the STC Discontinuity flag in the NEXT Cell entry, and clear the Seamless flag
	if {$::pgced::__numcells > $cur_cellnum} {
		set idx [expr {$cur_cellnum * 24}]
		set_int ::pgced::__cellpbtable $idx [expr {([get_int ::pgced::__cellpbtable $idx 1] | 2) & 0xF7}] 1
	}

	# reload the new tables in the editor
	set total_time [build_cell_table $parent]

	# correct the total playback time
	set ::pgced::__pgcheader "[string range $::pgced::__pgcheader 0 3]$total_time[string range $::pgced::__pgcheader 8 end]"
	set ::pgced::pbtimetxt [BCDtime $total_time true]
	set ::pgced::__pgcpbtime $total_time

	# we need to rebuild the time map of this PGC
	if {$rebuild_time_map} {
		set ::pgced::__need_new_tmap true
	}

	# If cell replaced, then we must remember the current PGC tables,
	# the simplest way to do this is to close the PGC Editor window, with an OK signal.
	if {$::pgced::reassign_cell_mode == 2} {
		::main::dvd_modified true false
		set ::pgced::done 1
		after 500 {
			tk_messageBox -title "Assign VOB/Cell ID" -icon warning \
					-message "The DVD will be saved now..." -parent . -skip the_dvd_will_be_saved
			::main::Write_DVD false
		}
	}

	# refresh PGC selector to show the new number of menubuttons
	::main::init_tree_menu_no_move
	return
}
proc validate_reassign_gui {w} {
	if {$::pgced::reassign_cell_mode == 1} {
		catch {::ced::scale_state $w.f1.s1 normal}
		catch {::ced::scale_state $w.f1.s2 normal}
		catch {$w.f1.tf.pv configure -state normal}
		catch {
			if {[$w.f1.reuse cget -command] != "bell"} {
				$w.f1.reuse configure -state normal
			}
		}
		$w.f2.cb configure -state disabled
		$w.f2.lab2 configure -foreground [$w.f1.lab cget -foreground] \
				-background [$w.f1.lab cget -background]
		catch {$w.f2.notm configure -state disabled}
	} else {
		catch {::ced::scale_state $w.f1.s1 disabled}
		catch {::ced::scale_state $w.f1.s2 disabled}
		catch {$w.f1.tf.pv configure -state disabled}
		catch {$w.f1.reuse configure -state disabled}
		$w.f2.cb configure -state normal
		$w.f2.lab2 configure -foreground red -background gray90
		catch {$w.f2.notm configure -state normal}
	}
}
proc validate_vobid_scale {w all_vobids all_cellids_upvar val} {
	upvar $all_cellids_upvar all_cellids
	set ::pgced::new_vobid [lindex $all_vobids $val]
	$w.f1.s2 configure -to [expr {[llength $all_cellids($::pgced::new_vobid)]-1}] \
			-command "::pgced::validate_cellid_scale \{$all_cellids($::pgced::new_vobid)\}"
	set pos [lsearch $all_cellids($::pgced::new_vobid) $::pgced::new_cellid]
	if {$pos == -1} {
		$w.f1.s2 set 0
		set ::pgced::new_cellid [lindex $all_cellids($::pgced::new_vobid) 0]
	} else {
		$w.f1.s2 set $pos
	}
}
proc validate_cellid_scale {all_cellids val} {
	set ::pgced::new_cellid [lindex $all_cellids $val]
}

# ::pgced::create_new_cell .pgceditor
proc create_new_cell {parent} {
	if {! [info exists ::pgced::rebuild_time_map]} {
		set ::pgced::rebuild_time_map 1
	}

	set utiltitle "Create new cell"
	set cur_type  [lindex $::editor_cmds_from 0]
	set cur_index [lindex $::editor_cmds_from 1]
	set cur_vts   [lindex [split $cur_index ","] 0]

	# Search for all VOB IDs in the current VTS, and for all Cell IDs in each VOB IDs
	# and build the list of vobids, and the lists of cellids for all vobids.
	# Also, for each pair VOB/Cell ID, keep track of the belonging PGC's cell.
	set all_vobids ""
	if {$cur_type == "menu"} {
		for {set lu 1} {$lu <= $::menus($cur_vts,numlus)} {incr lu} {
			for {set pgc 1} {$pgc <= $::menus($cur_vts,$lu,numpgcs)} {incr pgc} {
				set cellpostable $::menus($cur_vts,$lu,$pgc,cellpostable)
				for {set cell 1} {$cell <= $::menus($cur_vts,$lu,$pgc,numcells)} {incr cell} {
					set cellidx [expr {($cell - 1) * 4}]
					set vobid  [get_int cellpostable $cellidx 2]
					set cellid [get_int cellpostable [expr {$cellidx + 3}] 1]
					if {[lsearch $all_vobids $vobid] == -1} {
						lappend all_vobids $vobid
						set all_cellids($vobid) ""
					}
					if {[lsearch $all_cellids($vobid) $cellid] == -1} {
						lappend all_cellids($vobid) $cellid
					}
					set src_pgc_cell($vobid,$cellid) [list "menu" $lu $pgc $cell]
				}
			}
		}
	} else {
		for {set pgc 1} {$pgc <= $::pgcs($cur_vts,numpgcs)} {incr pgc} {
			set cellpostable $::pgcs($cur_vts,$pgc,cellpostable)
			for {set cell 1} {$cell <= $::pgcs($cur_vts,$pgc,numcells)} {incr cell} {
				set cellidx [expr {($cell - 1) * 4}]
				set vobid  [get_int cellpostable $cellidx 2]
				set cellid [get_int cellpostable [expr {$cellidx + 3}] 1]
				if {[lsearch $all_vobids $vobid] == -1} {
					lappend all_vobids $vobid
					set all_cellids($vobid) ""
				}
				if {[lsearch $all_cellids($vobid) $cellid] == -1} {
					lappend all_cellids($vobid) $cellid
				}
				set src_pgc_cell($vobid,$cellid) [list "pgc" 0 $pgc $cell]
			}
		}
	}
	if {$all_vobids != ""} {
		set all_vobids [lsort -integer $all_vobids]
		foreach vobid $all_vobids {
			set all_cellids($vobid) [lsort -integer $all_cellids($vobid)]
		}
		set cur_vobid  [lindex $all_vobids 0]
		set cur_cellid [lindex $all_cellids($cur_vobid) 0]

		set lastvobcelllist [::blankcell::LastVOBCellIsBlankCell $cur_type $cur_vts]
		if {[lindex $lastvobcelllist 0] > [lindex $all_vobids [expr [llength $all_vobids]-1]]} {
			set lastvobcelllist [list 0 0 ""]
		}
	}

	# GUI
	if {$cur_vts == 0} {
		set vobfilename "VIDEO_TS.VOB"
	} else {
		if {$cur_type == "menu"} {
			set vobfilename "VTS_[format %02d $cur_vts]_0.VOB"
		} else {
			set vobfilename "VTS_[format %02d $cur_vts]_1.VOB"
			for {set i 1} {$i < 10} {incr i} {
				if {[file exists [file join $::dvddir "VTS_[format %02d $cur_vts]_$i.VOB"]]} {
					set vobfilename "VTS_[format %02d $cur_vts]_$i.VOB"
				}
			}
		}
	}
	if {! [file exists [file join $::backupdir $vobfilename]] && \
				[file exists [file join $::dvddir $vobfilename]]} {
		set ::pgced::backup_vob 1
		set backuptxt "Backup VOB file?  (Recommended but might be slow.)"
	} else {
		set ::pgced::backup_vob 0
		if {[file exists [file join $::dvddir $vobfilename]]} {
			set backuptxt "Backup VOB file?  (VOB is already in the backup folder.)"
		} else {
			set backuptxt "Backup VOB file?  (No VOB file to backup.)"
		}
	}

	if {![info exists ::pgced::create_cell_at_end] || $::pgced::__numcells == 0} {
		set ::pgced::create_cell_at_end 1
		set ::pgced::create_new_prog 1
	}

	if {$all_vobids != ""} {
		set ::pgced::reassign_cell_mode 1
	} else {
		set ::pgced::reassign_cell_mode 2
	}

	set w .newcell
	if {[winfo exists $w]} {destroy $w}
	toplevel $w -class Dialog
	wm withdraw $w
	wm title $w $utiltitle
	if {$::config(tk) != "aqua"} {wm transient $w .}
	wm protocol $w WM_DELETE_WINDOW {set ::macros::newcell_done 0}
	bind $w <Escape> "set ::macros::newcell_done 0"
	bind $w <Return> "set ::macros::newcell_done 1"

	set txt "This function creates a new cell in the PGC."
	set txt "${txt}\nYou can create a new blank cell or use an existing VOB/Cell ID."
	set txt "${txt}\nIf you create a new cell, you can replace it later with VobBlanker."
	label $w.lab -text $txt -justify left
	pack  $w.lab -padx 2

	frame $w.f0 -bd 2 -relief groove
	pack  $w.f0 -padx 4 -pady 4 -fill x
	radiobutton $w.f0.before -text "Create cell at the beginning of the PGC" \
			-variable ::pgced::create_cell_at_end -value 0 -anchor w -pady 0 \
			-command "$w.f0.newprog configure -state disabled"
	pack  $w.f0.before -anchor w -fill x -expand 1 -pady 0
	radiobutton $w.f0.after -text "Create cell at the end of the PGC" \
			-variable ::pgced::create_cell_at_end -value 1 -anchor w -pady 0 \
			-command "$w.f0.newprog configure -state normal"
	pack  $w.f0.after -anchor w -fill x -expand 1 -pady 0
	if {$cur_type == "menu"} {
		set txt "Create new program"
	} else {
		set txt "Create new program and chapter"
	}
	checkbutton $w.f0.newprog -text $txt -variable ::pgced::create_new_prog -anchor w
	pack  $w.f0.newprog -anchor w -fill x -expand 1 -padx 20
	if {$::pgced::__numcells == 0} {
		$w.f0.before  configure -state disabled
		$w.f0.after   configure -state disabled
		$w.f0.newprog configure -state disabled
	} else {
		if {! $::pgced::create_cell_at_end} {
			$w.f0.newprog configure -state disabled
		}
	}

	if {$all_vobids != ""} {
		frame $w.f1 -bd 2 -relief groove
		pack  $w.f1 -fill x -padx 4
		frame $w.f1.tf
		pack  $w.f1.tf -fill x
		radiobutton $w.f1.tf.rb -text "Pick an existing VOB/Cell ID:" \
				-variable ::pgced::reassign_cell_mode -value 1 -anchor w \
				-command "::pgced::validate_reassign_gui $w"
		pack  $w.f1.tf.rb -side left
		if {$::config(use_preview)} {
			button $w.f1.tf.pv -text "Preview" -pady 0 -command \
					"::preview::preview_vcid [lindex $::editor_cmds_from 0] $cur_vts \$::pgced::new_vobid \$::pgced::new_cellid"
			pack  $w.f1.tf.pv -side right -padx 4 -pady 2
		}

		frame $w.f1.l1
		pack  $w.f1.l1 -fill x -padx 5
		label $w.f1.l1.lab -text "New VOB ID:" -width 17 -anchor w
		pack  $w.f1.l1.lab -side left
		entry $w.f1.l1.e -width 4 -state readonly -justify right -bd 0 -relief flat \
				-textvariable ::pgced::new_vobid
		pack  $w.f1.l1.e -side left
		scale $w.f1.s1 -showvalue false -from 0 -to [expr {[llength $all_vobids]-1}] \
				-orient horizontal -variable ::pgced::vobid_scale \
				-command {::pgced::validate_vobid_scale .newcell $all_vobids all_cellids}
		pack  $w.f1.s1 -padx 2 -fill x

		frame $w.f1.l2
		pack  $w.f1.l2 -fill x -padx 5
		label $w.f1.l2.lab -text "New Cell ID:" -width 17 -anchor w
		pack  $w.f1.l2.lab -side left
		entry $w.f1.l2.e -width 3 -state readonly -justify right -bd 0 -relief flat \
				-textvariable ::pgced::new_cellid
		pack  $w.f1.l2.e -side left
		scale $w.f1.s2 -showvalue false -from 0 -to [expr {[llength $all_cellids($cur_vobid)]-1}] \
				-orient horizontal -variable ::pgced::cellid_scale \
				-command "::pgced::validate_cellid_scale \{$all_cellids($cur_vobid)\}"
		pack  $w.f1.s2 -padx 2 -fill x
		if {[lindex $lastvobcelllist 0] == 0} {
			set txt " (Not available)"
			set state disabled
			set cmd bell
		} else {
			set txt "(VOB/Cell ID [lindex $lastvobcelllist 0]/[lindex $lastvobcelllist 1])"
			set state normal
			set tmp_vid [lindex $lastvobcelllist 0]
			set tmp_cid [lindex $lastvobcelllist 1]
			set cmd "set ::pgced::vobid_scale [lsearch $all_vobids $tmp_vid]; \
					 set ::pgced::cellid_scale [lsearch $all_cellids($tmp_vid) $tmp_cid]; \
					 set ::pgced::new_vobid  $tmp_vid; set ::pgced::new_cellid $tmp_cid"
		}
		button $w.f1.reuse -text "Use last blank cell in VOB $txt" -state $state -command $cmd
		pack  $w.f1.reuse -fill x -padx 4 -pady 2
		::balloon::set_balloon $w.f1.reuse "To blank a PGC Cell, use this button whenever\npossible, or create a new blank VOB cell."
		label $w.f1.lab -text "Note: The VOB file is not modified by this operation." -anchor w
		pack  $w.f1.lab -fill x -padx 2
	}

	frame $w.f2 -bd 2 -relief groove
	pack  $w.f2 -fill x -padx 4 -pady 4
	radiobutton $w.f2.rb -text "Create a new blank VOB Cell:" \
			-variable ::pgced::reassign_cell_mode -value 2 -anchor w \
			-command "::pgced::validate_reassign_gui $w"
	pack  $w.f2.rb -fill x
	set txt "A new blank cell will be appended to the VOB file and its VOB ID"
	set txt "$txt\nand Cell ID will be assigned to the new Cell."
	label $w.f2.lab -text $txt -justify left -anchor w
	pack  $w.f2.lab -fill x -padx 4
	checkbutton $w.f2.cb -text "Copy original VOB to Backup folder" \
			-variable ::pgced::backup_vob -anchor w
	pack  $w.f2.cb -fill x -padx 2
	set txt "WARNING: The VOB file will be modified immediately."
	set txt "$txt\nTHE DVD WILL BE SAVED after this operation, to keep track of"
	set txt "$txt\nthe new VOB cell in the IFOs!"
	label $w.f2.lab2 -text $txt -justify left -anchor w -bd 2 -relief groove
	pack  $w.f2.lab2 -fill x -padx 4 -pady 2
	checkbutton $w.f2.notm -text "Rebuild the Time Map of the current Title\n(Recommended, but slow.)" \
			-variable ::pgced::rebuild_time_map -anchor w -justify left
	if {$::pgced::__pgctype == "pgc"} {
		pack  $w.f2.notm -fill x -padx 2
	}

	# buttons
	frame  $w.fb
	pack   $w.fb -padx 2 -pady 4 -anchor e
	button $w.fb.ok -text "Create cell" -width 11 -command {set ::macros::newcell_done 1}
	pack   $w.fb.ok -side left -padx 2
	button $w.fb.cancel -text "Cancel" -width 11 -command {set ::macros::newcell_done 0}
	pack   $w.fb.cancel -side left -padx 2

	validate_reassign_gui $w

	update
	wm minsize $w [winfo reqwidth $w] [winfo reqheight $w]
	wm maxsize $w [winfo reqwidth $w] [winfo reqheight $w]
	set wx [expr {[winfo rootx .]+(([winfo width  .]-[winfo reqwidth  $w])/2)}]
	set wy [expr {[winfo rooty .]+(([winfo height .]-[winfo reqheight $w])/2)-25}]
	if {$wx < 10} {set wx 10}
	if {$wy < 22} {set wy 22}
	wm geometry $w +$wx+$wy
	wm deiconify $w
	focus $w
	grab set $w

	tkwait variable ::macros::newcell_done
	destroy $w
	grab set $parent
	if {!$::macros::newcell_done} return

	if {$::pgced::reassign_cell_mode == 1} {
		set new_vobid $::pgced::new_vobid
		set new_cellid $::pgced::new_cellid
		set src_lu   [lindex $src_pgc_cell($new_vobid,$new_cellid) 1]
		set src_pgc  [lindex $src_pgc_cell($new_vobid,$new_cellid) 2]
		set src_cell [lindex $src_pgc_cell($new_vobid,$new_cellid) 3]
		if {$cur_type == "menu"} {
			set src_cellpbtable $::menus($cur_vts,$src_lu,$src_pgc,cellpbtable)
		} else {
			set src_cellpbtable $::pgcs($cur_vts,$src_pgc,cellpbtable)
		}
		set cellpbidx [expr ($src_cell-1)*24]
		set src_cellpbentry [string range $src_cellpbtable $cellpbidx [expr $cellpbidx+23]]
		set rebuild_time_map 1
	} else {
		# Append a blank cell to the VOB
		set resultlist [::blankcell::AppendBlankCell $cur_type $cur_vts $::pgced::backup_vob true]
		set new_vobid		[lindex $resultlist 0]
		set new_cellid      1
		set src_cellpbentry	[lindex $resultlist 2]
		set new_lba [get_int src_cellpbentry 8 4]
		if {$new_vobid == 0} return
		if {$new_vobid < 0} {
			set txt "Error appending new Cell to VOB file:\n"
			set txt "$txt\n$src_cellpbentry"
			tk_messageBox -title "Create blank cell" -icon error -message $txt -parent $parent
			return
		}
		# update buttons informations
		if {$cur_type == "menu"} {
			if {$::pgced::__numcells == 0} {
				set ::menus($cur_index,numbuttons) 0
				if {![info exists ::buttons::butsinfo(menu,$cur_vts,errorcode)]} {
					set ::buttons::butsinfo(menu,$cur_vts,errorcode) 0
					set ::buttons::butsinfo(menu,$cur_vts,errormsg) ""
				}
			}
			set ::buttons::butsinfo(menu,$cur_vts,numvobids) $new_vobid
			set ::buttons::butsinfo(menu,$cur_vts,$::buttons::butsinfo(menu,$cur_vts,numvobids),numcellids) 1
			set ::buttons::butsinfo(menu,$cur_vts,$new_vobid,1,navpack) $new_lba
			set ::buttons::butsinfo(menu,$cur_vts,$new_vobid,1,numbuts) 0
		} else {
			if {$::pgced::__numcells == 0} {
				set ::pgcs($cur_index,numbuttons) 0
			}
			set ::buttons::butsinfo(pgc,$cur_vts,numvobids) $new_vobid
			set ::buttons::butsinfo(pgc,$cur_vts,$::buttons::butsinfo(pgc,$cur_vts,numvobids),numcellids) 1
			set ::buttons::butsinfo(pgc,$cur_vts,$new_vobid,1,navpack) $new_lba
			set ::buttons::butsinfo(pgc,$cur_vts,$new_vobid,1,numbuts) 0
		}
		set rebuild_time_map $::pgced::rebuild_time_map
	}

	incr ::pgced::__numcells

	# Add a new chapter if cell is created at end of PGC
	if {$cur_type == "pgc" && $::pgced::create_cell_at_end && $::pgced::create_new_prog} {
		set lastptt 0
		set ttu 1
		for {set p 1} {$p <= $::pgcs($cur_vts,numpgcs)} {incr p} {
			if {$::pgcs($cur_vts,$p,TTN) == $::pgcs($::pgced::__index,TTN)} {
				if {[info exists ::pgcs($cur_vts,$p,pttlist)]} {
					set pttlist $::pgcs($cur_vts,$p,pttlist)
					foreach i $pttlist {
						if {[lindex $i 1] > $lastptt} {set lastptt [lindex $i 1]}
						if {[lindex $i 2] > $ttu} {set ttu [lindex $i 2]}
					}
				}
			}
		}
		foreach i $::pgced::__PTTlist {
			if {[lindex $i 1] > $lastptt} {set lastptt [lindex $i 1]}
			if {[lindex $i 2] > $ttu} {set ttu [lindex $i 2]}
		}
		incr lastptt
		lappend ::pgced::__PTTlist "[expr $::pgced::__numprograms+1] $lastptt $ttu"
	}

	# update the program map table
	if {$::pgced::create_cell_at_end} {
		if {$::pgced::create_new_prog} {
			set ::pgced::__programmap "[string range $::pgced::__programmap 0 [expr $::pgced::__numprograms-1]][d2c $::pgced::__numcells 1]"
			incr ::pgced::__numprograms
		}
	} else {
		for {set p 1} {$p < $::pgced::__numprograms} {incr p} {
			set prog [get_int ::pgced::__programmap $p 1]
			if {$prog > 0} {
				set_int ::pgced::__programmap $p [expr $prog+1] 1
			}
		}
	}

	# update the cell playback table
	if {$::pgced::create_cell_at_end} {
		set ::pgced::__cellpbtable $::pgced::__cellpbtable$src_cellpbentry
	} else {
		set ::pgced::__cellpbtable $src_cellpbentry$::pgced::__cellpbtable
	}

	# update the Cell Position table
	set tmp "\000\000\000\000"
	set_int tmp 0 $new_vobid 2
	set_int tmp 3 $new_cellid 1
	if {$::pgced::create_cell_at_end} {
		set ::pgced::__cellpostable $::pgced::__cellpostable$tmp
	} else {
		set ::pgced::__cellpostable $tmp$::pgced::__cellpostable
	}

	# reload the new tables in the editor
	set total_time [build_cell_table $parent]

	# correct the total playback time
	set ::pgced::__pgcheader "[string range $::pgced::__pgcheader 0 3]$total_time[string range $::pgced::__pgcheader 8 end]"
	set ::pgced::pbtimetxt [BCDtime $total_time true]
	set ::pgced::__pgcpbtime $total_time

	# we need to rebuild the time map of this PGC
	if {$rebuild_time_map} {
		set ::pgced::__need_new_tmap true
	}

	# If cell added, then we must remember the current PGC tables,
	# the simplest way to do this is to close the PGC Editor window, with an OK signal.
	if {$::pgced::reassign_cell_mode != 1} {
		::main::dvd_modified true false
		set ::pgced::done 1
		# refresh PGC selector to show the new number of menubuttons
		::main::init_tree_menu_no_move
		after 1000 {
			tk_messageBox -title "Assign VOB/Cell ID" -icon info -parent . \
					-message "The DVD will be saved now..." -skip "Create New Cell: DVD saved now"
			::main::Write_DVD false
		}
	}
	return
}

proc create_playall_pgc {} {
	if {![info exists ::dvddir]} {bell; return}

	set cur_type  [lindex $::editor_cmds_from 0]
	if {$cur_type != "pgc"} {bell; return}
	set cur_index [lindex $::editor_cmds_from 1]
	set cur_vts   [lindex [split $cur_index ","] 0]
	if {$cur_vts == 0} {bell; return}
	set cur_pgc   [lindex [split $cur_index ","] 1]

	show_msg ""
	set txt "This utility will create a new Title (Title [expr {[get_int ::raw_sectors(0,VMG_TT_SRPT) 0 2]+1}]) with all the\n"
	set txt "${txt}VOB/Cell IDs of the current VTS (VTS $cur_vts).\n\n"
	set txt "${txt}The streams status and the color lookup table are\n"
	set txt "${txt}copied from the current PGC (PGC $cur_pgc).\n\n"
	set txt "${txt}A chapter will be created for each VOB/Cell ID.\n\n"
	set txt "${txt}This new Title will have the pre-commands of the\n"
	set txt "${txt}first PGC and the post-commands of the last PGC\n"
	set txt "${txt}of the current title domain.\n\n"
	set txt "${txt}Continue?"
	set r [tk_messageBox -title "New 'Play All' Title" -icon question \
			-message $txt -parent . -type okcancel -default ok -skip "New Play All Title usage"]
	if {$r == "cancel"} return

	#if {$::pgcs($cur_vts,numpgcs) < 2} {
	#	set txt "This utility needs two or more PGCs\n"
	#	set txt "${txt}in the current VTS to be useful!\n"
	#	set txt "${txt}(However, you may want to use this utility\n"
	#	set txt "${txt}to create a 'backup' of all VOB/Cell IDs\n"
	#	set txt "${txt}before remapping the IDs in the PGC Editor.)\n\n"
	#	set txt "${txt}There is only one PGC in this VTS.\nContinue anyway?"
	#	set r [tk_messageBox -title "New 'Play All' Title" -icon question \
	#			-message $txt -parent . -type okcancel -default ok]
	#	if {$r == "cancel"} return
	#}

	show_msg "Creating New PGC $::pgcs($cur_vts,numpgcs)..."

	# Search for all VOB IDs in the current VTS, and for all Cell IDs in each VOB IDs
	# and build the list of vobids, and the lists of cellids for all vobids.
	# Also, for each pair VOB/Cell ID, keep track of the belonging PGC's cell.
	set totcells 0
	set all_vobids ""
	for {set pgc 1} {$pgc <= $::pgcs($cur_vts,numpgcs)} {incr pgc} {
		set cellpostable $::pgcs($cur_vts,$pgc,cellpostable)
		for {set cell 1} {$cell <= $::pgcs($cur_vts,$pgc,numcells)} {incr cell} {
			incr totcells
			set cellidx [expr {($cell - 1) * 4}]
			set vobid  [get_int cellpostable $cellidx 2]
			set cellid [get_int cellpostable [expr {$cellidx + 3}] 1]
			if {[lsearch $all_vobids $vobid] == -1} {
				lappend all_vobids $vobid
				set all_cellids($vobid) ""
			}
			if {[lsearch $all_cellids($vobid) $cellid] == -1} {
				lappend all_cellids($vobid) $cellid
			}
			set src_pgc_cell($vobid,$cellid) [list $pgc $cell]
		}
	}
	if {$totcells > 255} {
		set txt "You have $totcells cells in this VTS.\n"
		set txt "${txt}This is too much for one PGC: the DVD\n"
		set txt "${txt}standard limit the number of cells per\n"
		set txt "${txt}PGC to 255.\n\n"
		set txt "${txt}Do you want ot create a title with only\n"
		set txt "${txt}the first 255 cells?"
		set r [tk_messageBox -title "New 'Play All' Title" -icon error \
				-message $txt -parent . -type okcancel -default cancel]
		if {$r == "cancel"} return
	}
	set all_vobids [lsort -integer $all_vobids]
	foreach vobid $all_vobids {
		set all_cellids($vobid) [lsort -integer $all_cellids($vobid)]
	}

	# Begin the job!
	#set ::pgced::__numprograms 0
	set numcells    0
	set header $::pgcs($cur_vts,[lindex [split $cur_index ","] 1],header)
	set programmap ""
	set cellpbtable ""
	set cellpostable ""
	set old_vobid 0
	set old_cellid 0
	set totalpbtime "\000\000\000\000"
	set numbuttons 0
	foreach vobid $all_vobids {
		if {$numcells == 256} break
		foreach cellid $all_cellids($vobid) {
			incr numcells
			if {$numcells == 256} break
			set thispgc  [lindex $src_pgc_cell($vobid,$cellid) 0]
			set thiscell [lindex $src_pgc_cell($vobid,$cellid) 1]
			set thiscellpbtable $::pgcs($cur_vts,$thispgc,cellpbtable)
			set idx [expr {($thiscell-1) * 0x18}]
			set thiscellpbentry [string range $thiscellpbtable $idx [expr {$idx+0x17}]]
			# fix playback mode
			if {$cellpostable == ""} {
				set mode 2
			} else {
				if {$old_vobid == $vobid && [expr {$old_cellid+1}] == $cellid} {
					set mode 8
				} else {
					set mode 10
				}
			}
			set oldmode [expr {[get_int thiscellpbentry 0 1] & 0xF5}]
			set_int thiscellpbentry 0 [expr {$oldmode | $mode}] 1
			# set cell still time to a max of 3 seconds
			if {[get_int thiscellpbentry 2 1] > 3} {
				set_int thiscellpbentry 2 3 1
			}
			# clear cell command
			set_int thiscellpbentry 3 0 1

			# Add cell time to total PGC time, but only if angle 1.
			set type [get_int thiscellpbtable [expr {($thiscell - 1) * 24}] 1]
			if {[expr {$type & 0x10}] == 0 || [expr {$type & 0xC0}] == 0x40} {
				set totalpbtime [AddBCDtime [string range $thiscellpbentry 4 7] $totalpbtime]
			}

			# add this cell infos to the new cell playback table
			set cellpbtable $cellpbtable$thiscellpbentry

			# add this cell position to the new cell position table
			set idx [string length $cellpostable]
			set cellpostable $cellpostable\000\000\000\000
			set_int cellpostable $idx $vobid 2
			incr idx 3
			set_int cellpostable $idx $cellid 1

			# create a new program for each cell
			set tmp \000
			set_int tmp 0 $numcells 1
			set programmap $programmap$tmp

			# compute number of buttons
			if {[info exists ::buttons::butsinfo(pgc,$cur_vts,scanned)] && \
					$::buttons::butsinfo(pgc,$cur_vts,scanned) == 1} {
				incr numbuttons $::buttons::butsinfo(pgc,$cur_vts,$vobid,$cellid,numbuts)
			}

			set old_cellid $cellid
			set old_vobid $vobid
		}
	}
	if {[expr {[string length $programmap] % 2}] == 1} {
		set programmap $programmap\000
	}
	if {$numcells > 255} {set numcells 255}

	set newpgcn [expr {$::pgcs($cur_vts,numpgcs)+1}]

	# fix PGC header attributes
	set_int header 2 $numcells 1	;# number of programs = total number of cells
	set_int header 3 $numcells 1	;# number of cells = total number of cells
	set_int header 4 [get_int totalpbtime 0 4] 4	;# total Playback time
	set_int header 8 0 4	;# No PUOs
	set_int header 0x9C $newpgcn 2	;# NextPGCN
	set_int header 0x9E $newpgcn 2	;# PrevPGCN
	set_int header 0xA0 0 2	;# GoUpPGCN
	set_int header 0xA3 0 1	;# PGC Still Time = 0
	set_int header 0xA2 0 1	;# PB Mode = sequential

	# take track of the new changes
	set precmds  [get_type_cmds pre  ::pgcs($cur_vts,1)]
	set postcmds [get_type_cmds post ::pgcs($cur_vts,$::pgcs($cur_vts,numpgcs))]
	incr ::pgcs($cur_vts,numpgcs)
	set ::pgcs($cur_vts,$newpgcn) \
			[concat {{* pre}} $precmds {{* post}} $postcmds {{* cell}}]
	set ::pgcs($cur_vts,$newpgcn,header) $header
	set ::pgcs($cur_vts,$newpgcn,numprograms) $numcells
	set ::pgcs($cur_vts,$newpgcn,numcells) $numcells
	set ::pgcs($cur_vts,$newpgcn,cellpbtable) $cellpbtable
	set ::pgcs($cur_vts,$newpgcn,cellpostable) $cellpostable
	set ::pgcs($cur_vts,$newpgcn,programmap) $programmap
	set ttn 0
	for {set i 1} {$i < $newpgcn} {incr i} {
		if {[info exists ::pgcs($cur_vts,$i,TTN)] && $::pgcs($cur_vts,$i,TTN) > $ttn} {
			set ttn $::pgcs($cur_vts,$i,TTN)
		}
	}
	incr ttn
	set ::pgcs($cur_vts,$newpgcn,TTN) $ttn
	set ::ttns($cur_vts,$ttn) $newpgcn
	if {[info exists ::buttons::butsinfo(pgc,$cur_vts,scanned)] && \
			$::buttons::butsinfo(pgc,$cur_vts,scanned) == 1} {
		set ::pgcs($cur_vts,$newpgcn,numbuttons) $numbuttons
	}

	# update VTS_PGCI table
	set idx [expr {($newpgcn * 8)}]
	set part1 [string range $::raw_sectors($cur_vts,VTS_PGCI) 0 [expr {$idx-1}]]
	set part2 [string range $::raw_sectors($cur_vts,VTS_PGCI) $idx end]
	set ::raw_sectors($cur_vts,VTS_PGCI) $part1[string repeat \000 8]$part2
	set_int ::raw_sectors($cur_vts,VTS_PGCI) $idx [expr {$ttn+128}] 1

	### build chapter table
	set ptttable $::raw_sectors($cur_vts,VTS_PTT_SRPT)
	set pttnumtitles [get_int ptttable 0 2]
	set newtitle [expr {$pttnumtitles+1}]
	set pttheader [string range $ptttable 0 [expr {($pttnumtitles*4) + 7}]]
	set pttrest [string range $ptttable [expr {($pttnumtitles*4) + 8}] end]
	# add entry to the PTT table header
	set idx [string length $pttheader]
	set pttheader $pttheader\000\000\000\000
	set_int pttheader $idx [expr {[string length $pttheader]+[string length $pttrest]}] 4
	for {set i 0} {$i < $pttnumtitles} {incr i} {
		set idx [expr {($i * 4)+8}]
		set_int pttheader $idx [expr {[get_int pttheader $idx 4] +4}] 4
	}
	# add a chapter for each cell
	set pttlist ""
	for {set ptt 1} {$ptt <= $numcells} {incr ptt} {
		set tmp \000\000\000\000
		set_int tmp 0 $newpgcn 2
		set_int tmp 2 $ptt 2
		set pttrest $pttrest$tmp
		lappend pttlist "$ptt $ptt $newtitle"
	}
	set ::pgcs($cur_vts,$newpgcn,pttlist) $pttlist
	set ::pgcs($cur_vts,numttus) $newtitle
	set ptttable $pttheader$pttrest
	set_int ptttable 0 [expr {$pttnumtitles+1}] 2
	set_int ptttable 4 [expr {[string length $ptttable]-1}] 4
	set ::raw_sectors($cur_vts,VTS_PTT_SRPT) $ptttable

	# Add the new title to the VMG global table of titles
	set data $::raw_sectors(0,VMG_TT_SRPT)
	set srptnumtitles [get_int data 0 2]
	set data [string range $data 0 [expr {($srptnumtitles*12)+7}]]
	incr srptnumtitles
	set_int data 0 $srptnumtitles 2
	set newentry \024\001\000\000\000\000\000\000\000\000\000\000
	set_int newentry 2 $numcells 2
	set_int newentry 6 $cur_vts 1
	set_int newentry 7 $::pgcs($cur_vts,$newpgcn,TTN) 1
	set data $data$newentry
	set_int data 4 [expr {[string length $data]-1}] 4
	set ::raw_sectors(0,VMG_TT_SRPT) $data
	set ::pgcs($cur_vts,$newpgcn,Title) $srptnumtitles
	set ::pgcs(Title,$srptnumtitles) [list $cur_vts $newpgcn]
	incr ::pgcs(Title,NumTitles)

	# refresh GUI
	::main::dvd_modified true false
	::main::init_tree_menu
	::main::display_pgc pgc $cur_vts $newpgcn
	show_msg "New PGC $newpgcn created OK."
	return
}

proc remove_cell {parent} {
	if {$::pgced::__numcells <= 1} {
		show_msg "Cannot remove the last cell!"
		bell
		return
	}
	set cur_type  [lindex $::editor_cmds_from 0]
	set cur_index [lindex $::editor_cmds_from 1]
	set cur_vts   [lindex [split $cur_index ","] 0]
	if {! [info exists ::pgced::remove_cell_mode]} {
		set ::pgced::remove_cell_mode one
		set ::pgced::syncpgs 1
		set ::pgced::shortenptttable 0
	}

	# GUI
	set w ".delcell"
	if {[winfo exists $w]} {destroy $w}
	toplevel $w -class Dialog
	wm withdraw $w
	wm protocol $w WM_DELETE_WINDOW {set ::pgced::delcelldone 0}
	bind $w <Escape> "set ::pgced::delcelldone 0"
	bind $w <Return> "set ::pgced::delcelldone 1"
	if {$::config(tk) != "aqua"} {wm transient $w $parent}
	if {$::tcl_platform(platform) == "windows"} {catch {wm attributes $w -toolwindow 1}}
	wm title $w "Remove Cell"

	set txt "WARNING: The old VOB/Cell ID of the removed cell"
	set txt "$txt\nwill be unavailable in the future, unless it is reused"
	set txt "$txt\nin another PGC !"
	if {$cur_type == "pgc"} {
		set txt "$txt\n(Hint: Use the \"New 'Play All' Title\" utility to 'backup'"
		set txt "$txt\nthe old VOB Cell references before using this function.)"
	}
	set txt "$txt\n\nNote: The VOB file is not modified by this operation."

	label $w.lab -text $txt -justify left -anchor w
	pack  $w.lab -fill x -padx 2

	frame $w.f1 -bd 2 -relief groove
	pack  $w.f1 -fill x -padx 4
	radiobutton $w.f1.rb -text "Remove one cell" -variable ::pgced::remove_cell_mode -value one \
			-command {::pgced::validate_remove_cell rb1} -pady 0 -bd $::config(min_bd) -highlightthickness 1
	pack  $w.f1.rb -anchor w
	frame $w.f1.f1
	pack  $w.f1.f1 -fill x
	label $w.f1.f1.lab -text "cell" -width 4 -anchor e
	pack  $w.f1.f1.lab -side left
	scale $w.f1.f1.s1 -showvalue true -from 1 -to $::pgced::__numcells \
			-orient horizontal -variable ::pgced::remove_cell_num
	pack  $w.f1.f1.s1 -padx 1 -fill x

	frame $w.f2 -bd 2 -relief groove
	pack  $w.f2 -fill x -padx 4 -pady 4
	radiobutton $w.f2.rb -text "Remove cell range" -variable ::pgced::remove_cell_mode -value range \
			-command {::pgced::validate_remove_cell rb2} -pady 0 -bd $::config(min_bd) -highlightthickness 1
	pack  $w.f2.rb -side top -anchor w
	frame $w.f2.f1
	pack  $w.f2.f1 -fill x
	label $w.f2.f1.lab -text "from" -width 4 -anchor e
	pack  $w.f2.f1.lab -side left
	scale $w.f2.f1.s1 -showvalue true -from 1 -to $::pgced::__numcells \
			-orient horizontal -variable ::pgced::remove_cell_from \
			-command {::pgced::validate_remove_cell from}
	pack  $w.f2.f1.s1 -padx 1 -fill x
	frame $w.f2.f2
	pack  $w.f2.f2 -fill x
	label $w.f2.f2.lab -text "to" -width 4 -anchor e
	pack  $w.f2.f2.lab -side left
	scale $w.f2.f2.s1 -showvalue true -from 1 -to $::pgced::__numcells \
			-orient horizontal -variable ::pgced::remove_cell_to \
			-command {::pgced::validate_remove_cell to}
	pack  $w.f2.f2.s1 -padx 1 -fill x

	checkbutton $w.cb -text "Keep the Programs in sync with the cells" \
			-variable ::pgced::syncpgs
	pack  $w.cb -anchor w
	if {$cur_type == "pgc"} {
		checkbutton $w.fc -text "Shorten the chapter table if needed" \
				-variable ::pgced::shortenptttable -command {
						if {$::pgced::shortenptttable && ![info exists ::pgced::shorten_ptt_table_warning]} {
							set ::pgced::shorten_ptt_table_warning true
							set txt "Warning: When this option is enabled,"
							set txt "$txt\nPgcEdit will automatically and silently"
							set txt "$txt\nremove all chapters not associated with"
							set txt "$txt\nprograms any more after the operation."
							set txt "$txt\n\nYou should be sure that those removed"
							set txt "$txt\nchapters are not called explicitly by a"
							set txt "$txt\nVM command, or that command will fail."
							set txt "$txt\n\nIt might also be dangerous to use this"
							set txt "$txt\noption on not-one_sequential PGCs!"
							set txt "$txt\n\n(Note: this warning is displayed only once)"
							tk_messageBox -title "Remove Cell" -message $txt -parent .delcell -icon warning
						}
					}
		pack  $w.fc -anchor w
	}

	frame  $w.fb
	pack   $w.fb -anchor e -padx 4 -pady 4
	button $w.fb.ok -text "OK" -width 10 -command {set ::pgced::delcelldone 1}
	pack   $w.fb.ok -side left -padx 4
	button $w.fb.cancel -text "Cancel" -width 10 -command {set ::pgced::delcelldone 0}
	pack   $w.fb.cancel -side left

	update
	set wx [expr {[winfo rootx $parent]+(([winfo width  $parent]-[winfo reqwidth  $w])/2)}]
	set wy [expr {[winfo rooty $parent]+(([winfo height $parent]-[winfo reqheight $w])/2)-25}]
	if {$wx < 10} {set wx 10}
	if {$wy < 22} {set wy 22}
	wm geometry $w +$wx+$wy
	wm resizable $w 0 0

	wm deiconify $w
	focus $w
	grab set $w

	set done false
	while {! $done} {
		set ::pgced::delcelldone -1
		tkwait variable ::pgced::delcelldone
		if {!$::pgced::delcelldone} {
			grab release $w
			if {$parent != "."} {
				grab set $parent
			}
			destroy $w
			return
		}

		if {$::pgced::remove_cell_mode == "one"} {
			set from $::pgced::remove_cell_num
			set to   $::pgced::remove_cell_num
			set done true
		} else {
			set from $::pgced::remove_cell_from
			set to   $::pgced::remove_cell_to
			if {[expr $::pgced::__numcells - ($to - $from + 1)] == 0} {
				show_msg "Cannot remove the last cell!"
				bell
			} else {
				set done true
			}
		}
	}
	grab release $w
	if {$parent != "."} {
		grab set $parent
	}
	destroy $w
	set cellnum $from
	set warning false
	for {set loop $to} {$loop >= $from} {incr loop -1} {
		incr ::pgced::__numcells -1

		# substract the buttons of the removed cells from the total number of buttons
		set cellposidx [expr {($cellnum - 1) * 4}]
		set vobid  [get_int ::pgced::__cellpostable $cellposidx 2]
		set cellid [get_int ::pgced::__cellpostable [expr {$cellposidx + 3}] 1]
		if {$cur_type == "menu"} {
			if {[info exists ::buttons::butsinfo(menu,$cur_vts,$vobid,$cellid,numbuts)]} {
				set numbuts $::buttons::butsinfo(menu,$cur_vts,$vobid,$cellid,numbuts)
				incr ::menus($cur_index,numbuttons) [expr {$numbuts * -1}]
			}
		} else {
			if {[info exists ::pgcs($cur_index,numbuttons)] && \
				[info exists ::buttons::butsinfo(pgc,$cur_vts,$vobid,$cellid,numbuts)]} {
				set numbuts $::buttons::butsinfo(pgc,$cur_vts,$vobid,$cellid,numbuts)
				incr ::pgcs($cur_index,numbuttons) [expr {$numbuts * -1}]
			} else {
			}
		}
		# remove the new entry from the current cell playback table
		set cellpbidx [expr {($cellnum - 1) * 24}]
		set ::pgced::__cellpbtable "[string range $::pgced::__cellpbtable 0 [expr {$cellpbidx-1}]][string range $::pgced::__cellpbtable [expr {$cellpbidx+24}] end]"

		# remove the new entry from the current cell position table
		set cellposidx [expr {($cellnum - 1) * 4}]
		set ::pgced::__cellpostable "[string range $::pgced::__cellpostable 0 [expr {$cellposidx-1}]][string range $::pgced::__cellpostable [expr {$cellposidx+4}] end]"

		# resync program map or remove unnecessary entries in the program map
		set newprogrammap {}
		if {$::pgced::syncpgs} {
			for {set i 0} {$i < [string length $::pgced::__programmap]} {incr i} {
				set keeppg 1
				set c [get_int ::pgced::__programmap $i 1]
				if {$c == 0} continue
				if {$c==$loop && $c==[expr $::pgced::__numcells+1]} {
					set keeppg 0
					set warning true
				}
				if {$c>$loop} {
					incr c -1
				}
				if {$keeppg} {
					set newprogrammap $newprogrammap[d2c $c 1]
				}
			}
		} else {
			for {set i 0} {$i < [string length $::pgced::__programmap]} {incr i} {
				set c [get_int ::pgced::__programmap $i 1]
				if {$c<=$::pgced::__numcells && $c!=0} {
					set newprogrammap $newprogrammap[d2c $c 1]
				}
			}
		}
		set ::pgced::__numprograms [string length $newprogrammap]
		if {[expr {[string length $newprogrammap] % 2}]} {
			set newprogrammap $newprogrammap\000
		}
		set ::pgced::__programmap $newprogrammap

		# Set the STC Discontinuity flag in the NEXT Cell entry, and clear the Seamless flag
		if {$::pgced::__numcells >= $cellnum} {
			set_int ::pgced::__cellpbtable $cellpbidx [expr {([get_int ::pgced::__cellpbtable $cellpbidx 1] | 2) & 0xF7}] 1
		}
	}
	# reload the new tables in the editor
	set total_time [build_cell_table $parent]

	# correct the total playback time
	set ::pgced::pbtimetxt [BCDtime $total_time true]
	set ::pgced::__pgcpbtime $total_time
	set ::pgced::__pgcheader "[string range $::pgced::__pgcheader 0 3]$::pgced::__pgcpbtime[string range $::pgced::__pgcheader 8 end]"
	::main::dvd_modified true false

	# we need to rebuild the time map of this PGC
	set ::pgced::__need_new_tmap true

	set multiplepgs false
	for {set i 0} {$i < [string length $::pgced::__programmap]} {incr i} {
		set c [get_int ::pgced::__programmap $i 1]
		if {$c > 0} {
			if {[info exists pg($c)]} {
				set pg($c) "$pg($c) [expr $i+1]"
				set multiplepgs true
			} else {
				set pg($c) [expr $i+1]
			}
		}
	}
	if {$multiplepgs} {
		set txt "Warning!\nSeveral programs are now pointing to the same cell."
		set txt "$txt\nThis situation might be illegal."
		set txt "$txt\nYou can fix it by clicking on the program buttons, but"
		set txt "$txt\nthe programs will not be in sync with the cells any more!\n"
		foreach i [array names pg] {
			if {[llength $pg($i)] > 1} {
				set txt "$txt\nPrograms [join $pg($i) {, }] -> Cell $i"
			}
		}
		tk_messageBox -title "Remove Cell" -icon warning -message $txt -parent .
	}

	if {$cur_type == "pgc" && $::pgced::shortenptttable} {
		set ::pgced::__PTTlist [shorten_ptt_table_if_needed $::pgced::__PTTlist $::pgced::__programmap]
	}

	if {$cur_type == "pgc"} {
		if {!$::pgced::syncpgs} {
			set warning true
		}
		if {$warning && !$::pgced::shortenptttable} {
			.pgceditor.fb.ptt configure -foreground red
			if {![info exists ::skip_dialog_during_session(chap_ed_warning)]} {
				set txt "You must edit the chapter table to remove or reassign\nchapters pointing to removed programs!\n\nClick on the \"Edit Chapter (PTT) Table\" button."
				set r [show_warning -title "Toggle Program" -icon warning -message $txt -parent .pgceditor \
						-skip "Remove cells: Edit PTT table warning" -type "OK to All|OK" -default ok]
				if {$r == "ok to all"} {
					set ::skip_dialog_during_session(chap_ed_warning) true
				}
			}
		}
	}
	return
}
proc validate_remove_cell {widget args} {
	set w ".delcell"
	if {$::pgced::remove_cell_mode == "one"} {
		::ced::scale_state $w.f1.f1.s1 normal
		::ced::scale_state $w.f2.f1.s1 disabled
		::ced::scale_state $w.f2.f2.s1 disabled
	} else {
		::ced::scale_state $w.f1.f1.s1 disabled
		::ced::scale_state $w.f2.f1.s1 normal
		::ced::scale_state $w.f2.f2.s1 normal
		if {$widget == "from"} {
			if {$::pgced::remove_cell_to < $::pgced::remove_cell_from} {
				set ::pgced::remove_cell_to $::pgced::remove_cell_from
			}
		}
		if {$widget == "to"} {
			if {$::pgced::remove_cell_to < $::pgced::remove_cell_from} {
				set ::pgced::remove_cell_from $::pgced::remove_cell_to
			}
		}
	}
}
# removes chapters not assigned to an existing PG from the end of the PTT list
proc shorten_ptt_table_if_needed {pttlist pgmap} {
	set numpgs [string length [string trim $pgmap \000]]
	for {set i [expr [llength $pttlist]-1]} {$i > 0} {incr i -1} {
		if {[lindex [lindex $pttlist $i] 0]<=$numpgs} {
			break
		}
		set pttlist [lreplace $pttlist $i $i]
	}
	return $pttlist
}

proc export_celltimes {parent} {
	set cellpbtable $::pgced::__cellpbtable
	set bcd [string range $cellpbtable 4 7]
	switch [expr {[get_int bcd 3 1] & 0xC0}] {
		192 {set fps 30}
		64  {set fps 25}
		default {bell; return}
	}

	# GUI
	if {![info exists ::pgced::celltimes_offset]} {
		set ::pgced::celltimes_offset 0
		set ::pgced::celltimes_includestarttime 0
		set ::pgced::celltimes_includeendtime 0
		set ::pgced::celltimes_output allcells
		set ::pgced::celltimes_format "frames"
	} else {
		if {[info exists ::pgced::lastcelltime] && $::pgced::lastcelltime > 1 && $::pgced::celltimes_offset > 0} {
			set ::pgced::celltimes_offset $::pgced::lastcelltime
		} else {
			set ::pgced::celltimes_offset 0
		}
	}

	set w ".celltimes"
	if {[winfo exists $w]} {destroy $w}
	toplevel $w -class Dialog
	wm withdraw $w
	wm protocol $w WM_DELETE_WINDOW {set ::pgced::celltimesdone 0}
	bind $w <Escape> "set ::pgced::celltimesdone 0"
	bind $w <Return> "set ::pgced::celltimesdone 1"
	if {$::config(tk) != "aqua"} {wm transient $w $parent}
	if {$::tcl_platform(platform) == "windows"} {catch {wm attributes $w -toolwindow 1}}
	wm title $w "Celltimes.txt file"

	label $w.lab -text "The Celltimes.txt file is needed to create the\nchapter points when you remux the elementary\nstreams with Muxman or IfoEdit, or when you\nreencode the video with an encoder like CCE." -justify left
	pack  $w.lab -anchor w -padx 4

	frame $w.f1 -bd 2 -relief groove
	pack  $w.f1 -padx 4 -pady 2 -fill x
	set txt "If you want to mux several clips in one PGC,\nyou may add an offset to the cell times.\nThe offset MUST be specified in frames."
	if {[info exists ::pgced::lastcelltime] && $::pgced::lastcelltime > 1} {
		set txt "$txt\n\nHint: The last time you used this function,\nthe End Time was $::pgced::lastcelltime."
	}
	label $w.f1.lab -text $txt -anchor w -justify left
	pack  $w.f1.lab -anchor w
	frame $w.f1.f
	pack  $w.f1.f -fill x
	label $w.f1.f.l -text "Offset:"
	pack  $w.f1.f.l -side left
	entry $w.f1.f.e -textvariable ::pgced::celltimes_offset -width 10 -validate all -validatecommand "::pgced::validate_celltimes_offset %P"
	pack  $w.f1.f.e -side left -padx 2 -pady 2
	if {[info exists ::pgced::lastcelltime] && $::pgced::lastcelltime > 1} {
		button $w.f1.f.b -text "<- Use last End Time" -command "set ::pgced::celltimes_offset $::pgced::lastcelltime" -pady 0
		pack   $w.f1.f.b -side left -padx 2 -pady 2 -fill y
	} else {
		label $w.f1.f.b -text "frames."
		pack  $w.f1.f.b -side left
	}
	checkbutton $w.f1.start -text "Include offset as first cell time ?" -variable ::pgced::celltimes_includestarttime
	pack  $w.f1.start -anchor w

	frame $w.f2 -bd 2 -relief groove
	pack  $w.f2 -padx 4 -pady 2 -fill x
	checkbutton $w.f2.last -text "Include End Time (= last frame + 1) ?" -variable ::pgced::celltimes_includeendtime
	pack  $w.f2.last -anchor w
	label $w.f2.lab -text "Don't use this option for Muxman or IfoEdit!" -anchor w -justify left
	pack  $w.f2.lab -anchor w

	frame $w.f3 -bd 2 -relief groove
	pack  $w.f3 -padx 4 -pady 2 -fill x
	radiobutton $w.f3.out1 -text "Output all cells" -variable ::pgced::celltimes_output -value allcells -bd $::config(min_bd)
	pack  $w.f3.out1 -anchor w -pady 0
	radiobutton $w.f3.out2 -text "Output only programs" -variable ::pgced::celltimes_output -value programs -bd $::config(min_bd)
	pack  $w.f3.out2 -anchor w -pady 0
	radiobutton $w.f3.out3 -text "Output only chapters" -variable ::pgced::celltimes_output -value chapters -bd $::config(min_bd)
	pack  $w.f3.out3 -anchor w -pady 0

	frame $w.f4 -bd 2 -relief groove
	pack  $w.f4 -padx 4 -pady 2 -fill x
	radiobutton $w.f4.out1 -text "Output frame numbers (Muxman, IfoEdit)" -variable ::pgced::celltimes_format -value frames -bd $::config(min_bd)
	pack  $w.f4.out1 -anchor w -pady 0
	radiobutton $w.f4.out2 -text "Output time codes (CCE...)" -variable ::pgced::celltimes_format -value timecodes -bd $::config(min_bd)
	pack  $w.f4.out2 -anchor w -pady 0
	radiobutton $w.f4.out3 -text "Output as Wobble .MLS file" -variable ::pgced::celltimes_format -value womble -bd $::config(min_bd)
	pack  $w.f4.out3 -anchor w -pady 0

	frame  $w.fb
	pack   $w.fb -padx 4 -pady 4 -fill x
	button $w.fb.def -text "Defaults" -width 10 -command {set ::pgced::celltimes_offset 0; \
		set ::pgced::celltimes_includestarttime 0; set ::pgced::celltimes_includeendtime 0; \
		set ::pgced::celltimes_output allcells; set ::pgced::celltimes_format "frames"}
	pack   $w.fb.def -side left
	button $w.fb.cancel -text "Cancel" -width 10 -command {set ::pgced::celltimesdone 0}
	pack   $w.fb.cancel -side right
	button $w.fb.ok -text "OK" -width 10 -command {set ::pgced::celltimesdone 1}
	pack   $w.fb.ok -side right -padx 4

	update
	set wx [expr {[winfo rootx $parent]+(([winfo width  $parent]-[winfo reqwidth  $w])/2)}]
	set wy [expr {[winfo rooty $parent]+(([winfo height $parent]-[winfo reqheight $w])/2)-25}]
	if {$wx < 10} {set wx 10}
	if {$wy < 22} {set wy 22}
	wm geometry $w +$wx+$wy
	wm resizable $w 0 0

	wm deiconify $w
	focus $w
	grab set $w

	set ::pgced::celltimesdone -1
	tkwait variable ::pgced::celltimesdone
	grab release $w
	if {"$parent" != "."} {
		grab set $parent
	}
	destroy $w
	if {!$::pgced::celltimesdone} return

	if {$::pgced::celltimes_offset == 0} {
		set lastcelltime 0
	} else {
		set lastcelltime $::pgced::celltimes_offset
	}
	if {$::pgced::celltimes_includestarttime} {
		if {$::pgced::celltimes_format == "timecodes"} {
			set txt "[BCDtime [frames_to_BCDtime $lastcelltime $fps] false]\n"
		} else {
			set txt "$lastcelltime\n"
		}
	} else {
		set txt ""
	}

	# build programs and chapters tables
	if {$::pgced::celltimes_output != "allcells"} {
		for {set c 1} {$c <= [expr $::pgced::__numcells+1]} {incr c} {
			set cell_program($c) 0
			set prog_chapter($c) 0
		}
		for {set p 1} {$p <= $::pgced::__numprograms} {incr p} {
			set cell_program([get_int ::pgced::__programmap [expr {$p-1}] 1]) $p
		}
		if {$::pgced::celltimes_output == "chapters"} {
			foreach pttpair $::pgced::__PTTlist {
				set pg  [lindex $pttpair 0]
				set ptt [lindex $pttpair 1]
				for {set c 1} {$c <= $::pgced::__numcells} {incr c} {
					if {$pg == $cell_program($c) && $prog_chapter($c) == 0} {
						set prog_chapter($c) $ptt
					}
				}
			}
		}
	}
	for {set c 1} {$c <= $::pgced::__numcells} {incr c} {
		set cellidx [expr ($c-1) * 24]
		set bcd [string range $cellpbtable [expr $cellidx+4] [expr $cellidx+7]]
		set numhours [expr (([get_int bcd 0 1]/16)*10)+([get_int bcd 0 1]&15)]
		set nummins  [expr (([get_int bcd 1 1]/16)*10)+([get_int bcd 1 1]&15)]
		set numsecs  [expr (([get_int bcd 2 1]/16)*10)+([get_int bcd 2 1]&15)]
		set numfrms  [expr ((([get_int bcd 3 1]&0x030)/16)*10)+([get_int bcd 3 1]&15)]
		set totalsecs [expr $numsecs+($nummins*60)+($numhours*60*60)]
		set totalframes [expr $numfrms + ($totalsecs*$fps)]
		#set txt "${txt}$c [BCDtime $bcd false] -> $totalframes\n"
		incr lastcelltime $totalframes
		if {$c < $::pgced::__numcells} {
			set c1 [expr $c+1]
			if {$::pgced::celltimes_format == "timecodes"} {
				set outputstr [BCDtime [frames_to_BCDtime $lastcelltime $fps] false]
			} else {
				set outputstr $lastcelltime
			}
			if {$::pgced::celltimes_format == "womble"} {
				set outputstr "[format %12d $outputstr] 1"
			}
			switch $::pgced::celltimes_output {
				"allcells" {
					set txt "${txt}$outputstr\n"
				}
				"programs" {
					if {$cell_program($c1) != 0} {set txt "${txt}$outputstr\n"}
				}
				"chapters" {
					if {$prog_chapter($c1) != 0} {set txt "${txt}$outputstr\n"}
				}
			}
		} elseif {$::pgced::celltimes_includeendtime} {
			if {$::pgced::celltimes_format == "timecodes"} {
				set txt "${txt}[BCDtime [frames_to_BCDtime $lastcelltime $fps] false]\n"
			} else {
				if {$::pgced::celltimes_format == "womble"} {
					set txt "${txt}[format %12d $lastcelltime] 1\n"
				} else {
					set txt "${txt}$lastcelltime\n"
				}
			}
		}
	}
	if {$::pgced::celltimes_format == "womble"} {
		set vts [lindex [split $::pgced::__index ,] 0]

		set defaultfilebasename [::utils::get_General_Name]
		if {$defaultfilebasename == ""} {
			if {[file tail $::dvddir] != "VIDEO_TS"} {
				set defaultfilebasename [file tail $::dvddir]
			} else {
				set defaultfilebasename [file tail [file dirname $::dvddir]]
			}
			set defaultfilebasename [string trim $defaultfilebasename ":"]
		}
		set defaultmlsfilename [file join [file dirname $::dvddir] \
				${defaultfilebasename}_[format VTS_%02d.mbs $vts]]

		set head ""
		if {$vts == 0} {
			set filename [file nativename [file normalize [file join $::dvddir VIDEO_TS.VOB]]]
		} else {
			if {$::pgced::__pgctype == "menu"} {
				set filename [file nativename [file normalize [file join $::dvddir \
						[format VTS_%02d_0.VOB $vts]]]]
			} else {
				set filename [file nativename [file normalize [file join $::dvddir \
						[format VTS_%02d_1.VOB $vts]]]]
				if {[file exists [file join $::dvddir [format VTS_%02d_2.VOB $vts]]]} {
					set msg "Since there are several VOB files in this domain,"
					set msg "$msg\nyou have to create a MBS file to open them in"
					set msg "$msg\nWomble MPEG Video Wizard DVD."
					set msg "$msg\n\nDo you want to create this file automatically now?"
					set msg "$msg\n\nCAUTION:  Do NOT use foreign, accented or special"
					set msg "$msg\ncharacters in the file name and path!"
					set r [tk_messageBox -title "Womble .MLS chapter list" -message $msg \
							-type yesno -default yes -icon question]
					if {$r == "yes"} {
						set tmp [tk_getSaveFile -filetypes {{"MBS files" {*.mbs}}} \
								-initialdir [file dirname $::dvddir] \
								-initialfile "${defaultfilebasename}_[format VTS_%02d.mbs $vts]" \
								-parent . -title "Save Womble MBS file as..."]
						if {$tmp != ""} {
							set filename $tmp
							set numfiles 0
							set out ""
							for {set i 1} {$i < 10} {incr i} {
								set fname [file nativename [file normalize [file join \
										$::dvddir [format VTS_%02d_%d.VOB $vts $i]]]]
								if {[file exists $fname]} {
									incr numfiles
									set out "${out}File=$fname\n"
								} else {
									break
								}
							}
							set out [string trim "\[MBSFile\]\nNumFile=$numfiles\n$out"]
							if {[string toupper [file extension $filename]] != ".MBS"} {
								set filename $filename.mbs
							}
							saveFile $filename auto $out
							set defaultmlsfilename [string range $filename 0 [expr [string length $filename]-5]].mls
						}
					} else {
						set head "// ***\n// *** WARNING: YOU HAVE PROBABLY TO CHANGE THE PATH NAME BELOW!\n// ***\n"
					}
				}
			}
		}
		set head "${head}\[BookmarkList\]"
		set head "$head\nPathName= [file nativename [file normalize $filename]]"
		set head "$head\nVideoStreamID= 0"
		set head "$head\nFormat= frame"
		set head "$head\nCount= [expr [llength $txt]/2]"
		set txt "$head\n$txt"
		::main::showtext .celltimes "CellTimes.txt - Current offset = $::pgced::celltimes_offset, End Time (next offset) = $lastcelltime" \
				$txt grab -defaultfilename $defaultmlsfilename
	} else {
		::main::showtext .celltimes "CellTimes.txt - Current offset = $::pgced::celltimes_offset, End Time (next offset) = $lastcelltime" $txt grab
	}
	set ::pgced::lastcelltime $lastcelltime
}
proc validate_celltimes_offset {num} {
	if {$num == "-"} {return true}
	if {[string is integer $num]} {return true}
	bell
	return false
}

proc remove_angles {} {
	set numangles 0
	set interleaved false
	set cellpbtable $::pgced::__cellpbtable
	set thisangle 0
	for {set c 1} {$c <= $::pgced::__numcells} {incr c} {
		set cellidx [expr ($c-1) * 24]
		set cellflags  [get_int cellpbtable $cellidx 1]
		set angleflags [expr {$cellflags & 0xD0}]
		if {[expr $cellflags & 4] == 4} {
			set interleaved true
		}
		switch $angleflags {
			0 {
				continue
			}
			80 {
				set thisangle 1
				if {$numangles < $thisangle} {set numangles $thisangle}
			}
			144 {
				incr thisangle
				if {$numangles < $thisangle} {set numangles $thisangle}
			}
			208 {
				incr thisangle
				if {$numangles < $thisangle} {set numangles $thisangle}
				set thisangle 0
			}
		}
	}
	if {$numangles == 0} {
		if {$interleaved} {
			set txt "This PGC is not a multi-angle PGC."
			set txt "$txt\nBut there are interleaved cells in the PGC."
			set txt "$txt\nPgcEdit can remove the ILV information from the cell table."
			set txt "$txt\nHowever, do it ONLY if you know what you are doing!"
			set txt "$txt\nBe sure to delete all other ILV cells (or PGCs)."
			set txt "$txt\nThen, process the Domain with VobBlanker (v2.1.1 or +)"
			set txt "$txt\n\nRemove the ILV flags and First ILVU End values now?"
			set r [tk_messageBox -message $txt -title "Delete Angles" -parent .pgceditor \
					-icon warning -type yesno -default no]
			if {$r == "no"} {
				return 0
			} else {
				# remove ILV informations from the cell table
				for {set c 1} {$c <= $::pgced::__numcells} {incr c} {
					set cellidx [expr ($c-1) * 24]
					set cellflags [get_int cellpbtable $cellidx 1]
					set_int cellpbtable $cellidx [expr {$cellflags & 0x0A}] 1
					set_int cellpbtable [expr {$cellidx+12}] 0 4
				}
				# reload the new tables in the editor
				set ::pgced::__cellpbtable $cellpbtable
				set total_time [build_cell_table .pgceditor]
				return -1
			}
		} else {
			set txt "This PGC is not a multi-angle PGC,"
			set txt "$txt\nand there are no interleaved cells!"
			set txt "$txt\n\nCannot remove any angle or ILV infos!"
			tk_messageBox -message $txt -title "Delete Angles" -parent .pgceditor -icon error
			return 0
		}
	}

	# GUI
	if {! [info exists ::pgced::keep_angle] || $::pgced::keep_angle < $numangles} {
		set ::pgced::keep_angle 1
	}
	set w .delangles
	if {[winfo exists $w]} {destroy $w}
	toplevel $w -class Dialog
	wm withdraw $w
	if {$::config(tk) != "aqua"} {wm transient $w .pgceditor}
	wm protocol $w WM_DELETE_WINDOW {set ::pgced::delanglesdone 0}
	bind $w <Escape> "set ::pgced::delanglesdone 0"
	bind $w <Return> "set ::pgced::delanglesdone 1"
	wm title $w "Delete Angles"
	wm resizable $w 0 0
	if {$::tcl_platform(platform) == "windows"} {catch {wm attributes $w -toolwindow 1}}

	frame $w.f
	pack  $w.f -padx 4 -pady 2 -fill x
	label $w.f.lab -text "Keep angle"
	pack  $w.f.lab -side left
	scale $w.f.s -from 1 -to $numangles -variable "::pgced::keep_angle" -length 170
	pack  $w.f.s -fill x -expand 1 -side left
	set txt "Note: The deleted angles are not removed automatically"
	set txt "$txt\nfrom the VOB files.  To remove them, you must process"
	set txt "$txt\nthe Domain with VobBlanker WITH the option \"Fix always"
	set txt "$txt\nVOB SYNCI/SRI pointers in Titles\"!   Do not use FixVTS."
	label $w.lab2 -text $txt -justify left -bd 2 -relief groove -padx 4 -background pink
	pack  $w.lab2 -anchor w -padx 4 -pady 2

	frame  $w.fb
	pack   $w.fb -padx 2 -pady 4 -anchor e
	button $w.fb.ok -text "OK" -width 8 -command {set ::pgced::delanglesdone 1}
	pack   $w.fb.ok -side left
	button $w.fb.cancel -text "Cancel" -width 8 -command {set ::pgced::delanglesdone 0}
	pack   $w.fb.cancel -side left -padx 4

	update
	set wx [expr {[winfo rootx .pgceditor]+(([winfo width  .pgceditor]-[winfo reqwidth  $w])/2)}]
	set wy [expr {[winfo rooty .pgceditor]+(([winfo height .pgceditor]-[winfo reqheight $w])/2)-25}]
	if {$wx < 10} {set wx 10}
	if {$wy < 22} {set wy 22}
	wm geometry $w +$wx+$wy
	wm resizable $w 0 0
	wm deiconify $w
	focus $w
	grab set $w

	set ::pgced::delanglesdone -1
	tkwait variable ::pgced::delanglesdone
	grab release $w
	grab set .pgceditor
	destroy $w
	if {$::pgced::delanglesdone == 0} {
		return 0
	}

	# remove all angles but one
	set cellstodelete {}
	set thisangle 0
	for {set c 1} {$c <= $::pgced::__numcells} {incr c} {
		set cellidx [expr ($c-1) * 24]
		set cellflags  [get_int cellpbtable $cellidx 1]
		set angleflags [expr {$cellflags & 0xD0}]
		switch $angleflags {
			0 {
				continue
			}
			80 {
				set thisangle 1
			}
			144 -
			208 {
				incr thisangle
			}
			16 {
				set_int cellpbtable $cellidx [expr {$cellflags & 0x0A}] 1
				continue
			}
		}
		if {$thisangle != $::pgced::keep_angle} {
			set cellstodelete [concat $c $cellstodelete]
		} else {
			set_int cellpbtable $cellidx [expr {$cellflags & 0x0A}] 1
			set_int cellpbtable [expr {$cellidx+12}] 0 4
		}
	}

	set programmap $::pgced::__programmap
	set numprograms [string length $programmap]
	set cellpostable $::pgced::__cellpostable
	foreach c $cellstodelete {
		set cellidx [expr ($c-1) * 24]
		set cellpbtable [string range $cellpbtable 0 [expr $cellidx-1]][string range $cellpbtable [expr $cellidx+24] end]
		set posidx [expr ($c-1) * 4]
		set cellpostable [string range $cellpostable 0 [expr $posidx-1]][string range $cellpostable [expr $posidx+4] end]
		for {set p 0} {$p < $numprograms} {incr p} {
			set pgcell [get_int programmap $p 1]
			if {$pgcell > $c} {
				set_int programmap $p [expr $pgcell-1] 1
			}
		}
	}

	set ::pgced::__numcells	[expr $::pgced::__numcells-[llength $cellstodelete]]
	set ::pgced::__cellpbtable $cellpbtable
	set ::pgced::__programmap $programmap
	set ::pgced::__cellpostable $cellpostable

	# reload the new tables in the editor
	set total_time [build_cell_table .pgceditor]
	set ::pgced::__pgcheader "[string range $::pgced::__pgcheader 0 3]$total_time[string range $::pgced::__pgcheader 8 end]"
	set ::pgced::pbtimetxt [BCDtime $total_time true]
	set ::pgced::__pgcpbtime $total_time

	# we need to rebuild the time map of this PGC
	set ::pgced::__need_new_tmap true

	return $::pgced::keep_angle
}

}	;# end of namespace ::pgced


namespace eval ::ptted {

# ::ptted::ptted
# PTT (Chapter) Editor.  Suitable for not-one_sequential Titles
proc ptted {} {
	set utiltitle "PTT (Chapter) Editor"
	namespace eval ::ptted::v {}
	set ::ptted::v::formatstring "PTT %2d :  PGC %3d ,  PG %2d"

	if {! [info exists ::dvddir]} {bell; return}
	set type [lindex $::editor_cmds_from 0]
	if {$type != "pgc"} {bell; return}
	set indexes [split [lindex $::editor_cmds_from 1] ,]
	set vts [lindex $indexes 0]
	set ::ptted::v::vts $vts
	set pgc [lindex $indexes 1]
	if {$vts == 0} {bell; return}
	set ttn $::pgcs($vts,$pgc,TTN)

	# Build list of chapters of current title
	set numpgcsinttn 0
	set ::ptted::v::pgcsinttn {}
	set title 0
	for {set p 1} {$p <= $::pgcs($vts,numpgcs)} {incr p} {
		if {$::pgcs($vts,$p,TTN) == $ttn && $::pgcs($vts,$p,numprograms)>0} {
			incr numpgcsinttn
			lappend ::ptted::v::pgcsinttn $p
			set pttilsts($p) $::pgcs($vts,$p,pttlist)
			if {[info exists ::pgcs($vts,$p,Title)] && $::pgcs($vts,$p,Title)!=""} {
				set title $::pgcs($vts,$p,Title)
			}
		}
	}
	set ::ptted::v::mainlist ""
	foreach p $::ptted::v::pgcsinttn {
		foreach i $pttilsts($p) {
			lappend ::ptted::v::mainlist [format $::ptted::v::formatstring [lindex $i 1] $p [lindex $i 0]]
		}
	}
	set ::ptted::v::mainlist [lsort -dictionary $::ptted::v::mainlist]

	set w .ptted
	if {[winfo exists $w]} {
		destroy $w
	}
	toplevel $w -class Dialog
	wm withdraw $w
	wm protocol $w WM_DELETE_WINDOW {set ::ptted::v::done 0}
	bind $w <Escape> "set ::ptted::v::done 0"
	bind $w <Return> "set ::ptted::v::done 1"
	if {$::config(tk) != "aqua"} {wm transient $w .}
	if {$::tcl_platform(platform) == "windows"} {catch {wm attributes $w -toolwindow 1}}
	wm title $w "$utiltitle: VTS $vts, TTN $ttn (Title $title)"

	frame  $w.mf -bd 2 -relief groove
	pack   $w.mf -padx 4 -pady 2 -expand 1 -fill both
	listbox $w.mf.c -yscrollcommand "$w.mf.vs set" -listvariable ::ptted::v::mainlist \
			-font $::config(fixed_width_font) -width 30 -height 20 -selectmode extended -exportselection 0
	pack   $w.mf.c -side left -expand 1 -fill both
	scrollbar $w.mf.vs -command "$w.mf.c yview" -orient vert
	pack   $w.mf.vs -side left -fill y


	set ::ptted::v::pttnum 1
	set ::ptted::v::pgcnum [lindex [lindex $::ptted::v::mainlist 0] 4]
	set ::ptted::v::pgnum  [lindex [lindex $::ptted::v::mainlist 0] end]
	set ::ptted::v::pgcnumidx [lsearch $::ptted::v::pgcsinttn $::ptted::v::pgcnum]
	$w.mf.c activate 0
	$w.mf.c selection clear 0 end
	$w.mf.c selection set 0 0

	frame  $w.sing
	pack   $w.sing -fill x
	label  $w.sing.pttfl -text "PTT:" -width 4
	entry  $w.sing.pttfe -textvariable ::ptted::v::pttnum -width 3 -state readonly -justify right
	scale  $w.sing.pttfs -variable ::ptted::v::pttnum -from 1 -to [llength $::ptted::v::mainlist] \
			-showvalue false -orient h -command ::ptted::init_ptt_val
	label  $w.sing.pgcfl -text "PGC:"
	entry  $w.sing.pgcfe -textvariable ::ptted::v::pgcnum -width 3 -state readonly -justify right
	scale  $w.sing.pgcfs -variable ::ptted::v::pgcnumidx -from 0 -to [expr [llength $::ptted::v::pgcsinttn]-1] \
			-showvalue false -orient h -command ::ptted::init_pgc_val
	label  $w.sing.pgfl -text "PG:"
	entry  $w.sing.pgfe -textvariable ::ptted::v::pgnum -width 3 -state readonly -justify right
	scale  $w.sing.pgfs -variable ::ptted::v::pgnum -from 1 -to $::pgcs($vts,$::ptted::v::pgnum,numprograms) \
			-showvalue false -orient h -command ::ptted::init_pg_val
	grid $w.sing.pttfl -row 0 -column 0 -sticky w
	grid $w.sing.pttfe -row 0 -column 1 -sticky ew
	grid $w.sing.pttfs -row 0 -column 2 -sticky ew -padx 2
	grid $w.sing.pgcfl -row 1 -column 0 -sticky w
	grid $w.sing.pgcfe -row 1 -column 1 -sticky ew
	grid $w.sing.pgcfs -row 1 -column 2 -sticky ew -padx 2
	grid $w.sing.pgfl  -row 2 -column 0 -sticky w
	grid $w.sing.pgfe  -row 2 -column 1 -sticky ew
	grid $w.sing.pgfs  -row 2 -column 2 -sticky ew -padx 2
	grid columnconf $w.sing 2 -weight 100
	if {[llength $::ptted::v::pgcsinttn] == 1} {
		::ced::scale_state $w.sing.pgcfs disabled
	}

	frame  $w.act
	pack   $w.act -padx 2 -pady 2 -fill x
	button $w.act.b1 -text "Remove selected PTTs (Del)" -command ::ptted::removeselptts
	button $w.act.b2 -text "Decr selected PGs (-)" -command {::ptted::incdpgs -1 $::ptted::v::vts}
	button $w.act.b3 -text "Create new PTT" -command {::ptted::newptt $::ptted::v::vts}
	button $w.act.b4 -text "Incr selected PGs (+)" -command {::ptted::incdpgs  1 $::ptted::v::vts}
	button $w.act.b5 -text " Recreate sequential PTT table (One PTT per PG) " -command {::ptted::createptttable $::ptted::v::vts}
	grid $w.act.b1 -row 0 -column 0 -sticky ew -padx 2 -pady 1
	grid $w.act.b2 -row 0 -column 1 -sticky ew -padx 2 -pady 1
	grid $w.act.b3 -row 1 -column 0 -sticky ew -padx 2 -pady 1
	grid $w.act.b4 -row 1 -column 1 -sticky ew -padx 2 -pady 1
	grid $w.act.b5 -row 2 -column 0 -columnspan 2 -sticky ew -padx 2 -pady 1
	grid columnconf $w.act 0 -uniform 1 -weight 1
	grid columnconf $w.act 1 -uniform 1 -weight 1

	# buttons
	frame  $w.fb
	pack   $w.fb -padx 2 -pady 4 -fill x
	frame  $w.fb.pad
	pack   $w.fb.pad -expand 1 -fill x -side left
	button $w.fb.ok -text "OK" -width 10 -command {set ::ptted::v::done 1}
	pack   $w.fb.ok -side left -padx 3
	button $w.fb.cancel -text "Cancel" -width 10 -command {set ::ptted::v::done 0}
	pack   $w.fb.cancel -side left -padx 2
	if {$::config(platform) == "mac"} {
		pack [frame $w.fb.macpad] -side left -padx 6
	}

	bind $w.mf.c <ButtonRelease-1> {::ptted::setselection}
	bind $w <Up>   {::ptted::selectentry -1}
	bind $w <Down> {::ptted::selectentry  1}
	bind $w <Home> {::ptted::selectentry -1000}
	bind $w <End>  {::ptted::selectentry  1000}
	bind $w <Shift-Up>   {::ptted::addentry -1}
	bind $w <Shift-Down> {::ptted::addentry  1}
	bind $w <Shift-Home> {::ptted::addentry -1000}
	bind $w <Shift-End>  {::ptted::addentry  1000}
	bind $w <+>     {::ptted::incdpgs  1 $::ptted::v::vts}
	bind $w <minus> {::ptted::incdpgs -1 $::ptted::v::vts}
	bind $w <Delete> ::ptted::removeselptts


	# window geometry
	update
	set ww [winfo reqwidth $w]
	set wh [winfo reqheight $w]
	set wx [expr {[winfo rootx .]+(([winfo width  .]-$ww)/2)}]
	set wy [expr {[winfo rooty .]+(([winfo height .]-$wh)/2)-25}]
	if {$wx < 10} {set wx 10}
	if {$wy < 22} {set wy 22}
	$w.mf.c configure -height 3
	update
	wm minsize $w [winfo reqwidth $w] [winfo reqheight $w]
	wm geometry $w ${ww}x$wh+$wx+$wy
	wm deiconify $w
	focus $w
	grab set $w

	set ::ptted::v::done -1
	tkwait variable ::ptted::v::done
	grab release $w
	destroy $w

	if {$::ptted::v::done} {
		foreach i $::ptted::v::pgcsinttn {
			set ::pgcs($vts,$i,pttlist) {}
		}
		foreach i $::ptted::v::mainlist {
			set ptt [lindex $i 1]
			set pgc [lindex $i 4]
			set pg  [lindex $i end]
			lappend ::pgcs($vts,$pgc,pttlist) [list $pg $ptt $ttn]
		}
	}
	namespace delete ::ptted::v
}

proc setselection {} {
	set w .ptted
	set sel [$w.mf.c curselection]
	set sel0 [lindex $sel 0]
	set ::ptted::v::pttnum [expr $sel0+1]
	set ::ptted::v::pgcnum [lindex [lindex $::ptted::v::mainlist $sel0] 4]
	set ::ptted::v::pgnum  [lindex [lindex $::ptted::v::mainlist $sel0] end]
	set ::ptted::v::pgcnumidx [lsearch $::ptted::v::pgcsinttn $::ptted::v::pgcnum]
	.ptted.sing.pgfs configure -to $::pgcs($::ptted::v::vts,$::ptted::v::pgcnum,numprograms)
}

proc init_ptt_val {{val -1}} {
	if {$val == -1} {set val $::ptted::v::pttnum}
	set ::ptted::v::pgcnum [lindex [lindex $::ptted::v::mainlist [expr $val-1]] 4]
	set ::ptted::v::pgnum  [lindex [lindex $::ptted::v::mainlist [expr $val-1]] end]
	set ::ptted::v::pgcnumidx [lsearch $::ptted::v::pgcsinttn $::ptted::v::pgcnum]
	.ptted.sing.pgfs configure -to $::pgcs($::ptted::v::vts,$::ptted::v::pgcnum,numprograms)
	set idx [expr $val -1]
	.ptted.mf.c activate $idx
	.ptted.mf.c selection clear 0 end
	.ptted.mf.c selection set $idx $idx
	.ptted.mf.c see $idx
}
proc init_pgc_val {{val -1}} {
	if {$val == -1} {set val $::ptted::v::pgcnumidx}
	set ::ptted::v::pgcnum [lindex $::ptted::v::pgcsinttn $val]
	foreach i [.ptted.mf.c curselection] {
		lset ::ptted::v::mainlist $i [format $::ptted::v::formatstring [expr $i+1] $::ptted::v::pgcnum [lindex [lindex $::ptted::v::mainlist $i] end]]
	}
	.ptted.sing.pgfs configure -to $::pgcs($::ptted::v::vts,$::ptted::v::pgcnum,numprograms)
}
proc init_pg_val {{val -1}} {
	if {$val == -1} {set val $::ptted::v::pgnum}
	foreach i [.ptted.mf.c curselection] {
		lset ::ptted::v::mainlist $i [format $::ptted::v::formatstring [expr $i+1] [lindex [lindex $::ptted::v::mainlist $i] 4] $val]
	}
}
proc incdpgs {amount vts} {
	set err false
	foreach i [.ptted.mf.c curselection] {
		set pgc [lindex [lindex $::ptted::v::mainlist $i] 4]
		set pg  [expr [lindex [lindex $::ptted::v::mainlist $i] end]+$amount]
		if {$pg < 1 || $pg > $::pgcs($vts,$pgc,numprograms)} {
			set err true
		} else {
			lset ::ptted::v::mainlist $i [format $::ptted::v::formatstring [expr $i+1] $pgc $pg]
		}
	}
	if {$err} bell
}
proc removeselptts {} {
	set sel [.ptted.mf.c curselection]
	if {[llength $sel] >= [llength $::ptted::v::mainlist]} {
		set sel [lreplace $sel 0 0]
		bell
	}
	set nextsel [expr [lindex $sel end]-[llength $sel]+1]
	for {set i [expr [llength $sel]-1]} {$i >= 0} {incr i -1} {
		set j [lindex $sel $i]
		set ::ptted::v::mainlist [lreplace $::ptted::v::mainlist $j $j]
	}
	if {$nextsel >= [llength $::ptted::v::mainlist]} {
		set nextsel [expr [llength $::ptted::v::mainlist]-1]
	}

	for {set i 0} {$i < [llength $::ptted::v::mainlist]} {incr i} {
		lset ::ptted::v::mainlist $i [format $::ptted::v::formatstring [expr $i+1] \
				[lindex [lindex $::ptted::v::mainlist $i] 4] \
				[lindex [lindex $::ptted::v::mainlist $i] end]]
	}
	.ptted.mf.c activate $nextsel
	.ptted.mf.c selection clear 0 end
	.ptted.mf.c selection set $nextsel
	.ptted.mf.c see $nextsel
	.ptted.sing.pttfs configure -to [llength $::ptted::v::mainlist]
	set ::ptted::v::pttnum [expr $nextsel+1]
	setselection
}
proc newptt {vts} {
	set ptt [llength $::ptted::v::mainlist]
	set pgc [lindex [lindex $::ptted::v::mainlist end] 4]
	set pg  [expr [lindex [lindex $::ptted::v::mainlist end] end]+1]
	if {$pg > $::pgcs($vts,$pgc,numprograms) && \
			$pgc != [lindex $::ptted::v::pgcsinttn end]} {
		set pgc [lindex $::ptted::v::pgcsinttn [expr [lsearch $::ptted::v::pgcsinttn $pgc]+1]]
		set pg 1
	}
	if {$pg > $::pgcs($vts,$pgc,numprograms)} {
		set pg $::pgcs($vts,$pgc,numprograms)
		bell
	}
	lappend ::ptted::v::mainlist [format $::ptted::v::formatstring $ptt $pgc $pg]
	.ptted.mf.c activate end
	.ptted.mf.c selection clear 0 end
	.ptted.mf.c selection set end
	.ptted.mf.c see end
	.ptted.sing.pttfs configure -to [llength $::ptted::v::mainlist]
	set ::ptted::v::pttnum $ptt
	setselection
}
proc createptttable {vts} {
	set ::ptted::v::mainlist {}
	set ptt 1
	foreach pgc $::ptted::v::pgcsinttn {
		for {set pg 1} {$pg <= $::pgcs($vts,$pgc,numprograms)} {incr pg} {
			lappend ::ptted::v::mainlist [format $::ptted::v::formatstring $ptt $pgc $pg]
			incr ptt
		}
	}
	.ptted.mf.c activate 0
	.ptted.mf.c selection clear 0 end
	.ptted.mf.c selection set 0
	.ptted.mf.c see 0
	.ptted.sing.pttfs configure -to [llength $::ptted::v::mainlist]
	set ::ptted::v::pttnum 1
	setselection
}
proc selectentry {dir} {
	set sel [.ptted.mf.c curselection]
	if {$dir < 0} {
		set newsel [expr [lindex $sel 0]+$dir]
		if {$newsel < 0} {
			set newsel 0
		}
	} else {
		set newsel [expr [lindex $sel end]+$dir]
		if {$newsel >= [llength $::ptted::v::mainlist]} {
			set newsel [expr [llength $::ptted::v::mainlist]-1]
		}
	}
	.ptted.mf.c activate $newsel
	.ptted.mf.c selection clear 0 end
	.ptted.mf.c selection set $newsel
	.ptted.mf.c see $newsel
	set ::ptted::v::pttnum [expr $newsel+1]
	setselection
}
proc addentry {dir} {
	set sel [.ptted.mf.c curselection]
	if {$dir < 0} {
		set lastsel [lindex $sel 0]
		set newsel [expr [lindex $sel 0]+$dir]
		if {$newsel < 0} {
			set newsel 0
		}
	} else {
		set newsel [lindex $sel end]
		set lastsel [expr [lindex $sel end]+$dir]
		if {$lastsel >= [llength $::ptted::v::mainlist]} {
			set lastsel [expr [llength $::ptted::v::mainlist]-1]
		}
	}
	.ptted.mf.c activate $newsel
	.ptted.mf.c selection set $newsel $lastsel
	.ptted.mf.c see $lastsel
	.ptted.mf.c see $newsel
	set ::ptted::v::pttnum [expr $newsel+1]
	setselection
}

}	;# end of namespace ::ptted
